//=====================================================================
//
//	j1Console.h - common Console Routines
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JOSHUA 1 SYSTEMS INC. OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#ifndef `$INSTANCE_NAME`_H	// include just once
#define `$INSTANCE_NAME`_H
	
#include <j1Types.h>
	
#include "`$XML_INSTANCE`.h"

#include "`$CMD_INSTANCE`.h"

//---------------------------------------------------------------------
//
//	function prototypes
//
//---------------------------------------------------------------------

void j1Console_Start();
bool j1Console_QueueRx(uint8 *bfr, uint16 bfrCnt);
bool j1Console_DeQueueTx(uint8 *bfr, uint16 bfrSize, uint16 *deQueued);
void j1Console_Tasks();
void j1Console_ProcessRx();

bool j1Console_Put(char valueBYTE);
bool j1Console_PutString(char *valueString);
bool j1Console_PutHEXBYTE(uint8 valueBYTE);
bool j1Console_PutHEXWORD(uint16 valueWORD);
bool j1Console_PutHEXDWORD(uint32 valueDWORD);
bool j1Console_PutBYTE(uint8 valueBYTE);
bool j1Console_PutWORD(uint16 valueWORD);
bool j1Console_PutDWORD(uint32 valueDWORD);
bool j1Console_PutCRLF(void);

bool j1Console_DumpCmd(j1XML_ELEMENT *cmd);

//---------------------------------------------------------------------
//
//	console command buffer variables
//
//---------------------------------------------------------------------

#define	j1CONSOLE_CMD_BFR_MAX	79

typedef enum
{
	j1CONSOLE_CMD_ESC_ESC,
	j1CONSOLE_CMD_ESC_BRKT,
} j1CONSOLE_CMD_ESC;

typedef	struct	_j1CONSOLE_CMD
{
	char				bfr[j1CONSOLE_CMD_BFR_MAX + 1];
	char				*bfrPtr;
	uint8				bfrCnt;
	bool				bfrRdy;
	bool				first;
	bool				debug;
	bool				escape;
	j1CONSOLE_CMD_ESC	escState;
} j1CONSOLE_CMD;

j1GLOBALQUALIFY	j1CONSOLE_CMD	j1ConsoleCmd;

//---------------------------------------------------------------------
//
//	console rx/tx queue
//
//---------------------------------------------------------------------

j1GLOBALQUALIFY	j1QUEUE(j1ConsoleRxQueue, `$RX_QUEUE_SIZE`, char, uint8);
j1GLOBALQUALIFY	j1QUEUE(j1ConsoleTxQueue, `$TX_QUEUE_SIZE`, char, uint16);

#endif
