//=====================================================================
//
//	j1ENC424.h - modified version of microchip driver for psoc creator
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

/*********************************************************************
 *
 * ENC424J600/624J600 Registers and Bits
 *
 *********************************************************************
 * FileName:        ENCX24J600.h
 * Dependencies:    None
 * Processor:       PIC18, PIC24F, PIC24H, dsPIC30F, dsPIC33F, PIC32
 * Compiler:        Microchip C32 v1.05 or higher
 *					Microchip C30 v3.12 or higher
 *					Microchip C18 v3.30 or higher
 *					HI-TECH PICC-18 PRO 9.63PL2 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright (C) 2002-2009 Microchip Technology Inc.  All rights
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and
 * distribute:
 * (i)  the Software when embedded on a Microchip microcontroller or
 *      digital signal controller product ("Device") which is
 *      integrated into Licensee's product; or
 * (ii) ONLY the Software driver source files ENC28J60.c, ENC28J60.h,
 *		ENCX24J600.c and ENCX24J600.h ported to a non-Microchip device
 *		used in conjunction with a Microchip ethernet controller for
 *		the sole purpose of interfacing with the ethernet controller.
 *
 * You should refer to the license agreement accompanying this
 * Software for additional information regarding your rights and
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Howard Schlunder		11/30/07	Original
 ********************************************************************/

#ifndef `$INSTANCE_NAME`_H
#define `$INSTANCE_NAME`_H
//#include "GenericTypeDefs.h"
//#include "HardwareProfile.h"
	
//---------------------------------------------------------------------
//	component parameters
//---------------------------------------------------------------------
	
#define	`$INSTANCE_NAME`_PARAM_lwIP_Driver	`$lwIP_Driver`

#include <project.h>

//---------------------------------------------------------------------
//	ENCx24J600 mapping
//---------------------------------------------------------------------

	// ENC624J600 Interface Configuration
	// Comment out `$INSTANCE_NAME`_INTERFACE_MODE if you don't have an ENC624J600 or 
	// ENC424J600.  Otherwise, choose the correct setting for the interface you 
	// are using.  Legal values are:
	//  - Commented out: No ENC424J600/624J600 present or used.  All other 
	//                   `$INSTANCE_NAME`_* macros are ignored.
	//	- 0: SPI mode using CS, SCK, SI, and SO pins
	//  - 1: 8-bit demultiplexed PSP Mode 1 with RD and WR pins
	//  - 2: *8-bit demultiplexed PSP Mode 2 with R/Wbar and EN pins
	//  - 3: *16-bit demultiplexed PSP Mode 3 with RD, WRL, and WRH pins
	//  - 4: *16-bit demultiplexed PSP Mode 4 with R/Wbar, B0SEL, and B1SEL pins
	//  - 5: 8-bit multiplexed PSP Mode 5 with RD and WR pins
	//  - 6: *8-bit multiplexed PSP Mode 6 with R/Wbar and EN pins
	//  - 9: 16-bit multiplexed PSP Mode 9 with AL, RD, WRL, and WRH pins
	//  - 10: *16-bit multiplexed PSP Mode 10 with AL, R/Wbar, B0SEL, and B1SEL 
	//        pins
	// *IMPORTANT NOTE: DO NOT USE PSP MODE 2, 4, 6, OR 10 ON EXPLORER 16! 
	// Attempting to do so will cause bus contention with the LCD module which 
	// shares the PMP.  Also, PSP Mode 3 is risky on the Explorer 16 since it 
	// can randomly cause bus contention with the 25LC256 EEPROM.

	#define `$INSTANCE_NAME`_INTERFACE_MODE			5

	// If using a parallel interface, direct RAM addressing can be used (if all 
	// addresses wires are connected), or a reduced number of pins can be used 
	// for indirect addressing.  If using an SPI interface or PSP Mode 9 or 10 
	// (multiplexed 16-bit modes), which require all address lines to always be 
	// connected, then this option is ignored. Comment out or uncomment this 
	// macro to match your hardware connections.

	#define `$INSTANCE_NAME`_PSP_USE_INDIRECT_RAM_ADDRESSING

	// ENC424J600/624J600 parallel indirect address remapping macro function.
	// This section translates SFR and RAM addresses presented to the 
	// ReadMemory() and WriteMemory() APIs in ENCX24J600.c to the actual 
	// addresses that must be presented on the parallel interface.  This macro 
	// must be modified to match your hardware if you are using an indirect PSP 
	// addressing mode (`$INSTANCE_NAME`_PSP_USE_INDIRECT_RAM_ADDRESSING is defined) and 
	// have some of your address lines tied off to Vdd.  If you are using the 
	// SPI interface, then this section can be ignored or deleted.

	#if (`$INSTANCE_NAME`_INTERFACE_MODE == 1) || (`$INSTANCE_NAME`_INTERFACE_MODE == 2) || (`$INSTANCE_NAME`_INTERFACE_MODE == 5) || (`$INSTANCE_NAME`_INTERFACE_MODE == 6) // 8-bit PSP
		#define `$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(a)		((((a)&0x0100)<<6) | ((a)&0x00FF))
	#elif (`$INSTANCE_NAME`_INTERFACE_MODE == 3) || (`$INSTANCE_NAME`_INTERFACE_MODE == 4) // 16-bit PSP
		#define `$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(a)		(a)
	#endif

	// Auto-crossover pins on Fast 100Mbps Ethernet PICtail/PICtail Plus.  If 
	// your circuit doesn't have such a feature, delete these two defines.

	//#define `$INSTANCE_NAME`_MDIX_TRIS				(TRISBbits.TRISB3)
	//#define `$INSTANCE_NAME`_MDIX_IO				(LATBbits.LATB3)

	// ENC624J600 I/O control and status pins
	// If a pin is not required for your selected `$INSTANCE_NAME`_INTERFACE_MODE 
	// interface selection (ex: WRH/B1SEL for PSP modes 1, 2, 5, and 6), then 
	// you can ignore, delete, or put anything for the pin definition.  Also, 
	// the INT and POR pins are entirely optional.  If not connected, comment 
	// them out.

	//#define `$INSTANCE_NAME`_INT_TRIS				(TRISEbits.TRISE9)		// INT signal is optional and currently unused in the Microchip TCP/IP Stack.  Leave this pin disconnected and comment out this pin definition if you don't want it.
	//#define `$INSTANCE_NAME`_INT_IO					(PORTEbits.RE9)

	#if (`$INSTANCE_NAME`_INTERFACE_MODE >= 1)	// Parallel mode
		// PSP control signal pinout
		//#define `$INSTANCE_NAME`_CS_TRIS				(TRISAbits.TRISA5)	// CS is optional in PSP mode.  If you are not sharing the parallel bus with another device, tie CS to Vdd and comment out this pin definition.
		//#define `$INSTANCE_NAME`_CS_IO					(LATAbits.LATA5)
		//#define `$INSTANCE_NAME`_POR_TRIS				(TRISCbits.TRISC1)	// POR signal is optional.  If your application doesn't have a power disconnect feature, comment out this pin definition.
		//#define `$INSTANCE_NAME`_POR_IO					(LATCbits.LATC1)
		#define `$INSTANCE_NAME`_SO_WR_B0SEL_EN_TRIS		(TRISDbits.TRISD8)
		#define `$INSTANCE_NAME`_SO_WR_B0SEL_EN_IO		(LATDbits.LATD8)
		#define `$INSTANCE_NAME`_SI_RD_RW_TRIS			(TRISDbits.TRISD9)
		#define `$INSTANCE_NAME`_SI_RD_RW_IO				(LATDbits.LATD9)
		#define `$INSTANCE_NAME`_SCK_AL_TRIS				(TRISDbits.TRISD10)
		#define `$INSTANCE_NAME`_SCK_AL_IO				(LATDbits.LATD10)
	#else	
		// SPI pinout
		#define `$INSTANCE_NAME`_CS_TRIS					(TRISBbits.TRISB12)	// CS is mandatory when using the SPI interface
		#define `$INSTANCE_NAME`_CS_IO					(LATBbits.LATB12)
		//#define `$INSTANCE_NAME`_POR_TRIS				(TRISDbits.TRISD15)	// POR signal is optional.  If your application doesn't have a power disconnect feature, comment out this pin definition.
		//#define `$INSTANCE_NAME`_POR_IO					(LATDbits.LATD15)
		#define `$INSTANCE_NAME`_SO_WR_B0SEL_EN_TRIS		(TRISFbits.TRISF2)	// SO is ENCX24J600 Serial Out, which needs to connect to the PIC SDI pin for SPI mode
		#define `$INSTANCE_NAME`_SO_WR_B0SEL_EN_IO		(PORTFbits.RF2)
		#define `$INSTANCE_NAME`_SI_RD_RW_TRIS			(TRISFbits.TRISF3)	// SI is ENCX24J600 Serial In, which needs to connect to the PIC SDO pin for SPI mode
		#define `$INSTANCE_NAME`_SI_RD_RW_IO				(LATFbits.LATF3)
		#define `$INSTANCE_NAME`_SCK_AL_TRIS				(TRISFbits.TRISF6)
	#endif

	// ENC624J600 Bit Bang PSP I/O macros and pin configuration for address and 
	// data.  If using the SPI interface (`$INSTANCE_NAME`_INTERFACE_MODE is 0) then this 
	// section is not used and can be ignored or deleted.  If using the PIC PMP
	// hardware module (if present), then `$INSTANCE_NAME`_BIT_BANG_PMP must be commented 
	// out and the remaining definitions will be ignored/can be deleted.  
	// Otherwise, if you are using a parallel interface mode, but do not have a 
	// PMP (or want to interface using different pins), define 
	// `$INSTANCE_NAME`_BIT_BANG_PMP and properly configure the applicable macros.
	
	#define `$INSTANCE_NAME`_BIT_BANG_PMP
	
	#if defined(`$INSTANCE_NAME`_BIT_BANG_PMP)
		#if `$INSTANCE_NAME`_INTERFACE_MODE == 1 || `$INSTANCE_NAME`_INTERFACE_MODE == 2	// Dumultiplexed 8-bit address/data modes
			// SPI2 CANNOT BE ENABLED WHEN ACCESSING THE ENC624J600 FOR THESE TWO MODES AS THE PINS OVERLAP WITH ADDRESS LINES.
			#if defined(`$INSTANCE_NAME`_PSP_USE_INDIRECT_RAM_ADDRESSING)	// Only ENC624J600 address pins A0-A8 connected (A9-A14 tied to Vdd)
				#define `$INSTANCE_NAME`_INIT_PSP_BIT_BANG()	do{((volatile BYTE*)&AD1PCFGH)[1] = 0xFF; ((volatile BYTE*)&AD1PCFGL)[1] |= 0xC0;}while(0)	// Disable AN24-AN31 and AN14-AN15 analog inputs on RE0-RE7 and RB14-RB15 pins (ENCX24J600 AD0-AD7, A1-A0)
				#define `$INSTANCE_NAME`_SET_ADDR_TRIS_OUT()	do{TRISB &= 0x3FFF; TRISG &= 0xFC3F; TRISA &= 0xF9FF; TRISDbits.TRISD11 = 0;}while(0)
				#define `$INSTANCE_NAME`_SET_ADDR_IO(a)		do{WORD _SetMacro = (a); LATBbits.LATB15 = 0; LATBbits.LATB14 = 0; LATG &= 0xFC3F; LATAbits.LATA10 = 0; LATAbits.LATA9 = 0; LATDbits.LATD11 = 0; if(_SetMacro & 0x0001) LATBbits.LATB15 = 1; if(_SetMacro & 0x0002) LATBbits.LATB14 = 1; if(_SetMacro & 0x0004) LATGbits.LATG9 = 1; if(_SetMacro & 0x0008) LATGbits.LATG8 = 1; if(_SetMacro & 0x0010) LATGbits.LATG7 = 1; if(_SetMacro & 0x0020) LATGbits.LATG6 = 1; if(_SetMacro & 0x0040) LATAbits.LATA10 = 1; if(_SetMacro & 0x0080) LATAbits.LATA9 = 1; if(_SetMacro & 0x4000) LATDbits.LATD11 = 1;}while(0)
				#define `$INSTANCE_NAME`_SET_AD_TRIS_IN()		(((volatile BYTE*)&TRISE)[0] = 0xFF)
				#define `$INSTANCE_NAME`_SET_AD_TRIS_OUT()	(((volatile BYTE*)&TRISE)[0] = 0x00)
				#define `$INSTANCE_NAME`_GET_AD_IO()			(((volatile BYTE*)&PORTE)[0])
				#define `$INSTANCE_NAME`_SET_AD_IO(data)		(((volatile BYTE*)&LATE)[0] = (data))
			#else 	// All ENC624J600 address pins A0-A14 connected
				#define `$INSTANCE_NAME`_INIT_PSP_BIT_BANG()	do{((volatile BYTE*)&AD1PCFGH)[1] = 0xFF; ((volatile BYTE*)&AD1PCFGL)[1] |= 0xFC;}while(0)	// Disable AN24-AN31 and AN10-AN15 analog inputs on RE0-RE7 and RB10-RB15 pins (ENCX24J600 AD0-AD7, A1-A0, A13-A10)
				#define `$INSTANCE_NAME`_SET_ADDR_TRIS_OUT()	do{TRISB &= 0x03FF; TRISG &= 0xFC3F; TRISA &= 0xF9FF; TRISF &= 0xFFCF; TRISDbits.TRISD11 = 0;}while(0)
				#define `$INSTANCE_NAME`_SET_ADDR_IO(a)		do{WORD _SetMacro = (a); LATB &= 0x03FF; LATG &= 0xFC3F; LATAbits.LATA10 = 0; LATAbits.LATA9 = 0; LATFbits.LATF5 = 0; LATFbits.LATF4 = 0; LATDbits.LATD11 = 0; if(_SetMacro & 0x0001) LATBbits.LATB15 = 1; if(_SetMacro & 0x0002) LATBbits.LATB14 = 1; if(_SetMacro & 0x0004) LATGbits.LATG9 = 1; if(_SetMacro & 0x0008) LATGbits.LATG8 = 1; if(_SetMacro & 0x0010) LATGbits.LATG7 = 1; if(_SetMacro & 0x0020) LATGbits.LATG6 = 1; if(_SetMacro & 0x0040) LATAbits.LATA10 = 1; if(_SetMacro & 0x0080) LATAbits.LATA9 = 1; if(_SetMacro & 0x0100) LATFbits.LATF5 = 1; if(_SetMacro & 0x0200) LATFbits.LATF4 = 1; if(_SetMacro & 0x0400) LATBbits.LATB13 = 1; if(_SetMacro & 0x0800) LATBbits.LATB12 = 1; if(_SetMacro & 0x1000) LATBbits.LATB11 = 1; if(_SetMacro & 0x2000) LATBbits.LATB10 = 1; if(_SetMacro & 0x4000) LATDbits.LATD11 = 1;}while(0)
				#define `$INSTANCE_NAME`_SET_AD_TRIS_IN()		(((volatile BYTE*)&TRISE)[0] = 0xFF)
				#define `$INSTANCE_NAME`_SET_AD_TRIS_OUT()	(((volatile BYTE*)&TRISE)[0] = 0x00)
				#define `$INSTANCE_NAME`_GET_AD_IO()			(((volatile BYTE*)&PORTE)[0])
				#define `$INSTANCE_NAME`_SET_AD_IO(data)		(((volatile BYTE*)&LATE)[0] = (data))
			#endif
		#elif `$INSTANCE_NAME`_INTERFACE_MODE == 3 || `$INSTANCE_NAME`_INTERFACE_MODE == 4	// Dumultiplexed 16-bit address/data modes
			#if defined(`$INSTANCE_NAME`_PSP_USE_INDIRECT_RAM_ADDRESSING)	// Only ENC624J600 address pins A0-A7 connected (A8-A13 tied to Vdd)
				#define `$INSTANCE_NAME`_INIT_PSP_BIT_BANG()	do{AD1PCFGH = 0xFFFF; AD1PCFGL = 0xFFFF; AD2PCFGL = 0xFFFF;}while(0)
				#define `$INSTANCE_NAME`_SET_ADDR_TRIS_OUT()	do{TRISB &= 0x3FFF; TRISG &= 0xFC3F; TRISA &= 0xF9FF;}while(0)
				#define `$INSTANCE_NAME`_SET_ADDR_IO(a)		do{WORD _SetMacro = (a); LATBbits.LATB15 = 0; LATBbits.LATB14 = 0; LATG &= 0xFC3F; LATAbits.LATA10 = 0; LATAbits.LATA9 = 0; if(_SetMacro & 0x0001) LATBbits.LATB15 = 1; if(_SetMacro & 0x0002) LATBbits.LATB14 = 1; if(_SetMacro & 0x0004) LATGbits.LATG9 = 1; if(_SetMacro & 0x0008) LATGbits.LATG8 = 1; if(_SetMacro & 0x0010) LATGbits.LATG7 = 1; if(_SetMacro & 0x0020) LATGbits.LATG6 = 1; if(_SetMacro & 0x0040) LATAbits.LATA10 = 1; if(_SetMacro & 0x0080) LATAbits.LATA9 = 1;}while(0)
				#define `$INSTANCE_NAME`_WRH_B1SEL_TRIS		`$INSTANCE_NAME`_SO_WR_B0SEL_EN_TRIS
				#define `$INSTANCE_NAME`_WRH_B1SEL_IO			`$INSTANCE_NAME`_SO_WR_B0SEL_EN_IO
				#define `$INSTANCE_NAME`_SET_AD_TRIS_IN()		do{((volatile BYTE*)&TRISE)[0] = 0xFF; TRISG |= 0x0003; TRISF |= 0x0003; TRISD |= 0x30C0;}while(0)
				#define `$INSTANCE_NAME`_SET_AD_TRIS_OUT()	do{((volatile BYTE*)&TRISE)[0] = 0x00; TRISG &= 0xFFFC; TRISF |= 0xFFFC; TRISD |= 0xCF3F;}while(0)
				#define `$INSTANCE_NAME`_GET_AD_IOL()			(((volatile BYTE*)&PORTE)[0])
				#if defined(__ENCX24J600_C)
				    static inline __attribute__((__always_inline__)) BYTE `$INSTANCE_NAME`_GET_AD_IOH(void)
				    {
					    BYTE_VAL ret = {0}; if(PORTGbits.RG0) ret.bits.b0 = 1; if(PORTGbits.RG1) ret.bits.b1 = 1; if(PORTFbits.RF1) ret.bits.b2 = 1; if(PORTFbits.RF0) ret.bits.b3 = 1; if(PORTDbits.RD12) ret.bits.b4 = 1; if(PORTDbits.RD13) ret.bits.b5 = 1; if(PORTDbits.RD6) ret.bits.b6 = 1; if(PORTDbits.RD7) ret.bits.b7 = 1; return ret.Val;
				    }
				#endif
				#define `$INSTANCE_NAME`_GET_AD_IO()			((((volatile BYTE*)&PORTE)[0]) | `$INSTANCE_NAME`_GET_AD_IOH();)
				#define `$INSTANCE_NAME`_SET_AD_IO(data)		do{WORD _SetMacro = (data); ((volatile BYTE*)&LATE)[0] = (BYTE)_SetMacro; LATG &= 0xFFFC; LATF |= 0xFFFC; LATD |= 0xCF3F; if(_SetMacro & 0x0100) LATGbits.LATG0 = 1; if(_SetMacro & 0x0200) LATGbits.LATG1 = 1; if(_SetMacro & 0x0400) LATFbits.LATF1 = 1; if(_SetMacro & 0x0800) LATFbits.LATF0 = 1; if(_SetMacro & 0x1000) LATDbits.LATD12 = 1; if(_SetMacro & 0x2000) LATDbits.LATD13 = 1; if (_SetMacro & 0x4000) LATDbits.LATD6 = 1; if(_SetMacro & 0x8000) LATDbits.LATD7 = 1;}while(0)
			#else 	// All ENC624J600 address pins A0-A13 connected
				#define `$INSTANCE_NAME`_INIT_PSP_BIT_BANG()	do{AD1PCFGH = 0xFFFF; AD1PCFGL = 0xFFFF; AD2PCFGL = 0xFFFF;}while(0)
				#define `$INSTANCE_NAME`_SET_ADDR_TRIS_OUT()	do{TRISE &= 0xFF00; TRISG &= 0x8FF0;}while(0)
				#define `$INSTANCE_NAME`_SET_ADDR_IO(a)		do{WORD _wAddrSetMacro = (a); LATBbits.LATB15 = ((((BYTE*)&_wAddrSetMacro)[0]) & 0x1) == 0x1; LATBbits.LATB14 = ((((BYTE*)&_wAddrSetMacro)[0]) & 0x2) == 0x2; LATGbits.LATG9 = ((((BYTE*)&_wAddrSetMacro)[0]) & 0x4) == 0x4; LATG8bits.LATG8 = ((((BYTE*)&_wAddrSetMacro)[0]) & 0x8) == 0x8; LATGbits.LATG7 = ((((BYTE*)&_wAddrSetMacro)[0]) & 0x10) == 0x10; LATGbits.LATG6 = ((((BYTE*)&_wAddrSetMacro)[0]) & 0x20) == 0x20; LATAbits.LATA10 = ((((BYTE*)&_wAddrSetMacro)[0]) & 0x40) == 0x40; LATAbits.LATA9 = ((((BYTE*)&_wAddrSetMacro)[0]) & 0x80) == 0x80; LATDbits.LATD11 = ((((BYTE*)&_wAddrSetMacro)[1]) & 0x40) == 0x40;}while(0)
				#define `$INSTANCE_NAME`_WRH_B1SEL_TRIS		`$INSTANCE_NAME`_SO_WR_B0SEL_EN_TRIS
				#define `$INSTANCE_NAME`_WRH_B1SEL_IO			`$INSTANCE_NAME`_SO_WR_B0SEL_EN_IO
				#define `$INSTANCE_NAME`_SET_AD_TRIS_IN		do{TRISD = 0xFFFF;}while(0)
				#define `$INSTANCE_NAME`_SET_AD_TRIS_OUT		do{TRISD = 0x0000;}while(0)
				#define `$INSTANCE_NAME`_GET_AD_IO()			(((volatile WORD*)&PORTD)[0])
				#define `$INSTANCE_NAME`_SET_AD_IO(data)		(((volatile WORD*)&LATD)[0] = (data))
			#endif
		#elif `$INSTANCE_NAME`_INTERFACE_MODE == 5 || `$INSTANCE_NAME`_INTERFACE_MODE == 6	// Mutliplexed 8-bit address/data modes
			#if defined(`$INSTANCE_NAME`_PSP_USE_INDIRECT_RAM_ADDRESSING)	// Only ENCX24J600 address pins AD0-AD8 connected (AD9-AD14 tied to Vdd)
				#define `$INSTANCE_NAME`_INIT_PSP_BIT_BANG()	do{}while(0)
				#define `$INSTANCE_NAME`_SET_AD_IN()			do{`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control &= ~0x80;}while(0)
				#define `$INSTANCE_NAME`_SET_AD_OUT()			do{`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control |=  0x80;}while(0)
				#define `$INSTANCE_NAME`_SET_AD_DATA(data)		(`$INSTANCE_NAME`_AD0_7_Out_Control = (BYTE)(data))
				#define `$INSTANCE_NAME`_GET_AD_DATA()			(`$INSTANCE_NAME`_AD0_7_In_Status)
				#define `$INSTANCE_NAME`_SET_AD_ADDR(addr)		do{`$INSTANCE_NAME`_AD0_7_Out_Control = (BYTE)(addr);`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control &= ~0x01; if ((WORD)(addr) & 0x4000)`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control |= 0x01;}while(0)
				#define `$INSTANCE_NAME`_SET_WR()				do{`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control |=  0x02;}while(0)
				#define `$INSTANCE_NAME`_CLR_WR()				do{`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control &= ~0x02;}while(0)
				#define `$INSTANCE_NAME`_SET_RD()				do{`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control |=  0x04;}while(0)
				#define `$INSTANCE_NAME`_CLR_RD()				do{`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control &= ~0x04;}while(0)
				#define `$INSTANCE_NAME`_SET_AL()				do{`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control |=  0x08;}while(0)
				#define `$INSTANCE_NAME`_CLR_AL()				do{`$INSTANCE_NAME`_AD8_WR_RD_AL_DIR_Control &= ~0x08;}while(0)
			#else 	// All ENCX24J600 address pins AD0-AD14 connected
				// This pinout is bad for doing 8-bit bit-bang operations with all address lines.  The Fast 100Mbps Ethernet PICtail Plus hardware is wired for PMP hardware support, which requires this pinout.  However, if you are designing a custom board, you can simplify these read/write operations dramatically if you wire things more logically by putting all 15 I/O pins, in order, on PORTB or PORTD.  Such a change would enhance performance.
				// UART2 CANNOT BE USED OR ENABLED FOR THESE TWO MODES AS THE PINS OVERLAP WITH ADDRESS LINES.
				#define `$INSTANCE_NAME`_INIT_PSP_BIT_BANG()	do{AD1PCFGL |= 0x3C00; ((volatile BYTE*)&AD1PCFGH)[1] = 0xFF;}while(0)	// Disable AN10-AN13 and AN24-AN31 analog inputs on RB10-RB13 and RE0-RE7 pins (ENCX24J600 AD13-AD10 and AD0-AD7)
				#define `$INSTANCE_NAME`_SET_AD_TRIS_IN		do{((volatile BYTE*)&TRISE)[0] = 0xFF; TRISF |= 0x0030; TRISB |= 0x3C00; TRISD |= 0x0C00;}while(0)
				#define `$INSTANCE_NAME`_SET_AD_TRIS_OUT		do{((volatile BYTE*)&TRISE)[0] = 0x00; TRISF &= 0xFFCF; TRISB &= 0xC3FF; TRISD &= 0xF3FF;}while(0)
				#define `$INSTANCE_NAME`_GET_AD_IO()			(((volatile BYTE*)&PORTE)[0])
				#define `$INSTANCE_NAME`_SET_AD_IO(data)		do{WORD _wDataSetMacro = (data); ((volatile BYTE*)&LATE)[0] = ((BYTE*)&_wDataSetMacro)[0]; LATFbits.LATF5 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x1) == 0x1; LATFbits.LATF4 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x2) == 0x2; LATBbits.LATB13 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x4) == 0x4; LATBbits.LATB12 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x8) == 0x8; LATBbits.LATB11 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x10) == 0x10; LATBbits.LATB10 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x20) == 0x20; LATDbits.LATD10 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x40) == 0x40; LATDbits.LATD11 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x80) == 0x80;}while(0)
				#define `$INSTANCE_NAME`_SET_AD_IOL(data)		(((volatile BYTE*)&LATE)[0] = (BYTE)(data))
			#endif
		#elif `$INSTANCE_NAME`_INTERFACE_MODE == 9 || `$INSTANCE_NAME`_INTERFACE_MODE == 10	// Mutliplexed 16-bit address/data modes
			// All ENC624J600 adddress/data pins AD0-AD15 connected (required for 16-bit data, so there is no differentiation for indirect versus direct addressing mode)
			// This pinout is awful for doing 16-bit bit-bang operations.  The Fast 100Mbps Ethernet PICtail Plus hardware is wired for PMP hardware support, which requires this pinout.  However, if you are designing a custom board, you can simplify these read/write operations dramatically if you wire things more logically by putting all 16 I/O pins, in order, on PORTB or PORTD.  Such a change would enhance performance.
			#define `$INSTANCE_NAME`_INIT_PSP_BIT_BANG()	do{((volatile BYTE*)&AD1PCFGH)[1] = 0xFF;}while(0)	// Disable AN24-AN31 analog inputs on RE0-RE7 pins (ENCX24J600 AD0-AD7)
			#define `$INSTANCE_NAME`_SET_AD_TRIS_IN		do{((volatile BYTE*)&TRISE)[0] = 0xFF; TRISG |= 0x0003; TRISF |= 0x0003; TRISD |= 0x30C0;}while(0)
			#define `$INSTANCE_NAME`_SET_AD_TRIS_OUT		do{((volatile BYTE*)&TRISE)[0] = 0x00; TRISG &= 0xFFFC; TRISF &= 0xFFFC; TRISD &= 0xCF3F;}while(0)
			#define `$INSTANCE_NAME`_GET_AD_IO()			(((volatile BYTE*)&PORTE)[0] | (PORTGbits.RG0<<8) | (PORTGbits.RG1<<9) | (PORTFbits.RF1<<10) | (PORTFbits.RF0<<11) | (PORTDbits.RD12<<12) | (PORTDbits.RD13<<13) | (PORTDbits.RD6<<14) | (PORTDbits.RD7<<15)
			#define `$INSTANCE_NAME`_SET_AD_IO(data)		do{WORD _wDataSetMacro = (data); ((volatile BYTE*)&LATE)[0] = ((BYTE*)&_wDataSetMacro)[0]; LATGbits.LATG0 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x1) == 0x1; LATGbits.LATG1 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x2) == 0x2; LATFbits.LATF1 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x4) == 0x4; LATFbits.LATF0 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x8) == 0x8; LATDbits.LATD12 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x10) == 0x10; LATDbits.LATD13 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x20) == 0x20; LATDbits.LATD6 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x40) == 0x40; LATDbits.LATD7 = ((((BYTE*)&_wDataSetMacro)[1]) & 0x80) == 0x80;}while(0)
		#endif
	#endif

	// ENC624J600 SPI SFR register selection (controls which SPI peripheral to 
	// use on PICs with multiple SPI peripherals).  If a parallel interface is 
	// used (`$INSTANCE_NAME`_INTERFACE_MODE is >= 1), then the SPI is not used and this 
	// section can be ignored or deleted.
	
	#if defined(__C30__)	// PIC24F, PIC24H, dsPIC30, dsPIC33
		#define `$INSTANCE_NAME`_ISR_ENABLE		(IEC1bits.INT2IE)
		#define `$INSTANCE_NAME`_ISR_FLAG			(IFS1bits.INT2IF)	
		#define `$INSTANCE_NAME`_ISR_POLARITY		(INTCON2bits.INT2EP)
		#define `$INSTANCE_NAME`_ISR_PRIORITY		(IPC7bits.INT2IP)
		#define `$INSTANCE_NAME`_SPI_ENABLE		(`$INSTANCE_NAME`_SPISTATbits.SPIEN)
		#define `$INSTANCE_NAME`_SPI_IF			(IFS0bits.SPI1IF)
		#define `$INSTANCE_NAME`_SSPBUF			(SPI1BUF)
		#define `$INSTANCE_NAME`_SPISTAT			(SPI1STAT)
		#define `$INSTANCE_NAME`_SPISTATbits		(SPI1STATbits)
		#define `$INSTANCE_NAME`_SPICON1			(SPI1CON1)
		#define `$INSTANCE_NAME`_SPICON1bits		(SPI1CON1bits)
		#define `$INSTANCE_NAME`_SPICON2			(SPI1CON2)
		
	#else					// PIC32MX
	
//		#define `$INSTANCE_NAME`_ISR_ENABLE		(IEC0bits.INT2IE)
//		#define `$INSTANCE_NAME`_ISR_FLAG			(IFS0bits.INT2IF)
//		#define `$INSTANCE_NAME`_ISR_POLARITY		(INTCONbits.INT2EP)	
//		#define `$INSTANCE_NAME`_ISR_PRIORITY		(IPC2bits.INT2IP)	
//		#define `$INSTANCE_NAME`_SPI_ENABLE		(`$INSTANCE_NAME`_SPICON1bits.ON)
//		#define `$INSTANCE_NAME`_SPI_IF			(IFS0bits.SPI1RXIF)
//		#define `$INSTANCE_NAME`_SSPBUF			(SPI1BUF)
//		#define `$INSTANCE_NAME`_SPICON1			(SPI1CON)
//		#define `$INSTANCE_NAME`_SPISTATbits		(SPI1STATbits)
//		#define `$INSTANCE_NAME`_SPICON1bits		(SPI1CONbits)
//		#define `$INSTANCE_NAME`_SPIBRG			(SPI1BRG)
		
	#endif
	
// Crypto memory addresses.  These are accessible by the DMA only and therefore 
// have the same addresses no matter what MCU interface is being used (SPI, 
// 8-bit PSP, or 16-bit PSP)
#define `$INSTANCE_NAME`_MODEX_Y			(0x7880u)
#define `$INSTANCE_NAME`_MODEX_E			(0x7800u)
#define `$INSTANCE_NAME`_MODEX_X			(0x7880u)
#define `$INSTANCE_NAME`_MODEX_M			(0x7900u)
#define `$INSTANCE_NAME`_HASH_DATA_IN		(0x7A00u)
#define `$INSTANCE_NAME`_HASH_IV_IN		(0x7A40u)
#define `$INSTANCE_NAME`_HASH_LEN_IN		(0x7A54u)
#define `$INSTANCE_NAME`_HASH_DIGEST_OUT	(0x7A70u)
#define `$INSTANCE_NAME`_HASH_LEN_OUT		(0x7A84u)
#define `$INSTANCE_NAME`_HASH_BASE_ADDR	(0x7A00u)
#define `$INSTANCE_NAME`_AES_KEY			(0x7C00u)
#define `$INSTANCE_NAME`_AES_TEXTA		(0x7C20u)
#define `$INSTANCE_NAME`_AES_TEXTB		(0x7C30u)
#define `$INSTANCE_NAME`_AES_XOROUT		(0x7C40u)


#if (`$INSTANCE_NAME`_INTERFACE_MODE >= 1)	// Parallel mode
	#define `$INSTANCE_NAME`_SET_OFFSET			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x0100u)
	#define `$INSTANCE_NAME`_CLR_OFFSET			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x0180u)
	
#else	// SPI mode
	////////////////////////////////////////////////////
	// ENC424J600/624J600 SPI Opcodes				  //
	////////////////////////////////////////////////////
	#define `$INSTANCE_NAME`_RCR 		(0x0u<<5)// Read Control Register
	#define `$INSTANCE_NAME`_WCR			(0x2u<<5)// Write Control Register
	#define `$INSTANCE_NAME`_RCRU		(0x20u)	// Read Control Register Unbanked
	#define `$INSTANCE_NAME`_WCRU		(0x22u)	// Write Control Register Unbanked
	#define `$INSTANCE_NAME`_BFS			(0x4u<<5)// Bit Field Set
	#define `$INSTANCE_NAME`_BFSU		(0x24u)	// Bit Field Set Unbanked
	#define `$INSTANCE_NAME`_BFC			(0x5u<<5)// Bit Field Clear
	#define `$INSTANCE_NAME`_BFCU		(0x26u)	// Bit Field Clear Unbanked
	#define `$INSTANCE_NAME`_RBMGP		(0x28u)	// Read Buffer Memory General Purpose
	#define `$INSTANCE_NAME`_WBMGP		(0x2Au)	// Write Buffer Memory General Purpose
	#define `$INSTANCE_NAME`_RBMRX		(0x2Cu)	// Read Buffer Memory RX
	#define `$INSTANCE_NAME`_WBMRX		(0x2Eu)	// Write Buffer Memory RX
	#define `$INSTANCE_NAME`_RBMUDA		(0x30u)	// Read Buffer Memory User Defined Area
	#define `$INSTANCE_NAME`_WBMUDA		(0x32u)	// Write Buffer Memory User Defined Area
	#define `$INSTANCE_NAME`_WGPRDPT		(0x60u)	// Write General Purpose Read Pointer
	#define `$INSTANCE_NAME`_RGPRDPT		(0x62u)	// Read General Purpose Read Pointer
	#define `$INSTANCE_NAME`_WRXRDPT		(0x64u)	// Write RX Read Pointer
	#define `$INSTANCE_NAME`_RRXRDPT		(0x66u)	// Read RX Read Pointer
	#define `$INSTANCE_NAME`_WUDARDPT	(0x68u)	// Write User Defined Area Read Pointer
	#define `$INSTANCE_NAME`_RUDARDPT	(0x6Au)	// Read User Defined Area Read Pointer
	#define `$INSTANCE_NAME`_WGPWRPT		(0x6Cu)	// Write General Purpose Write Pointer
	#define `$INSTANCE_NAME`_RGPWRPT		(0x6Eu)	// Read General Purpose Write Pointer
	#define `$INSTANCE_NAME`_WRXWRPT		(0x70u)	// Write RX Write Pointer
	#define `$INSTANCE_NAME`_RRXWRPT		(0x72u)	// Read RX Write Pointer
	#define `$INSTANCE_NAME`_WUDAWRPT	(0x74u)	// Write User Defined Area Write Pointer
	#define `$INSTANCE_NAME`_RUDAWRPT	(0x76u)	// Read User Defined Area Write Pointer
	#define `$INSTANCE_NAME`_B0SEL		(0xC0u)	// Bank 0 Select
	#define `$INSTANCE_NAME`_B1SEL		(0xC2u)	// Bank 1 Select
	#define `$INSTANCE_NAME`_B2SEL		(0xC4u)	// Bank 2 Select
	#define `$INSTANCE_NAME`_B3SEL		(0xC6u)	// Bank 3 Select
	#define `$INSTANCE_NAME`_RBSEL		(0xC8u)	// Read Bank Select
	#define `$INSTANCE_NAME`_SETETHRST	(0xCAu)	// Set ETHRST bit (perform system reset)
	#define `$INSTANCE_NAME`_FCDIS		(0xE0u)	// Flow Control Disable
	#define `$INSTANCE_NAME`_FCSINGLE	(0xE2u)	// Flow Control Single
	#define `$INSTANCE_NAME`_FCMULTIPLE	(0xE4u)	// Flow Control Multiple
	#define `$INSTANCE_NAME`_FCCLEAR		(0xE6u)	// Flow Control Clear
	#define `$INSTANCE_NAME`_SETPKTDEC	(0xCCu)	// Set PKTDEC bit (decrement RX packet pending counter)
	#define `$INSTANCE_NAME`_DMASTOP		(0xD0u)	// DMA Stop
	#define `$INSTANCE_NAME`_DMACKSUM	(0xD8u)	// DMA Start Checksum
	#define `$INSTANCE_NAME`_DMACKSUMS	(0xDAu)	// DMA Start Checksum with Seed
	#define `$INSTANCE_NAME`_DMACOPY		(0xDCu)	// DMA Start Copy
	#define `$INSTANCE_NAME`_DMACOPYS	(0xDEu)	// DMA Start Copy and Checksum with Seed
	#define `$INSTANCE_NAME`_SETTXRTS	(0xD4u)	// Set TXRTS bit (transmit a packet)
	#define `$INSTANCE_NAME`_ENABLERX	(0xE8u)	// Enable RX
	#define `$INSTANCE_NAME`_DISABLERX	(0xEAu)	// Disable RX
	#define `$INSTANCE_NAME`_SETEIE		(0xECu)	// Set Ethernet Interrupt Enable (EIE)
	#define `$INSTANCE_NAME`_CLREIE		(0xEEu)	// Clear Ethernet Interrupt Enable (EIE)
#endif

////////////////////////////////////////////////////
// ENC424J600/624J600 register addresses		  //
////////////////////////////////////////////////////
// SPI Bank 0 registers --------
#define `$INSTANCE_NAME`_ETXST		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E00u)
#define `$INSTANCE_NAME`_ETXSTL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E00u)
#define `$INSTANCE_NAME`_ETXSTH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E01u)
#define `$INSTANCE_NAME`_ETXLEN		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E02u)
#define `$INSTANCE_NAME`_ETXLENL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E02u)
#define `$INSTANCE_NAME`_ETXLENH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E03u)
#define `$INSTANCE_NAME`_ERXST		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E04u)
#define `$INSTANCE_NAME`_ERXSTL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E04u)
#define `$INSTANCE_NAME`_ERXSTH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E05u)
#define `$INSTANCE_NAME`_ERXTAIL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E06u)
#define `$INSTANCE_NAME`_ERXTAILL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E06u)
#define `$INSTANCE_NAME`_ERXTAILH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E07u)
#define `$INSTANCE_NAME`_ERXHEAD		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E08u)
#define `$INSTANCE_NAME`_ERXHEADL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E08u)
#define `$INSTANCE_NAME`_ERXHEADH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E09u)
#define `$INSTANCE_NAME`_EDMAST		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E0Au)
#define `$INSTANCE_NAME`_EDMASTL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E0Au)
#define `$INSTANCE_NAME`_EDMASTH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E0Bu)
#define `$INSTANCE_NAME`_EDMALEN		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E0Cu)
#define `$INSTANCE_NAME`_EDMALENL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E0Cu)
#define `$INSTANCE_NAME`_EDMALENH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E0Du)
#define `$INSTANCE_NAME`_EDMADST		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E0Eu)
#define `$INSTANCE_NAME`_EDMADSTL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E0Eu)
#define `$INSTANCE_NAME`_EDMADSTH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E0Fu)
#define `$INSTANCE_NAME`_EDMACS		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E10u)
#define `$INSTANCE_NAME`_EDMACSL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E10u)
#define `$INSTANCE_NAME`_EDMACSH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E11u)
#define `$INSTANCE_NAME`_ETXSTAT		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E12u)
#define `$INSTANCE_NAME`_ETXSTATL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E12u)
#define `$INSTANCE_NAME`_ETXSTATH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E13u)
#define `$INSTANCE_NAME`_ETXWIRE		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E14u)
#define `$INSTANCE_NAME`_ETXWIREL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E14u)
#define `$INSTANCE_NAME`_ETXWIREH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E15u)

// SPI all bank registers
#define `$INSTANCE_NAME`_EUDAST		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E16u)
#define `$INSTANCE_NAME`_EUDASTL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E16u)
#define `$INSTANCE_NAME`_EUDASTH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E17u)
#define `$INSTANCE_NAME`_EUDAND		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E18u)
#define `$INSTANCE_NAME`_EUDANDL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E18u)
#define `$INSTANCE_NAME`_EUDANDH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E19u)
#define `$INSTANCE_NAME`_ESTAT		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E1Au)
#define `$INSTANCE_NAME`_ESTATL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E1Au)
#define `$INSTANCE_NAME`_ESTATH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E1Bu)
#define `$INSTANCE_NAME`_EIR			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E1Cu)
#define `$INSTANCE_NAME`_EIRL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E1Cu)
#define `$INSTANCE_NAME`_EIRH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E1Du)
#define `$INSTANCE_NAME`_ECON1		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E1Eu)
#define `$INSTANCE_NAME`_ECON1L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E1Eu)
#define `$INSTANCE_NAME`_ECON1H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E1Fu)


// SPI Bank 1 registers -----
#define `$INSTANCE_NAME`_EHT1		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E20u)
#define `$INSTANCE_NAME`_EHT1L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E20u)
#define `$INSTANCE_NAME`_EHT1H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E21u)
#define `$INSTANCE_NAME`_EHT2		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E22u)
#define `$INSTANCE_NAME`_EHT2L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E22u)
#define `$INSTANCE_NAME`_EHT2H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E23u)
#define `$INSTANCE_NAME`_EHT3		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E24u)
#define `$INSTANCE_NAME`_EHT3L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E24u)
#define `$INSTANCE_NAME`_EHT3H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E25u)
#define `$INSTANCE_NAME`_EHT4		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E26u)
#define `$INSTANCE_NAME`_EHT4L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E26u)
#define `$INSTANCE_NAME`_EHT4H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E27u)
#define `$INSTANCE_NAME`_EPMM1		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E28u)
#define `$INSTANCE_NAME`_EPMM1L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E28u)
#define `$INSTANCE_NAME`_EPMM1H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E29u)
#define `$INSTANCE_NAME`_EPMM2		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E2Au)
#define `$INSTANCE_NAME`_EPMM2L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E2Au)
#define `$INSTANCE_NAME`_EPMM2H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E2Bu)
#define `$INSTANCE_NAME`_EPMM3		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E2Cu)
#define `$INSTANCE_NAME`_EPMM3L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E2Cu)
#define `$INSTANCE_NAME`_EPMM3H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E2Du)
#define `$INSTANCE_NAME`_EPMM4		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E2Eu)
#define `$INSTANCE_NAME`_EPMM4L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E2Eu)
#define `$INSTANCE_NAME`_EPMM4H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E2Fu)
#define `$INSTANCE_NAME`_EPMCS		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E30u)
#define `$INSTANCE_NAME`_EPMCSL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E30u)
#define `$INSTANCE_NAME`_EPMCSH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E31u)
#define `$INSTANCE_NAME`_EPMO		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E32u)
#define `$INSTANCE_NAME`_EPMOL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E32u)
#define `$INSTANCE_NAME`_EPMOH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E33u)
#define `$INSTANCE_NAME`_ERXFCON		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E34u)
#define `$INSTANCE_NAME`_ERXFCONL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E34u)
#define `$INSTANCE_NAME`_ERXFCONH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E35u)

// SPI all bank registers from 0x36 to 0x3F


// SPI Bank 2 registers -----
#define `$INSTANCE_NAME`_MACON1		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E40u)
#define `$INSTANCE_NAME`_MACON1L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E40u)
#define `$INSTANCE_NAME`_MACON1H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E41u)
#define `$INSTANCE_NAME`_MACON2		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E42u)
#define `$INSTANCE_NAME`_MACON2L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E42u)
#define `$INSTANCE_NAME`_MACON2H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E43u)
#define `$INSTANCE_NAME`_MABBIPG		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E44u)
#define `$INSTANCE_NAME`_MABBIPGL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E44u)
#define `$INSTANCE_NAME`_MABBIPGH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E45u)
#define `$INSTANCE_NAME`_MAIPG		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E46u)
#define `$INSTANCE_NAME`_MAIPGL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E46u)
#define `$INSTANCE_NAME`_MAIPGH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E47u)
#define `$INSTANCE_NAME`_MACLCON		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E48u)
#define `$INSTANCE_NAME`_MACLCONL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E48u)
#define `$INSTANCE_NAME`_MACLCONH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E49u)
#define `$INSTANCE_NAME`_MAMXFL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E4Au)
#define `$INSTANCE_NAME`_MAMXFLL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E4Au)
#define `$INSTANCE_NAME`_MAMXFLH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E4Bu)
//#define `$INSTANCE_NAME`_r			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E4Cu)
//#define `$INSTANCE_NAME`_r			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E4Du)
//#define `$INSTANCE_NAME`_r			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E4Eu)
//#define `$INSTANCE_NAME`_r			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E4Fu)
//#define `$INSTANCE_NAME`_r			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E50u)
//#define `$INSTANCE_NAME`_r			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E51u)
#define `$INSTANCE_NAME`_MICMD		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E52u)
#define `$INSTANCE_NAME`_MICMDL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E52u)
#define `$INSTANCE_NAME`_MICMDH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E53u)
#define `$INSTANCE_NAME`_MIREGADR	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E54u)
#define `$INSTANCE_NAME`_MIREGADRL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E54u)
#define `$INSTANCE_NAME`_MIREGADRH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E55u)

// SPI all bank registers from 0x56 to 0x5F


// SPI Bank 3 registers -----
#define `$INSTANCE_NAME`_MAADR3		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E60u)
#define `$INSTANCE_NAME`_MAADR3L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E60u)
#define `$INSTANCE_NAME`_MAADR3H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E61u)
#define `$INSTANCE_NAME`_MAADR2		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E62u)
#define `$INSTANCE_NAME`_MAADR2L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E62u)
#define `$INSTANCE_NAME`_MAADR2H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E63u)
#define `$INSTANCE_NAME`_MAADR1		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E64u)
#define `$INSTANCE_NAME`_MAADR1L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E64u)
#define `$INSTANCE_NAME`_MAADR1H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E65u)
#define `$INSTANCE_NAME`_MIWR		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E66u)
#define `$INSTANCE_NAME`_MIWRL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E66u)
#define `$INSTANCE_NAME`_MIWRH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E67u)
#define `$INSTANCE_NAME`_MIRD		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E68u)
#define `$INSTANCE_NAME`_MIRDL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E68u)
#define `$INSTANCE_NAME`_MIRDH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E69u)
#define `$INSTANCE_NAME`_MISTAT		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E6Au)
#define `$INSTANCE_NAME`_MISTATL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E6Au)
#define `$INSTANCE_NAME`_MISTATH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E6Bu)
#define `$INSTANCE_NAME`_EPAUS		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E6Cu)
#define `$INSTANCE_NAME`_EPAUSL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E6Cu)
#define `$INSTANCE_NAME`_EPAUSH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E6Du)
#define `$INSTANCE_NAME`_ECON2		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E6Eu)
#define `$INSTANCE_NAME`_ECON2L		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E6Eu)
#define `$INSTANCE_NAME`_ECON2H		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E6Fu)
#define `$INSTANCE_NAME`_ERXWM		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E70u)
#define `$INSTANCE_NAME`_ERXWML		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E70u)
#define `$INSTANCE_NAME`_ERXWMH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E71u)
#define `$INSTANCE_NAME`_EIE			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E72u)
#define `$INSTANCE_NAME`_EIEL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E72u)
#define `$INSTANCE_NAME`_EIEH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E73u)
#define `$INSTANCE_NAME`_EIDLED		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E74u)
#define `$INSTANCE_NAME`_EIDLEDL		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E74u)
#define `$INSTANCE_NAME`_EIDLEDH		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E75u)

// SPI all bank registers from 0x66 to 0x6F


// SPI Non-banked Special Function Registers
#define `$INSTANCE_NAME`_EGPDATA		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E80u)
#define `$INSTANCE_NAME`_EGPDATAL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E80u)
//#define `$INSTANCE_NAME`_r			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E81u)
#define `$INSTANCE_NAME`_ERXDATA		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E82u)
#define `$INSTANCE_NAME`_ERXDATAL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E82u)
//#define `$INSTANCE_NAME`_r			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E83u)
#define `$INSTANCE_NAME`_EUDADATA	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E84u)
#define `$INSTANCE_NAME`_EUDADATAL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E84u)
//#define `$INSTANCE_NAME`_r			`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E85u)
#define `$INSTANCE_NAME`_EGPRDPT		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E86u)
#define `$INSTANCE_NAME`_EGPRDPTL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E86u)
#define `$INSTANCE_NAME`_EGPRDPTH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E87u)
#define `$INSTANCE_NAME`_EGPWRPT		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E88u)
#define `$INSTANCE_NAME`_EGPWRPTL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E88u)
#define `$INSTANCE_NAME`_EGPWRPTH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E89u)
#define `$INSTANCE_NAME`_ERXRDPT		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E8Au)
#define `$INSTANCE_NAME`_ERXRDPTL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E8Au)
#define `$INSTANCE_NAME`_ERXRDPTH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E8Bu)
#define `$INSTANCE_NAME`_ERXWRPT		`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E8Cu)
#define `$INSTANCE_NAME`_ERXWRPTL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E8Cu)
#define `$INSTANCE_NAME`_ERXWRPTH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E8Du)
#define `$INSTANCE_NAME`_EUDARDPT	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E8Eu)
#define `$INSTANCE_NAME`_EUDARDPTL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E8Eu)
#define `$INSTANCE_NAME`_EUDARDPTH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E8Fu)
#define `$INSTANCE_NAME`_EUDAWRPT	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E90u)
#define `$INSTANCE_NAME`_EUDAWRPTL	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E90u)
#define `$INSTANCE_NAME`_EUDAWRPTH	`$INSTANCE_NAME`_TRANSLATE_TO_PIN_ADDR(0x7E91u)



////////////////////////////////////////////////////
// ENC424J600/624J600 PHY Register Addresses	  //
////////////////////////////////////////////////////
#define `$INSTANCE_NAME`_PHCON1	0x00u
#define `$INSTANCE_NAME`_PHSTAT1	0x01u
#define `$INSTANCE_NAME`_PHANA	0x04u
#define `$INSTANCE_NAME`_PHANLPA	0x05u
#define `$INSTANCE_NAME`_PHANE	0x06u
#define `$INSTANCE_NAME`_PHCON2	0x11u
#define `$INSTANCE_NAME`_PHSTAT2	0x1Bu
#define `$INSTANCE_NAME`_PHSTAT3	0x1Fu



////////////////////////////////////////////////////
// ENC424J600/624J600 register bits				  //
////////////////////////////////////////////////////
// ESTAT bits ----------
#define `$INSTANCE_NAME`_ESTAT_INT		((WORD)1<<15)
#define `$INSTANCE_NAME`_ESTAT_FCIDLE	((WORD)1<<14)
#define `$INSTANCE_NAME`_ESTAT_RXBUSY	((WORD)1<<13)
#define `$INSTANCE_NAME`_ESTAT_CLKRDY	((WORD)1<<12)
#define `$INSTANCE_NAME`_ESTAT_RSTDONE	((WORD)1<<11)
#define `$INSTANCE_NAME`_ESTAT_PHYDPX	((WORD)1<<10)
#define `$INSTANCE_NAME`_ESTAT_PHYRDY	((WORD)1<<9)
#define `$INSTANCE_NAME`_ESTAT_PHYLNK	((WORD)1<<8)
#define `$INSTANCE_NAME`_ESTAT_PKTCNT7	(1<<7)
#define `$INSTANCE_NAME`_ESTAT_PKTCNT6	(1<<6)
#define `$INSTANCE_NAME`_ESTAT_PKTCNT5	(1<<5)
#define `$INSTANCE_NAME`_ESTAT_PKTCNT4	(1<<4)
#define `$INSTANCE_NAME`_ESTAT_PKTCNT3	(1<<3)
#define `$INSTANCE_NAME`_ESTAT_PKTCNT2	(1<<2)
#define `$INSTANCE_NAME`_ESTAT_PKTCNT1	(1<<1)
#define `$INSTANCE_NAME`_ESTAT_PKTCNT0	(1)

// EIR bits ------------
#define `$INSTANCE_NAME`_EIR_CRYPTEN		((WORD)1<<15)
#define `$INSTANCE_NAME`_EIR_MODEXIF		((WORD)1<<14)
#define `$INSTANCE_NAME`_EIR_HASHIF		((WORD)1<<13)
#define `$INSTANCE_NAME`_EIR_AESIF		((WORD)1<<12)
#define `$INSTANCE_NAME`_EIR_LINKIF		((WORD)1<<11)
#define `$INSTANCE_NAME`_EIR_PRDYIF		((WORD)1<<10)
#define `$INSTANCE_NAME`_EIR_r9			((WORD)1<<9)
#define `$INSTANCE_NAME`_EIR_r8			((WORD)1<<8)
#define `$INSTANCE_NAME`_EIR_r7			(1<<7)
#define `$INSTANCE_NAME`_EIR_PKTIF		(1<<6)
#define `$INSTANCE_NAME`_EIR_DMAIF		(1<<5)
#define `$INSTANCE_NAME`_EIR_r4			(1<<4)
#define `$INSTANCE_NAME`_EIR_TXIF		(1<<3)
#define `$INSTANCE_NAME`_EIR_TXABTIF		(1<<2)
#define `$INSTANCE_NAME`_EIR_RXABTIF		(1<<1)
#define `$INSTANCE_NAME`_EIR_PCFULIF		(1)

// ECON1 bits ----------
#define `$INSTANCE_NAME`_ECON1_MODEXST	((WORD)1<<15)
#define `$INSTANCE_NAME`_ECON1_HASHEN	((WORD)1<<14)
#define `$INSTANCE_NAME`_ECON1_HASHOP	((WORD)1<<13)
#define `$INSTANCE_NAME`_ECON1_HASHLST	((WORD)1<<12)
#define `$INSTANCE_NAME`_ECON1_AESST		((WORD)1<<11)
#define `$INSTANCE_NAME`_ECON1_AESOP1	((WORD)1<<10)
#define `$INSTANCE_NAME`_ECON1_AESOP0	((WORD)1<<9)
#define `$INSTANCE_NAME`_ECON1_PKTDEC	((WORD)1<<8)
#define `$INSTANCE_NAME`_ECON1_FCOP1		(1<<7)
#define `$INSTANCE_NAME`_ECON1_FCOP0		(1<<6)
#define `$INSTANCE_NAME`_ECON1_DMAST		(1<<5)
#define `$INSTANCE_NAME`_ECON1_DMACPY	(1<<4)
#define `$INSTANCE_NAME`_ECON1_DMACSSD	(1<<3)
#define `$INSTANCE_NAME`_ECON1_DMANOCS	(1<<2)
#define `$INSTANCE_NAME`_ECON1_TXRTS		(1<<1)
#define `$INSTANCE_NAME`_ECON1_RXEN		(1)

// ETXSTAT bits --------
#define `$INSTANCE_NAME`_ETXSTAT_r12		((WORD)1<<12)
#define `$INSTANCE_NAME`_ETXSTAT_r11		((WORD)1<<11)
#define `$INSTANCE_NAME`_ETXSTAT_LATECOL	((WORD)1<<10)
#define `$INSTANCE_NAME`_ETXSTAT_MAXCOL	((WORD)1<<9)
#define `$INSTANCE_NAME`_ETXSTAT_EXDEFER	((WORD)1<<8)
#define `$INSTANCE_NAME`_ETXSTAT_DEFER	(1<<7)
#define `$INSTANCE_NAME`_ETXSTAT_r6		(1<<6)
#define `$INSTANCE_NAME`_ETXSTAT_r5		(1<<5)
#define `$INSTANCE_NAME`_ETXSTAT_CRCBAD	(1<<4)
#define `$INSTANCE_NAME`_ETXSTAT_COLCNT3 (1<<3)
#define `$INSTANCE_NAME`_ETXSTAT_COLCNT2 (1<<2)
#define `$INSTANCE_NAME`_ETXSTAT_COLCNT1 (1<<1)
#define `$INSTANCE_NAME`_ETXSTAT_COLCNT0 (1)

// ERXFCON bits --------
#define `$INSTANCE_NAME`_ERXFCON_HTEN	((WORD)1<<15)
#define `$INSTANCE_NAME`_ERXFCON_MPEN	((WORD)1<<14)
#define `$INSTANCE_NAME`_ERXFCON_NOTPM	((WORD)1<<12)
#define `$INSTANCE_NAME`_ERXFCON_PMEN3	((WORD)1<<11)
#define `$INSTANCE_NAME`_ERXFCON_PMEN2	((WORD)1<<10)
#define `$INSTANCE_NAME`_ERXFCON_PMEN1	((WORD)1<<9)
#define `$INSTANCE_NAME`_ERXFCON_PMEN0	((WORD)1<<8)
#define `$INSTANCE_NAME`_ERXFCON_CRCEEN	(1<<7)
#define `$INSTANCE_NAME`_ERXFCON_CRCEN	(1<<6)
#define `$INSTANCE_NAME`_ERXFCON_RUNTEEN	(1<<5)
#define `$INSTANCE_NAME`_ERXFCON_RUNTEN	(1<<4)
#define `$INSTANCE_NAME`_ERXFCON_UCEN	(1<<3)
#define `$INSTANCE_NAME`_ERXFCON_NOTMEEN	(1<<2)
#define `$INSTANCE_NAME`_ERXFCON_MCEN	(1<<1)
#define `$INSTANCE_NAME`_ERXFCON_BCEN	(1)

// MACON1 bits ---------
#define `$INSTANCE_NAME`_MACON1_r15		((WORD)1<<15)
#define `$INSTANCE_NAME`_MACON1_r14		((WORD)1<<14)
#define `$INSTANCE_NAME`_MACON1_r11		((WORD)1<<11)
#define `$INSTANCE_NAME`_MACON1_r10		((WORD)1<<10)
#define `$INSTANCE_NAME`_MACON1_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_MACON1_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_MACON1_LOOPBK	(1<<4)
#define `$INSTANCE_NAME`_MACON1_r3		(1<<3)
#define `$INSTANCE_NAME`_MACON1_RXPAUS	(1<<2)
#define `$INSTANCE_NAME`_MACON1_PASSALL	(1<<1)
#define `$INSTANCE_NAME`_MACON1_r0		(1)

// MACON2 bits ---------
#define `$INSTANCE_NAME`_MACON2_DEFER	((WORD)1<<14)
#define `$INSTANCE_NAME`_MACON2_BPEN		((WORD)1<<13)
#define `$INSTANCE_NAME`_MACON2_NOBKOFF	((WORD)1<<12)
#define `$INSTANCE_NAME`_MACON2_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_MACON2_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_MACON2_PADCFG2	(1<<7)
#define `$INSTANCE_NAME`_MACON2_PADCFG1	(1<<6)
#define `$INSTANCE_NAME`_MACON2_PADCFG0	(1<<5)
#define `$INSTANCE_NAME`_MACON2_TXCRCEN	(1<<4)
#define `$INSTANCE_NAME`_MACON2_PHDREN	(1<<3)
#define `$INSTANCE_NAME`_MACON2_HFRMEN	(1<<2)
#define `$INSTANCE_NAME`_MACON2_r1		(1<<1)
#define `$INSTANCE_NAME`_MACON2_FULDPX	(1)

// MABBIPG bits --------
#define `$INSTANCE_NAME`_MABBIPG_BBIPG6  (1<<6)
#define `$INSTANCE_NAME`_MABBIPG_BBIPG5  (1<<5)
#define `$INSTANCE_NAME`_MABBIPG_BBIPG4  (1<<4)
#define `$INSTANCE_NAME`_MABBIPG_BBIPG3  (1<<3)
#define `$INSTANCE_NAME`_MABBIPG_BBIPG2  (1<<2)
#define `$INSTANCE_NAME`_MABBIPG_BBIPG1  (1<<1)
#define `$INSTANCE_NAME`_MABBIPG_BBIPG0  (1)

// MAIPG bits ----------
#define `$INSTANCE_NAME`_MAIPG_r14		((WORD)1<<14)
#define `$INSTANCE_NAME`_MAIPG_r13		((WORD)1<<13)
#define `$INSTANCE_NAME`_MAIPG_r12		((WORD)1<<12)
#define `$INSTANCE_NAME`_MAIPG_r11		((WORD)1<<11)
#define `$INSTANCE_NAME`_MAIPG_r10		((WORD)1<<10)
#define `$INSTANCE_NAME`_MAIPG_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_MAIPG_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_MAIPG_IPG6		(1<<6)
#define `$INSTANCE_NAME`_MAIPG_IPG5		(1<<5)
#define `$INSTANCE_NAME`_MAIPG_IPG4		(1<<4)
#define `$INSTANCE_NAME`_MAIPG_IPG3		(1<<3)
#define `$INSTANCE_NAME`_MAIPG_IPG2		(1<<2)
#define `$INSTANCE_NAME`_MAIPG_IPG1		(1<<1)
#define `$INSTANCE_NAME`_MAIPG_IPG0		(1)

// MACLCON bits --------
#define `$INSTANCE_NAME`_MACLCON_r13		((WORD)1<<13)
#define `$INSTANCE_NAME`_MACLCON_r12		((WORD)1<<12)
#define `$INSTANCE_NAME`_MACLCON_r11		((WORD)1<<11)
#define `$INSTANCE_NAME`_MACLCON_r10		((WORD)1<<10)
#define `$INSTANCE_NAME`_MACLCON_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_MACLCON_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_MACLCON_MAXRET3	(1<<3)
#define `$INSTANCE_NAME`_MACLCON_MAXRET2	(1<<2)
#define `$INSTANCE_NAME`_MACLCON_MAXRET1	(1<<1)
#define `$INSTANCE_NAME`_MACLCON_MAXRET0	(1)

// MICMD bits ----------
#define `$INSTANCE_NAME`_MICMD_MIISCAN	(1<<1)
#define `$INSTANCE_NAME`_MICMD_MIIRD		(1)

// MIREGADR bits -------
#define `$INSTANCE_NAME`_MIREGADR_r12	((WORD)1<<12)
#define `$INSTANCE_NAME`_MIREGADR_r11	((WORD)1<<11)
#define `$INSTANCE_NAME`_MIREGADR_r10	((WORD)1<<10)
#define `$INSTANCE_NAME`_MIREGADR_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_MIREGADR_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_MIREGADR_PHREG4	(1<<4)
#define `$INSTANCE_NAME`_MIREGADR_PHREG3	(1<<3)
#define `$INSTANCE_NAME`_MIREGADR_PHREG2	(1<<2)
#define `$INSTANCE_NAME`_MIREGADR_PHREG1	(1<<1)
#define `$INSTANCE_NAME`_MIREGADR_PHREG0	(1)

// MISTAT bits ---------
#define `$INSTANCE_NAME`_MISTAT_r3		(1<<3)
#define `$INSTANCE_NAME`_MISTAT_NVALID	(1<<2)
#define `$INSTANCE_NAME`_MISTAT_SCAN		(1<<1)
#define `$INSTANCE_NAME`_MISTAT_BUSY		(1)

// ECON2 bits ----------
#define `$INSTANCE_NAME`_ECON2_ETHEN		((WORD)1<<15)
#define `$INSTANCE_NAME`_ECON2_STRCH		((WORD)1<<14)
#define `$INSTANCE_NAME`_ECON2_TXMAC		((WORD)1<<13)
#define `$INSTANCE_NAME`_ECON2_SHA1MD5	((WORD)1<<12)
#define `$INSTANCE_NAME`_ECON2_COCON3	((WORD)1<<11)
#define `$INSTANCE_NAME`_ECON2_COCON2	((WORD)1<<10)
#define `$INSTANCE_NAME`_ECON2_COCON1	((WORD)1<<9)
#define `$INSTANCE_NAME`_ECON2_COCON0	((WORD)1<<8)
#define `$INSTANCE_NAME`_ECON2_AUTOFC	(1<<7)
#define `$INSTANCE_NAME`_ECON2_TXRST		(1<<6)
#define `$INSTANCE_NAME`_ECON2_RXRST		(1<<5)
#define `$INSTANCE_NAME`_ECON2_ETHRST	(1<<4)
#define `$INSTANCE_NAME`_ECON2_MODLEN1	(1<<3)
#define `$INSTANCE_NAME`_ECON2_MODLEN0	(1<<2)
#define `$INSTANCE_NAME`_ECON2_AESLEN1	(1<<1)
#define `$INSTANCE_NAME`_ECON2_AESLEN0	(1)

// ERXWM bits ----------
#define `$INSTANCE_NAME`_ERXWM_RXFWM7	((WORD)1<<15)
#define `$INSTANCE_NAME`_ERXWM_RXFWM6	((WORD)1<<14)
#define `$INSTANCE_NAME`_ERXWM_RXFWM5	((WORD)1<<13)
#define `$INSTANCE_NAME`_ERXWM_RXFWM4	((WORD)1<<12)
#define `$INSTANCE_NAME`_ERXWM_RXFWM3	((WORD)1<<11)
#define `$INSTANCE_NAME`_ERXWM_RXFWM2	((WORD)1<<10)
#define `$INSTANCE_NAME`_ERXWM_RXFWM1	((WORD)1<<9)
#define `$INSTANCE_NAME`_ERXWM_RXFWM0	((WORD)1<<8)
#define `$INSTANCE_NAME`_ERXWM_RXEWM7	(1<<7)
#define `$INSTANCE_NAME`_ERXWM_RXEWM6	(1<<6)
#define `$INSTANCE_NAME`_ERXWM_RXEWM5	(1<<5)
#define `$INSTANCE_NAME`_ERXWM_RXEWM4	(1<<4)
#define `$INSTANCE_NAME`_ERXWM_RXEWM3	(1<<3)
#define `$INSTANCE_NAME`_ERXWM_RXEWM2	(1<<2)
#define `$INSTANCE_NAME`_ERXWM_RXEWM1	(1<<1)
#define `$INSTANCE_NAME`_ERXWM_RXEWM0	(1)

// EIE bits ------------
#define `$INSTANCE_NAME`_EIE_INTIE		((WORD)1<<15)
#define `$INSTANCE_NAME`_EIE_MODEXIE		((WORD)1<<14)
#define `$INSTANCE_NAME`_EIE_HASHIE		((WORD)1<<13)
#define `$INSTANCE_NAME`_EIE_AESIE		((WORD)1<<12)
#define `$INSTANCE_NAME`_EIE_LINKIE		((WORD)1<<11)
#define `$INSTANCE_NAME`_EIE_PRDYIE		((WORD)1<<10)
#define `$INSTANCE_NAME`_EIE_r9			((WORD)1<<9)
#define `$INSTANCE_NAME`_EIE_r8			((WORD)1<<8)
#define `$INSTANCE_NAME`_EIE_r7			(1<<7)
#define `$INSTANCE_NAME`_EIE_PKTIE		(1<<6)
#define `$INSTANCE_NAME`_EIE_DMAIE		(1<<5)
#define `$INSTANCE_NAME`_EIE_r4			(1<<4)
#define `$INSTANCE_NAME`_EIE_TXIE		(1<<3)
#define `$INSTANCE_NAME`_EIE_TXABTIE		(1<<2)
#define `$INSTANCE_NAME`_EIE_RXABTIE		(1<<1)
#define `$INSTANCE_NAME`_EIE_PCFULIE		(1)

// EIDLED bits ---------
#define `$INSTANCE_NAME`_EIDLED_LACFG3	((WORD)1<<15)
#define `$INSTANCE_NAME`_EIDLED_LACFG2	((WORD)1<<14)
#define `$INSTANCE_NAME`_EIDLED_LACFG1	((WORD)1<<13)
#define `$INSTANCE_NAME`_EIDLED_LACFG0	((WORD)1<<12)
#define `$INSTANCE_NAME`_EIDLED_LBCFG3	((WORD)1<<11)
#define `$INSTANCE_NAME`_EIDLED_LBCFG2	((WORD)1<<10)
#define `$INSTANCE_NAME`_EIDLED_LBCFG1	((WORD)1<<9)
#define `$INSTANCE_NAME`_EIDLED_LBCFG0	((WORD)1<<8)
#define `$INSTANCE_NAME`_EIDLED_DEVID2	(1<<7)
#define `$INSTANCE_NAME`_EIDLED_DEVID1	(1<<6)
#define `$INSTANCE_NAME`_EIDLED_DEVID0	(1<<5)
#define `$INSTANCE_NAME`_EIDLED_REVID4	(1<<4)
#define `$INSTANCE_NAME`_EIDLED_REVID3	(1<<3)
#define `$INSTANCE_NAME`_EIDLED_REVID2	(1<<2)
#define `$INSTANCE_NAME`_EIDLED_REVID1	(1<<1)
#define `$INSTANCE_NAME`_EIDLED_REVID0	(1)

// PHCON1 bits ---------
#define `$INSTANCE_NAME`_PHCON1_PRST		((WORD)1<<15)
#define `$INSTANCE_NAME`_PHCON1_PLOOPBK	((WORD)1<<14)
#define `$INSTANCE_NAME`_PHCON1_SPD100	((WORD)1<<13)
#define `$INSTANCE_NAME`_PHCON1_ANEN		((WORD)1<<12)
#define `$INSTANCE_NAME`_PHCON1_PSLEEP	((WORD)1<<11)
#define `$INSTANCE_NAME`_PHCON1_r10		((WORD)1<<10)
#define `$INSTANCE_NAME`_PHCON1_RENEG	((WORD)1<<9)
#define `$INSTANCE_NAME`_PHCON1_PFULDPX	((WORD)1<<8)
#define `$INSTANCE_NAME`_PHCON1_r7		(1<<7)
#define `$INSTANCE_NAME`_PHCON1_r6		(1<<6)
#define `$INSTANCE_NAME`_PHCON1_r5		(1<<5)
#define `$INSTANCE_NAME`_PHCON1_r4		(1<<4)
#define `$INSTANCE_NAME`_PHCON1_r3		(1<<3)
#define `$INSTANCE_NAME`_PHCON1_r2		(1<<2)
#define `$INSTANCE_NAME`_PHCON1_r1		(1<<1)
#define `$INSTANCE_NAME`_PHCON1_r0		(1)

// PHSTAT1 bits --------
#define `$INSTANCE_NAME`_PHSTAT1_r15		((WORD)1<<15)
#define `$INSTANCE_NAME`_PHSTAT1_FULL100	((WORD)1<<14)
#define `$INSTANCE_NAME`_PHSTAT1_HALF100	((WORD)1<<13)
#define `$INSTANCE_NAME`_PHSTAT1_FULL10	((WORD)1<<12)
#define `$INSTANCE_NAME`_PHSTAT1_HALF10	((WORD)1<<11)
#define `$INSTANCE_NAME`_PHSTAT1_r10		((WORD)1<<10)
#define `$INSTANCE_NAME`_PHSTAT1_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_PHSTAT1_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_PHSTAT1_r7		(1<<7)
#define `$INSTANCE_NAME`_PHSTAT1_r6		(1<<6)
#define `$INSTANCE_NAME`_PHSTAT1_ANDONE	(1<<5)
#define `$INSTANCE_NAME`_PHSTAT1_LRFAULT	(1<<4)
#define `$INSTANCE_NAME`_PHSTAT1_ANABLE	(1<<3)
#define `$INSTANCE_NAME`_PHSTAT1_LLSTAT	(1<<2)
#define `$INSTANCE_NAME`_PHSTAT1_r1		(1<<1)
#define `$INSTANCE_NAME`_PHSTAT1_EXTREGS	(1)

// PHANA bits ----------
#define `$INSTANCE_NAME`_PHANA_ADNP		((WORD)1<<15)
#define `$INSTANCE_NAME`_PHANA_r14		((WORD)1<<14)
#define `$INSTANCE_NAME`_PHANA_ADFAULT	((WORD)1<<13)
#define `$INSTANCE_NAME`_PHANA_r12		((WORD)1<<12)
#define `$INSTANCE_NAME`_PHANA_ADPAUS1	((WORD)1<<11)
#define `$INSTANCE_NAME`_PHANA_ADPAUS0	((WORD)1<<10)
#define `$INSTANCE_NAME`_PHANA_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_PHANA_AD100FD	((WORD)1<<8)
#define `$INSTANCE_NAME`_PHANA_AD100		(1<<7)
#define `$INSTANCE_NAME`_PHANA_AD10FD	(1<<6)
#define `$INSTANCE_NAME`_PHANA_AD10		(1<<5)
#define `$INSTANCE_NAME`_PHANA_ADIEEE4	(1<<4)
#define `$INSTANCE_NAME`_PHANA_ADIEEE3	(1<<3)
#define `$INSTANCE_NAME`_PHANA_ADIEEE2	(1<<2)
#define `$INSTANCE_NAME`_PHANA_ADIEEE1	(1<<1)
#define `$INSTANCE_NAME`_PHANA_ADIEEE0	(1)

// PHANLPA bits --------
#define `$INSTANCE_NAME`_PHANLPA_LPNP	((WORD)1<<15)
#define `$INSTANCE_NAME`_PHANLPA_LPACK	((WORD)1<<14)
#define `$INSTANCE_NAME`_PHANLPA_LPFAULT	((WORD)1<<13)
#define `$INSTANCE_NAME`_PHANLPA_r12		((WORD)1<<12)
#define `$INSTANCE_NAME`_PHANLPA_LPPAUS1	((WORD)1<<11)
#define `$INSTANCE_NAME`_PHANLPA_LPPAUS0	((WORD)1<<10)
#define `$INSTANCE_NAME`_PHANLPA_LP100T4	((WORD)1<<9)
#define `$INSTANCE_NAME`_PHANLPA_LP100FD	((WORD)1<<8)
#define `$INSTANCE_NAME`_PHANLPA_LP100	(1<<7)
#define `$INSTANCE_NAME`_PHANLPA_LP10FD	(1<<6)
#define `$INSTANCE_NAME`_PHANLPA_LP10	(1<<5)
#define `$INSTANCE_NAME`_PHANLPA_LPIEEE4	(1<<4)
#define `$INSTANCE_NAME`_PHANLPA_LPIEEE3	(1<<3)
#define `$INSTANCE_NAME`_PHANLPA_LPIEEE2	(1<<2)
#define `$INSTANCE_NAME`_PHANLPA_LPIEEE1	(1<<1)
#define `$INSTANCE_NAME`_PHANLPA_LPIEEE0	(1)

// PHANE bits ----------
#define `$INSTANCE_NAME`_PHANE_r15		((WORD)1<<15)
#define `$INSTANCE_NAME`_PHANE_r14		((WORD)1<<14)
#define `$INSTANCE_NAME`_PHANE_r13		((WORD)1<<13)
#define `$INSTANCE_NAME`_PHANE_r12		((WORD)1<<12)
#define `$INSTANCE_NAME`_PHANE_r11		((WORD)1<<11)
#define `$INSTANCE_NAME`_PHANE_r10		((WORD)1<<10)
#define `$INSTANCE_NAME`_PHANE_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_PHANE_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_PHANE_r7		(1<<7)
#define `$INSTANCE_NAME`_PHANE_r6		(1<<6)
#define `$INSTANCE_NAME`_PHANE_r5		(1<<5)
#define `$INSTANCE_NAME`_PHANE_PDFLT		(1<<4)
#define `$INSTANCE_NAME`_PHANE_r3		(1<<3)
#define `$INSTANCE_NAME`_PHANE_r2		(1<<2)
#define `$INSTANCE_NAME`_PHANE_LPARCD	(1<<1)
#define `$INSTANCE_NAME`_PHANA_LPANABL	(1)

// PHCON2 bits ---------
#define `$INSTANCE_NAME`_PHCON2_r15		((WORD)1<<15)
#define `$INSTANCE_NAME`_PHCON2_r14		((WORD)1<<14)
#define `$INSTANCE_NAME`_PHCON2_EDPWRDN	((WORD)1<<13)
#define `$INSTANCE_NAME`_PHCON2_r12		((WORD)1<<12)
#define `$INSTANCE_NAME`_PHCON2_EDTHRES	((WORD)1<<11)
#define `$INSTANCE_NAME`_PHCON2_r10		((WORD)1<<10)
#define `$INSTANCE_NAME`_PHCON2_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_PHCON2_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_PHCON2_r7		(1<<7)
#define `$INSTANCE_NAME`_PHCON2_r6		(1<<6)
#define `$INSTANCE_NAME`_PHCON2_r5		(1<<5)
#define `$INSTANCE_NAME`_PHCON2_r4		(1<<4)
#define `$INSTANCE_NAME`_PHCON2_r3		(1<<3)
#define `$INSTANCE_NAME`_PHCON2_FRCLNK	(1<<2)
#define `$INSTANCE_NAME`_PHCON2_EDSTAT	(1<<1)
#define `$INSTANCE_NAME`_PHCON2_r0		(1)

// PHSTAT2 bits ---------
#define `$INSTANCE_NAME`_PHSTAT2_r15		((WORD)1<<15)
#define `$INSTANCE_NAME`_PHSTAT2_r14		((WORD)1<<14)
#define `$INSTANCE_NAME`_PHSTAT2_r13		((WORD)1<<13)
#define `$INSTANCE_NAME`_PHSTAT2_r12		((WORD)1<<12)
#define `$INSTANCE_NAME`_PHSTAT2_r11		((WORD)1<<11)
#define `$INSTANCE_NAME`_PHSTAT2_r10		((WORD)1<<10)
#define `$INSTANCE_NAME`_PHSTAT2_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_PHSTAT2_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_PHSTAT2_r7		(1<<7)
#define `$INSTANCE_NAME`_PHSTAT2_r6		(1<<6)
#define `$INSTANCE_NAME`_PHSTAT2_r5		(1<<5)
#define `$INSTANCE_NAME`_PHSTAT2_PLRITY	(1<<4)
#define `$INSTANCE_NAME`_PHSTAT2_r3		(1<<3)
#define `$INSTANCE_NAME`_PHSTAT2_r2		(1<<2)
#define `$INSTANCE_NAME`_PHSTAT2_r1		(1<<1)
#define `$INSTANCE_NAME`_PHSTAT2_r0		(1)

// PHSTAT3 bits --------
#define `$INSTANCE_NAME`_PHSTAT3_r15		((WORD)1<<15)
#define `$INSTANCE_NAME`_PHSTAT3_r14		((WORD)1<<14)
#define `$INSTANCE_NAME`_PHSTAT3_r13		((WORD)1<<13)
#define `$INSTANCE_NAME`_PHSTAT3_r12		((WORD)1<<12)
#define `$INSTANCE_NAME`_PHSTAT3_r11		((WORD)1<<11)
#define `$INSTANCE_NAME`_PHSTAT3_r10		((WORD)1<<10)
#define `$INSTANCE_NAME`_PHSTAT3_r9		((WORD)1<<9)
#define `$INSTANCE_NAME`_PHSTAT3_r8		((WORD)1<<8)
#define `$INSTANCE_NAME`_PHSTAT3_r7		(1<<7)
#define `$INSTANCE_NAME`_PHSTAT3_r6		(1<<6)
#define `$INSTANCE_NAME`_PHSTAT3_r5		(1<<5)
#define `$INSTANCE_NAME`_PHSTAT3_SPDDPX2	(1<<4)
#define `$INSTANCE_NAME`_PHSTAT3_SPDDPX1	(1<<3)
#define `$INSTANCE_NAME`_PHSTAT3_SPDDPX0	(1<<2)
#define `$INSTANCE_NAME`_PHSTAT3_r1		(1<<1)
#define `$INSTANCE_NAME`_PHSTAT3_r0		(1)

//---------------------------------------------------------------------
//	function prototypes
//---------------------------------------------------------------------

void `$INSTANCE_NAME`_Start();
BOOL `$INSTANCE_NAME`_MACIsLinked(void);
BOOL `$INSTANCE_NAME`_MACIsTxReady(void);
void `$INSTANCE_NAME`_MACDiscardRx(void);
WORD `$INSTANCE_NAME`_MACGetFreeRxSize(void);
//BOOL `$INSTANCE_NAME`_MACGetHeader(MAC_ADDR *remote, BYTE* type);
BOOL `$INSTANCE_NAME`_MACGetHeader(`$INSTANCE_NAME`_PREAMBLE *preamble);
BOOL `$INSTANCE_NAME`_MACGet_NPP_RSV(`$INSTANCE_NAME`_PREAMBLE *preamble);
//void `$INSTANCE_NAME`_MACPutHeader(`$INSTANCE_NAME`_MAC_ADDR *source, `$INSTANCE_NAME`_MAC_ADDR *remote, BYTE type, WORD dataLen);
void `$INSTANCE_NAME`_MACPutHeader(`$INSTANCE_NAME`_PREAMBLE *preamble, WORD dataLen);
void `$INSTANCE_NAME`_MACPutStart(WORD totalLen);
void `$INSTANCE_NAME`_MACFlush(void);
void `$INSTANCE_NAME`_MACSetReadPtrInRx(WORD offset);
PTR_BASE `$INSTANCE_NAME`_MACSetWritePtr(PTR_BASE address);
PTR_BASE `$INSTANCE_NAME`_MACSetReadPtr(PTR_BASE address);
WORD `$INSTANCE_NAME`_MACCalcRxChecksum(WORD offset, WORD len);
WORD `$INSTANCE_NAME`_CalcIPBufferChecksum(WORD len);
void `$INSTANCE_NAME`_MACMemCopyAsync(PTR_BASE destAddr, PTR_BASE sourceAddr, WORD len);
BYTE `$INSTANCE_NAME`_MACGet();
WORD `$INSTANCE_NAME`_MACGetArray(BYTE *val, WORD len);
void `$INSTANCE_NAME`_MACPut(BYTE val);
void `$INSTANCE_NAME`_MACPutArray(BYTE *val, WORD len);
void `$INSTANCE_NAME`_MACPowerDown(void);
void `$INSTANCE_NAME`_MACEDPowerDown(void);
void `$INSTANCE_NAME`_MACPowerUp(void);
void `$INSTANCE_NAME`_SetCLKOUT(BYTE NewConfig);
BYTE `$INSTANCE_NAME`_GetCLKOUT(void);
void `$INSTANCE_NAME`_SetRXHashTableEntry(`$INSTANCE_NAME`_MAC_ADDR DestMACAddr);

void `$INSTANCE_NAME`_SendSystemReset(void);

void `$INSTANCE_NAME`_WriteMemoryWindow(BYTE vWindow, BYTE *vData, WORD wLength);
void `$INSTANCE_NAME`_ReadMemoryWindow(BYTE vWindow, BYTE *vData, WORD wLength);

WORD `$INSTANCE_NAME`_ReadReg(WORD wAddress);
void `$INSTANCE_NAME`_WriteReg(WORD wAddress, WORD wValue);
void `$INSTANCE_NAME`_BFSReg(WORD wAddress, WORD wBitMask);
void `$INSTANCE_NAME`_BFCReg(WORD wAddress, WORD wBitMask);
void `$INSTANCE_NAME`_ToggleCRYPTEN(void);

WORD `$INSTANCE_NAME`_ReadPHYReg(BYTE Register);
void `$INSTANCE_NAME`_WritePHYReg(BYTE Register, WORD Data);

#endif
