//=====================================================================
//
//	j1ENC424_lwip.c - modified version of lwip skeleton for psoc creator
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

/**
 * @file
 * Ethernet Interface Skeleton
 *
 */

/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

#include <cytypes.h>
#include <string.h>

#include "`$INSTANCE_NAME`.h"

#if `$INSTANCE_NAME`_PARAM_lwIP_Driver

#include "lwIP_lwip_opt.h"
	
#include "lwIP_lwip_def.h"
#include "lwIP_lwip_mem.h"
#include "lwIP_lwip_pbuf.h"
#include "lwIP_lwip_stats.h"
#include "lwIP_lwip_snmp.h"
#include "lwIP_netif_etharp.h"
#include "lwIP_netif_ppp_oe.h"

//---------------------------------------------------------------------
//	interface name constants
//---------------------------------------------------------------------
	
#define IFNAME0 'e'
#define IFNAME1 '0'

//---------------------------------------------------------------------
//	private data structure - not used
//---------------------------------------------------------------------

//struct ethernetif {
//  struct eth_addr *ethaddr;
//  /* Add whatever per-interface state that is needed here. */
//};

//---------------------------------------------------------------------
//	low_level_init() - private function to init the hardware
//---------------------------------------------------------------------

static void low_level_init(struct netif *netif)
{
	// private data - not used
//	struct ethernetif *ethernetif = netif->state;

	// start the component
	`$INSTANCE_NAME`_Start();
  
	// set MAC hardware address length
	netif->hwaddr_len = ETHARP_HWADDR_LEN;
	// set MAC hardware address
	`$INSTANCE_NAME`_CPY_MAC(netif->hwaddr[0], `$INSTANCE_NAME`_mac_addr);
	// set maximum transfer unit
	netif->mtu = 1500;
	// set device capabilities
	netif->flags = NETIF_FLAG_BROADCAST | NETIF_FLAG_ETHARP | NETIF_FLAG_LINK_UP;
	
	`$INSTANCE_NAME`_rxUsed			= 0;
	`$INSTANCE_NAME`_rxTotal		= 0;
	`$INSTANCE_NAME`_rxStatReset	= true;
}

//---------------------------------------------------------------------
//
//	low_level_output() - packet transmission
//
//	This function should do the actual transmission of the packet. The packet is
//	contained in the pbuf that is passed to the function. This pbuf
//	might be chained.
//
//	@param netif the lwip network interface structure for this ethernetif
//	@param p the MAC packet to send (e.g. IP packet including MAC addresses and type)
//	@return ERR_OK if the packet could be sent
//		an err_t value if the packet couldn't be sent
//
//	@note Returning ERR_MEM here if a DMA queue of your MAC is full can lead to
//		strange results. You might consider waiting for space in the DMA queue
//		to become availale since the stack doesn't retry to send a packet
//		dropped because of memory failure (except for the TCP timers).
//
//---------------------------------------------------------------------

static err_t low_level_output(struct netif *netif, struct pbuf *p)
{
	// private data - not used
//	struct ethernetif *ethernetif = netif->state;

	struct pbuf		*q;
	WORD			totalLen;

	while (!`$INSTANCE_NAME`_MACIsTxReady()) {}
	
#if ETH_PAD_SIZE
	pbuf_header(p, -ETH_PAD_SIZE); /* drop the padding word */
#endif

	totalLen = 0;
	for (q = p; q != NULL; q = q->next) { totalLen += q->len; }
	`$INSTANCE_NAME`_MACPutStart(totalLen);

  	for (q = p; q != NULL; q = q->next)
	{
		// Send the data from the pbuf to the interface, one pbuf at a
		// time. The size of the data in each pbuf is kept in the ->len
		// variable.
		
		`$INSTANCE_NAME`_MACPutArray(q->payload, q->len);
	}

	`$INSTANCE_NAME`_MACFlush();

#if ETH_PAD_SIZE
	pbuf_header(p, ETH_PAD_SIZE); /* reclaim the padding word */
#endif
  
	LINK_STATS_INC(link.xmit);

	return ERR_OK;
}

//---------------------------------------------------------------------
//
//	low_level_input() - fetch the next received packet in buffer
//
//	Should allocate a pbuf and transfer the bytes of the incoming
//	packet from the interface into the pbuf.
//
//	@param netif the lwip network interface structure for this ethernetif
//	@return a pbuf filled with the received packet (including MAC header)
//		NULL on memory error
//
//---------------------------------------------------------------------

static struct pbuf * low_level_input(struct netif *netif)
{
	uint16	rxUsed;
	
	// private data - not used
//	struct ethernetif *ethernetif = netif->state;
	
	`$INSTANCE_NAME`_PREAMBLE	preamble;
	
	struct pbuf *p, *q;
	u16_t len;
	
	if (`$INSTANCE_NAME`_MACGet_NPP_RSV(&preamble) == FALSE) return NULL;
	
	if (`$INSTANCE_NAME`_rxStatReset)
	{
		`$INSTANCE_NAME`_rxUsed			= 0;
		`$INSTANCE_NAME`_rxTotal		= `$INSTANCE_NAME`_RXSIZE;
		`$INSTANCE_NAME`_rxPktCnt		= 0;
		`$INSTANCE_NAME`_rxStatReset	= false;
	}
	
  	// get packet size and put it into len variable
	len = preamble.StatusVector.bits.ByteCount;
	
	// if nothing/empty then return NULL to caller
	if (len == 0) return NULL;

	rxUsed = `$INSTANCE_NAME`_RXSIZE - `$INSTANCE_NAME`_MACGetFreeRxSize();
	if (rxUsed > `$INSTANCE_NAME`_rxUsed) `$INSTANCE_NAME`_rxUsed = rxUsed;
	`$INSTANCE_NAME`_rxPktCnt++;

#if ETH_PAD_SIZE
	len += ETH_PAD_SIZE; /* allow room for Ethernet padding */
#endif

	// allocate a pbuf chain of pbufs from the pool
  	p = pbuf_alloc(PBUF_RAW, len, PBUF_POOL);
	
	// if we have a chain fill it
	if (p != NULL)
	{
#if ETH_PAD_SIZE
	    pbuf_header(p, -ETH_PAD_SIZE); /* drop the padding word */
#endif

    	// iterate over the pbuf chain until we have read the entire
		// packet into the pbuf.
		for (q = p; q != NULL; q = q->next)
		{
			// Read enough bytes to fill this pbuf in the chain. The
			// available data in the pbuf is given by the q->len
			// variable.
			// This does not necessarily have to be a memcpy, you can also preallocate
			// pbufs for a DMA-enabled MAC and after receiving truncate it to the
			// actually received size. In this case, ensure the tot_len member of the
			// pbuf is the sum of the chained pbuf len members.

			`$INSTANCE_NAME`_MACGetArray(q->payload, q->len);
		}
		
		`$INSTANCE_NAME`_MACDiscardRx();
		
#if ETH_PAD_SIZE
		pbuf_header(p, ETH_PAD_SIZE); /* reclaim the padding word */
#endif
		LINK_STATS_INC(link.recv);
	}
	
	else
	{
		`$INSTANCE_NAME`_MACDiscardRx();
		LINK_STATS_INC(link.memerr);
		LINK_STATS_INC(link.drop);
	}
	
	return p;  
}

//---------------------------------------------------------------------
//
//	`$INSTANCE_NAME`_lwip_input() - read a packet
//
//	This function should be called when a packet is ready to be read
//	from the interface. It uses the function low_level_input() that
//	should handle the actual reception of bytes from the network
//	interface. Then the type of the received packet is determined and
//	the appropriate input function is called.
//
//	@param netif the lwip network interface structure for this ethernetif
//
//---------------------------------------------------------------------

void `$INSTANCE_NAME`_lwip_input(struct netif *netif)
{
	// private data - not used
//	struct ethernetif *ethernetif = netif->state;

	struct eth_hdr *ethhdr;
	struct pbuf *p;

  	/* move received packet into a new pbuf */
  	p = low_level_input(netif);
	
  	/* no packet could be read, silently ignore this */
  	if (p == NULL) return;
	
	/* points to packet payload, which starts with an Ethernet header */
	ethhdr = p->payload;

	switch (htons(ethhdr->type))
	{
	/* IP or ARP packet? */
	case ETHTYPE_IP:
	case ETHTYPE_ARP:
#if PPPOE_SUPPORT
	/* PPPoE packet? */
	case ETHTYPE_PPPOEDISC:
	case ETHTYPE_PPPOE:
#endif /* PPPOE_SUPPORT */
    	/* full packet send to tcpip_thread to process */
    	if (netif->input(p, netif) != ERR_OK)
		{
			LWIP_DEBUGF(NETIF_DEBUG, ("ethernetif_input: IP input error\n"));
			pbuf_free(p);
			p = NULL;
		}
		break;

	default:
		pbuf_free(p);
		p = NULL;
		break;
  	}
}

//---------------------------------------------------------------------
//
//	`$INSTANCE_NAME`_lwip_init() - init the interface
//
//	Should be called at the beginning of the program to set up the
//	network interface. It calls the function low_level_init() to do the
//	actual setup of the hardware.
//
//	This function should be passed as a parameter to netif_add().
//
//	@param netif the lwip network interface structure for this ethernetif
//	@return ERR_OK if the loopif is initialized
//		ERR_MEM if private data couldn't be allocated
//		any other err_t on error
//
//---------------------------------------------------------------------

err_t `$INSTANCE_NAME`_lwip_init(struct netif *netif)
{
	// private data - not used
//	struct ethernetif *ethernetif = netif->state;

	LWIP_ASSERT("netif != NULL", (netif != NULL));
    
//	ethernetif = mem_malloc(sizeof(struct ethernetif));
//	if (ethernetif == NULL)
//	{
//		LWIP_DEBUGF(NETIF_DEBUG, ("ethernetif_init: out of memory\n"));
//		return ERR_MEM;
//	}

#if LWIP_NETIF_HOSTNAME
	// Initialize interface hostname
	netif->hostname = "lwip";
#endif /* LWIP_NETIF_HOSTNAME */

	// Initialize the snmp variables and counters inside the struct netif.
	// The last argument should be replaced with your link speed, in units
	// of bits per second.

	NETIF_INIT_SNMP(netif, snmp_ifType_ethernet_csmacd, LINK_SPEED_OF_YOUR_NETIF_IN_BPS);

	netif->state = NULL; //ethernetif;
	netif->name[0] = IFNAME0;
	netif->name[1] = IFNAME1;
	
	// We directly use etharp_output() here to save a function call.
	// You can instead declare your own function an call etharp_output()
	// from it if you have to do some checks before sending (e.g. if link
	// is available...)
	
	netif->output = etharp_output;
	netif->linkoutput = low_level_output;
  
	// initialize the hardware
	low_level_init(netif);

	return ERR_OK;
}

#endif	// `$INSTANCE_NAME`_PARAM_lwIP_Driver
