//=====================================================================
//
//	`$INSTANCE_NAME`_support.c - simple protocol support routines
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#include <project.h>

uint16	rxUsed  = 0;
uint16	rxTotal = 0;

//---------------------------------------------------------------------
//
//	void ARPReply()
//
//---------------------------------------------------------------------

void `$INSTANCE_NAME`_ARPReply(`$INSTANCE_NAME`_PREAMBLE *preamble, `$INSTANCE_NAME`_ARP_PACKET *requestPacket)
{
	`$INSTANCE_NAME`_PREAMBLE	replyPreamble;
	`$INSTANCE_NAME`_ARP_PACKET	replyPacket;
	
	`$INSTANCE_NAME`_CPY_MAC(replyPreamble.DestMACAddr, preamble->SourceMACAddr);
	`$INSTANCE_NAME`_CPY_MAC(replyPreamble.SourceMACAddr, `$INSTANCE_NAME`_mac_addr);
	replyPreamble.Type.Val = `$INSTANCE_NAME`_SWAP_WORD(`$INSTANCE_NAME`_TYPE_ARP);
	
	replyPacket.HardwareType	= `$INSTANCE_NAME`_SWAP_WORD(`$INSTANCE_NAME`_HW);			// swapped
	replyPacket.Protocol		= `$INSTANCE_NAME`_SWAP_WORD(`$INSTANCE_NAME`_TYPE_IP);		// swapped
	replyPacket.MACAddrLen		= 6;
	replyPacket.ProtocolLen		= 4;
	replyPacket.Operation		= `$INSTANCE_NAME`_SWAP_WORD(`$INSTANCE_NAME`_ARP_REPLY);	// swapped
	`$INSTANCE_NAME`_CPY_MAC(replyPacket.SenderMACAddr, `$INSTANCE_NAME`_mac_addr);
	`$INSTANCE_NAME`_CPY_IPV4(replyPacket.SenderIPAddr, `$INSTANCE_NAME`_ipv4_addr);
	`$INSTANCE_NAME`_CPY_MAC(replyPacket.TargetMACAddr, requestPacket->SenderMACAddr);
	`$INSTANCE_NAME`_CPY_IPV4(replyPacket.TargetIPAddr, requestPacket->SenderIPAddr);
	
	`$INSTANCE_NAME`_MACPutHeader(&replyPreamble, sizeof(`$INSTANCE_NAME`_ARP_PACKET));
	`$INSTANCE_NAME`_MACPutArray((BYTE *) &replyPacket, sizeof(`$INSTANCE_NAME`_ARP_PACKET));
	
	// send the packet
	`$INSTANCE_NAME`_MACFlush();
}

//---------------------------------------------------------------------
//
//	void ICMPEchoReply()
//
//---------------------------------------------------------------------

void `$INSTANCE_NAME`_ICMPEchoReply(`$INSTANCE_NAME`_PREAMBLE *preamble, `$INSTANCE_NAME`_IP_HEADER *ipHdr, `$INSTANCE_NAME`_ICMP_HEADER *icmpHdr)
{
	`$INSTANCE_NAME`_PREAMBLE		replyPreamble;
	`$INSTANCE_NAME`_IP_HEADER		ipReplyHdr;
	`$INSTANCE_NAME`_ICMP_HEADER	icmpReplyHdr;
	
	`$INSTANCE_NAME`_CPY_MAC(replyPreamble.DestMACAddr  , preamble->SourceMACAddr);
	`$INSTANCE_NAME`_CPY_MAC(replyPreamble.SourceMACAddr, `$INSTANCE_NAME`_mac_addr);
	replyPreamble.Type.Val = `$INSTANCE_NAME`_SWAP_WORD(`$INSTANCE_NAME`_TYPE_IP);
	
	ipReplyHdr.Protocol	= `$INSTANCE_NAME`_IP_ICMP;
	`$INSTANCE_NAME`_CPY_IPV4(ipReplyHdr.SourceIPAddr, `$INSTANCE_NAME`_ipv4_addr);
	`$INSTANCE_NAME`_CPY_IPV4(ipReplyHdr.DestIPAddr  , ipHdr->SourceIPAddr);
	
	icmpReplyHdr.Type			= `$INSTANCE_NAME`_ICMP_REPLY;
	icmpReplyHdr.Code			= icmpHdr->Code;
	icmpReplyHdr.Identifier		= icmpHdr->Identifier;
	icmpReplyHdr.SequenceNumber	= icmpHdr->SequenceNumber;
	icmpReplyHdr.Checksum		= `$INSTANCE_NAME`_CalcIPChecksum(NULL, 0, NULL, 0, (BYTE *) &icmpReplyHdr.Identifier, sizeof(icmpReplyHdr.Identifier) + sizeof(icmpReplyHdr.SequenceNumber));
	
	`$INSTANCE_NAME`_MACPutHeader(&replyPreamble, sizeof(`$INSTANCE_NAME`_IP_HEADER) + sizeof(`$INSTANCE_NAME`_ICMP_HEADER));
	`$INSTANCE_NAME`_PutIPHeader(&ipReplyHdr, sizeof(`$INSTANCE_NAME`_IP_HEADER) + sizeof(`$INSTANCE_NAME`_ICMP_HEADER));
	`$INSTANCE_NAME`_MACPutArray((BYTE *) &icmpReplyHdr, sizeof(`$INSTANCE_NAME`_ICMP_HEADER));
	
	// send the packet
	`$INSTANCE_NAME`_MACFlush();
}

//---------------------------------------------------------------------
//
//	CalcIPChecksum()
//
//---------------------------------------------------------------------

WORD `$INSTANCE_NAME`_CalcIPChecksum(BYTE *ipHdr, WORD ipCnt, BYTE *protHdr, WORD protCnt, BYTE *bfr, WORD count)
{
	WORD	i;
	WORD	*val;
	DWORD_VAL	sum = {0x00000000ul};
	
	if (ipCnt)
	{
		i	= ipCnt >> 1;
		val	= (WORD *) ipHdr;
	
		// calc sum of all words
		while (i--) sum.Val += (DWORD) *val++;
	}
	
	if (protCnt)
	{
		i	= protCnt >> 1;
		val	= (WORD *) protHdr;
	
		// calc sum of all words
		while (i--) sum.Val += (DWORD) *val++;
	}
	
	i	= count >> 1;
	val	= (WORD *) bfr;
	
	// calc sum of all words
	while (i--) sum.Val += (DWORD) *val++;
	
	// add in the sum of the remaining byte if present
	if (((WORD_VAL *) &count)->bits.b0) sum.Val += (DWORD) *(BYTE*) val;
	
	// do an end around carry (1's complement math
	sum.Val = (DWORD) sum.w[0] + (DWORD) sum.w[1];
	
	// do again in case of carry out
	sum.w[0] += sum.w[1];
	
	// return the resulting checksum
	return ~sum.w[0];
}

//---------------------------------------------------------------------
//
//	void PutIPHeader()
//
//---------------------------------------------------------------------

void `$INSTANCE_NAME`_PutIPHeader(`$INSTANCE_NAME`_IP_HEADER *ipHdr, WORD ipLength)
{
	ipHdr->Ver_Flags	= 0x45;
	ipHdr->TOS			= 0x00;
	ipHdr->IP_Len		= `$INSTANCE_NAME`_SWAP_WORD(ipLength);
	ipHdr->ID			= 0x0000;
	ipHdr->Flags_Frag	= 0x0000;
	ipHdr->TTL			= 0xff;
	ipHdr->Checksum		= `$INSTANCE_NAME`_CalcIPChecksum((BYTE *) ipHdr, sizeof(`$INSTANCE_NAME`_IP_HEADER), NULL, 0, NULL, 0);
	
	`$INSTANCE_NAME`_MACPutArray((BYTE *) ipHdr, sizeof(`$INSTANCE_NAME`_IP_HEADER));
}

