//=====================================================================
//
//	`$INSTANCE_NAME`_support.h - support routines 'h'
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================


#ifndef `$INSTANCE_NAME`_SUPPORT_H
#define `$INSTANCE_NAME`_SUPPORT_H
	
#include "j1Types.h"

#include "`$INSTANCE_NAME`.h"

typedef	struct { BYTE v[6]; } `$INSTANCE_NAME`_MAC_ADDR;
typedef	DWORD	PTR_BASE;
#define	swaps(x)	(((uint16) (x) >> 8) | ((uint16) (x) << 8))

#define	`$INSTANCE_NAME`_ETHER_HEADER_SIZE	(sizeof(`$INSTANCE_NAME`_MAC_ADDR) + sizeof(`$INSTANCE_NAME`_MAC_ADDR) + sizeof(WORD))

#define	`$INSTANCE_NAME`_MAC_IP			(0x00)
#define	`$INSTANCE_NAME`_MAC_ARP		(0x06)
#define	`$INSTANCE_NAME`_MAC_UNKNOWN	(0xff)
#define `$INSTANCE_NAME`_ETHER_IP		((WORD)0x00)
#define `$INSTANCE_NAME`_ETHER_ARP		((WORD)0x06)

#define `$INSTANCE_NAME`_RAM_SIZE			(24*1024ul)
#define	`$INSTANCE_NAME`_TX_BFR_SIZE		( 2*1024ul)
#define	`$INSTANCE_NAME`_TXSTART			(0x0000ul)
#define	`$INSTANCE_NAME`_RXSTART			((`$INSTANCE_NAME`_TXSTART + 1518ul + `$INSTANCE_NAME`_TX_BFR_SIZE + 1ul) & 0xfffe)
#define	`$INSTANCE_NAME`_RXSTOP				(`$INSTANCE_NAME`_RAM_SIZE - 1ul)
#define	`$INSTANCE_NAME`_RXSIZE				(`$INSTANCE_NAME`_RXSTOP - `$INSTANCE_NAME`_RXSTART + 1ul)

// Receive Status Vector bit fields
typedef union __attribute__((aligned(2), packed)) {
	BYTE v[6];
	struct {
		WORD	 		ByteCount;
		
		unsigned char	PreviouslyIgnored:1;
		unsigned char	RXDCPreviouslySeen:1;
		unsigned char	CarrierPreviouslySeen:1;
		unsigned char	CodeViolation:1;
		unsigned char	CRCError:1;
		unsigned char	LengthCheckError:1;
		unsigned char	LengthOutOfRange:1;
		unsigned char	ReceiveOk:1;
		unsigned char	Multicast:1;
		unsigned char	Broadcast:1;
		unsigned char	DribbleNibble:1;
		unsigned char	ControlFrame:1;
		unsigned char	PauseControlFrame:1;
		unsigned char	UnsupportedOpcode:1;
		unsigned char	VLANType:1;
		unsigned char	RuntMatch:1;

		unsigned char	filler:1;
		unsigned char	HashMatch:1;
		unsigned char	MagicPacketMatch:1;
		unsigned char	PatternMatch:1;
		unsigned char	UnicastMatch:1;
		unsigned char	BroadcastMatch:1;
		unsigned char	MulticastMatch:1;
		unsigned char	ZeroH:1;
		unsigned char	Zero:8;
	} bits;
} `$INSTANCE_NAME`_RXSTATUS;


// A header appended at the start of all RX frames by the hardware
typedef struct  __attribute__((aligned(2), packed))
{
    WORD			NextPacketPointer;
    `$INSTANCE_NAME`_RXSTATUS		StatusVector;

    `$INSTANCE_NAME`_MAC_ADDR        DestMACAddr;
    `$INSTANCE_NAME`_MAC_ADDR        SourceMACAddr;
    WORD_VAL        Type;
} `$INSTANCE_NAME`_PREAMBLE;


void		`$INSTANCE_NAME`_CyBtldrCommStart();
cystatus	`$INSTANCE_NAME`_CyBtldrCommWrite(uint8* buffer, uint16 size, uint16* count, uint8 timeOut);
cystatus	`$INSTANCE_NAME`_CyBtldrCommRead(uint8* buffer, uint16 size, uint16* count, uint8 timeOut);

//---------------------------------------------------------------------
//
//	simple protocol definitions
//
//---------------------------------------------------------------------

// hardware type
#define	`$INSTANCE_NAME`_HW				0x0001

// protocol type
#define	`$INSTANCE_NAME`_TYPE_IP		((WORD) 0x0800)
#define	`$INSTANCE_NAME`_TYPE_ARP		((WORD) 0x0806)

// ip contents
#define	`$INSTANCE_NAME`_IP_ICMP		((BYTE) 1)
#define	`$INSTANCE_NAME`_IP_TCP			((BYTE) 6)
#define	`$INSTANCE_NAME`_IP_UDP			((BYTE) 17)

// arp contents
#define	`$INSTANCE_NAME`_ARP_REQUEST	((WORD) 0x0001)
#define	`$INSTANCE_NAME`_ARP_REPLY		((WORD) 0x0002)

// icmp contents
#define	`$INSTANCE_NAME`_ICMP_ECHO		((BYTE) 0x08)	// ping
#define	`$INSTANCE_NAME`_ICMP_REPLY		((BYTE) 0x00)	// ping reply

typedef struct __attribute__((__packed__))
{
	union
	{
		BYTE	octets[4];
		DWORD	dwIP;
	};
} `$INSTANCE_NAME`_IPV4;

#define	`$INSTANCE_NAME`_IP_PORT	WORD

// utility macros
#define	`$INSTANCE_NAME`_CMP_MAC(x,y)	(memcmp(&x, &y, sizeof(`$INSTANCE_NAME`_MAC_ADDR)) == 0)
#define	`$INSTANCE_NAME`_CMP_IPV4(x,y)	(x.dwIP == y.dwIP)
#define	`$INSTANCE_NAME`_CPY_MAC(x,y)	memcpy(&x, &y, sizeof(`$INSTANCE_NAME`_MAC_ADDR))
#define	`$INSTANCE_NAME`_CPY_IPV4(x,y)	x.dwIP = y.dwIP
#define	`$INSTANCE_NAME`_SWAP_WORD(x)	((x >> 8) | ((x & 0xff) << 8))

// ip_header
typedef struct __attribute__((aligned(2), packed)) _`$INSTANCE_NAME`_IP_HEADER
{
	BYTE	Ver_Flags;
	BYTE	TOS;
	WORD	IP_Len;
	WORD	ID;
	WORD	Flags_Frag;
	BYTE	TTL;
	BYTE	Protocol;
	WORD	Checksum;
	`$INSTANCE_NAME`_IPV4	SourceIPAddr;
	`$INSTANCE_NAME`_IPV4	DestIPAddr;
} `$INSTANCE_NAME`_IP_HEADER;

// udp_header
typedef struct __attribute__((aligned(2), packed)) _`$INSTANCE_NAME`_UDP_HEADER
{
	WORD	Source_Port;
	WORD	Dest_Port;
	WORD	UDP_Len;
	WORD	UDP_Checksum;
} `$INSTANCE_NAME`_UDP_HEADER;

// icmp header
typedef struct __attribute__((aligned(2), packed)) `$INSTANCE_NAME`_ICMP_HEADER
{
	BYTE	Type;
	BYTE	Code;
	WORD	Checksum;
	WORD	Identifier;
	WORD	SequenceNumber;
} `$INSTANCE_NAME`_ICMP_HEADER;

// arp packet
typedef struct __attribute__((aligned(2), packed)) `$INSTANCE_NAME`_ARP_PACKET
{
	WORD	HardwareType;
	WORD	Protocol;
	BYTE	MACAddrLen;
	BYTE	ProtocolLen;
	WORD	Operation;
	`$INSTANCE_NAME`_MAC_ADDR	SenderMACAddr;
	`$INSTANCE_NAME`_IPV4		SenderIPAddr;
	`$INSTANCE_NAME`_MAC_ADDR	TargetMACAddr;
	`$INSTANCE_NAME`_IPV4		TargetIPAddr;
} `$INSTANCE_NAME`_ARP_PACKET;

//---------------------------------------------------------------------
//	Global Data
//---------------------------------------------------------------------

j1GLOBALQUALIFY	`$INSTANCE_NAME`_MAC_ADDR	`$INSTANCE_NAME`_mac_addr;
j1GLOBALQUALIFY	`$INSTANCE_NAME`_IPV4		`$INSTANCE_NAME`_ipv4_addr;
j1GLOBALQUALIFY	uint16						`$INSTANCE_NAME`_ht_filters[4];

j1GLOBALQUALIFY	uint16						`$INSTANCE_NAME`_rxUsed;
j1GLOBALQUALIFY	uint16						`$INSTANCE_NAME`_rxTotal;
j1GLOBALQUALIFY	uint32						`$INSTANCE_NAME`_rxPktCnt;
j1GLOBALQUALIFY	bool						`$INSTANCE_NAME`_rxStatReset;

//---------------------------------------------------------------------
//	Function Prototypes
//---------------------------------------------------------------------

void `$INSTANCE_NAME`_ARPReply(`$INSTANCE_NAME`_PREAMBLE *preamble, `$INSTANCE_NAME`_ARP_PACKET *requestPacket);
void `$INSTANCE_NAME`_ICMPEchoReply(`$INSTANCE_NAME`_PREAMBLE *preamble, `$INSTANCE_NAME`_IP_HEADER *ipHdr, `$INSTANCE_NAME`_ICMP_HEADER *icmpHdr);
WORD `$INSTANCE_NAME`_CalcIPChecksum(BYTE *ipHdr, WORD ipCnt, BYTE *protHdr, WORD protCnt, BYTE *bfr, WORD count);
void `$INSTANCE_NAME`_PutIPHeader(`$INSTANCE_NAME`_IP_HEADER *ipHdr, WORD ipLength);

#endif	// `$INSTANCE_NAME`_SUPPORT_H
