//=====================================================================
//
//	`$INSTANCE_NAME`.c - j1sys fat32 lib component - read-only currently
//
//	version 1.0 - 16 dec 2013
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#include <project.h>

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_Start() - init the control
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_Start(`$j1LIB_NAME`_CTRL *f32C, bool (*readSector)(void *deviceCtrl, uint32 lba, uint8 *secBfr), void *deviceCtrl)
{
	`$j1LIB_NAME`_SEC_BFR	secBfr;
	uint32	lba;
	uint8	part;
	`$j1LIB_NAME`_PART_TABLE_ENTRY	*partEntryPtr;
	
	if (f32C == NULL) return false;
	
	memset(f32C, 0, sizeof(`$j1LIB_NAME`_CTRL));
	
	f32C->readSector	= readSector;
	f32C->deviceCtrl	= deviceCtrl;
	
	if (readSector(deviceCtrl, 0, secBfr.bfr.bytes))
	{
		if (secBfr.mbr.signature == 0xAA55)
		{
			for (part = 0, partEntryPtr = secBfr.mbr.partEntries; part < 4; part++, partEntryPtr++)
			{
				if (partEntryPtr->partType == 0x0b || partEntryPtr->partType == 0x0c)
				 if ((lba = partEntryPtr->partStartLBA) != 0)
				{
					if (readSector(deviceCtrl, lba, secBfr.bfr.bytes))
					{
						if (secBfr.fat32.signature == 0xAA55)
						 if (secBfr.fat32.extSig == 0x29)
						  if (memcmp(secBfr.fat32.fatName, "FAT32", 5) == 0)
						{
							// got it, fill in the fields
							f32C->volumeMounted		= true;
							f32C->secsPerCluster	= secBfr.fat32.secsPerCluster;
							f32C->partStartLBA		= lba;
							f32C->fatStartLBA		= lba + secBfr.fat32.resvdSecs;
							f32C->clusterStartLBA	= lba + secBfr.fat32.resvdSecs + (secBfr.fat32.fatCopies * secBfr.fat32.secsPerFAT);
							f32C->rootStartCluster	= secBfr.fat32.rootStartCluster;
							return true;
						}
					}
				}
			}
		}
	}
	
	return false;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_End() - clear the control
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_End(`$j1LIB_NAME`_CTRL *f32C)
{
	if (f32C == NULL) return false;
	
	memset(f32C, 0, sizeof(`$j1LIB_NAME`_CTRL));
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_GetVolumeID()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_GetVolumeID(`$j1LIB_NAME`_CTRL *f32C, char *volumeIDBfr, uint8 bfrLen)
{
	`$j1LIB_NAME`_DIR_ENTRY	dirEntry;

	if (f32C == NULL || volumeIDBfr == NULL || bfrLen < 13) return false;
	
	memset(volumeIDBfr, 0, bfrLen);
	
	if (`$j1LIB_NAME`_OpenDir(f32C, NULL))
	{
		if (j1FAT32_ReadNextDirEntry(f32C, &dirEntry, volumeIDBfr, bfrLen, NULL, j1FAT32_DIR_ATTRIB_VOLUME_ID, NULL))
		{
			return true;
		}
	}
		
	return false;
}

//---------------------------------------------------------------------
//	j1FAT32_OpenDir()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_OpenDir(`$j1LIB_NAME`_CTRL *f32C, char *dirPath)
{
	uint16						pathLen;
	bool						openRoot;
	bool						result;
	char						*sepPtr;
	uint16						dirLen;
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
	char						dirBfr[255];
	char						lfnBfr[255];
#else
	char						dirBfr[13];
	char						lfnBfr[13];
#endif
	`$j1LIB_NAME`_DIR_ENTRY		dirEntry;
	uint32						cluster;
	
	pathLen 	= 0;
	openRoot	= false;

	if (f32C == NULL) return false;
	
	if (dirPath == NULL) openRoot = true;
	else
	{
		pathLen = strlen(dirPath);
		
		switch (pathLen)
		{
		case 0:
			openRoot = true;
			break;
			
		case 1:
			if (*dirPath == '\\')
			{
				openRoot	= true;
				pathLen		= 0;
			}
			break;
			
		default:
			if (*dirPath == '\\')
			{
				openRoot	= true;
				dirPath++;
				pathLen--;
			}
			else if (f32C->cwdStartCluster == 0)
			{
				openRoot	= true;
			}
		}
	}
		
	if (openRoot)
	{
		result = `$j1LIB_NAME`_OpenDirCluster(f32C, f32C->rootStartCluster);
		if (pathLen == 0) return result;
	}
	
	while (pathLen)
	{
		sepPtr = strchr(dirPath, '\\');
		
		if (sepPtr)
		{
			dirLen = sepPtr - dirPath;
			
			if (dirLen == 0 || dirLen >= sizeof(dirBfr)) return false;
			
			memcpy(dirBfr, dirPath, dirLen);
			dirBfr[dirLen] = 0;
			dirPath += dirLen + 1;
			pathLen -= dirLen + 1;
		}
		else
		{
			if (pathLen >= sizeof(dirBfr)) return false;
			
			strcpy(dirBfr, dirPath);
			pathLen = 0;
		}
		
		result = `$j1LIB_NAME`_ReadNextDirEntry(f32C, &dirEntry, lfnBfr, sizeof(lfnBfr), dirBfr, `$j1LIB_NAME`_DIR_ATTRIB_DIRECTORY, NULL);

		if (!result) return false;
		
		cluster = (((uint32) dirEntry.normal.firstClusterHi) << 16) + dirEntry.normal.firstClusterLo;
		
		result = j1FAT32_OpenDirCluster(f32C, cluster);
		
		if (!result) return false;
	}
	
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_OpenDirCluster()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_OpenDirCluster(`$j1LIB_NAME`_CTRL *f32C, uint32 cluster)
{
	if (f32C == NULL || !f32C->volumeMounted || cluster < 2) return false;

	f32C->cwdStartCluster	= cluster;
	f32C->cwdCurrCluster	= cluster;
	f32C->cwdCurrLBA		= `$j1LIB_NAME`_GetClusterStartLBA(f32C, cluster);
	f32C->cwdEntryInSec		= 0;
	f32C->cwdSecInCluster	= 0;
	f32C->cwdEndNotReached	= true;
	
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_ReadNextDirEntry()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_ReadNextDirEntry(`$j1LIB_NAME`_CTRL *f32C, `$j1LIB_NAME`_DIR_ENTRY *dirEntryBfr, char *lfnBfr, uint8 lfnBfrLen, char *match, uint8 matchAttrib, char *extMatch)
{
	`$j1LIB_NAME`_SEC_BFR	secBfr;
	`$j1LIB_NAME`_DIR_ENTRY	*entryPtr;
	uint8					entryInSec;
	uint8					entryState;
	bool					success;
	char					nameBfr[13];
	char					*namePtr;
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
	uint8					lfnSeq;
	char					*lfnBfrPtr;
	uint8					lfnIdx;
	uint16					lfnUni;
	uint8					lfnAsc;
	bool					lfnFilled;
	uint8					lfnChk;
#endif

	if (f32C == NULL || !f32C->volumeMounted || dirEntryBfr == NULL || !f32C->cwdEndNotReached) return false;
	
	success		= false;
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
	lfnSeq		= 0;
	lfnFilled	= false;
	lfnChk		= 0;
#endif
	
	if (lfnBfr) if (lfnBfrLen < 13) return false;

	while (f32C->cwdEndNotReached)
	{
		if (!f32C->readSector(f32C->deviceCtrl, f32C->cwdCurrLBA, secBfr.bfr.bytes))
		{
			f32C->cwdEndNotReached = false;
			return false;
		}
		
		entryInSec	= f32C->cwdEntryInSec;
		for (entryPtr = &secBfr.dir.dirEntries[entryInSec];
		     entryInSec < (sizeof(secBfr.dir.dirEntries) / sizeof(secBfr.dir.dirEntries[0]));
			 entryInSec++, entryPtr++)

		{
			entryState = entryPtr->lfn.seq;
			
			if (entryState == 0x00)
			{
				f32C->cwdEndNotReached = false;
				return false;
			}
			
			if (entryState == 0xe5) continue;

			if ((entryPtr->normal.attrib & `$j1LIB_NAME`_DIR_ATTRIB_LFN) == `$j1LIB_NAME`_DIR_ATTRIB_LFN)
			{
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
				if (lfnBfr == NULL) continue;
				
				if (entryState & 0x40)
				{
					memset(lfnBfr, 0, lfnBfrLen);
					lfnSeq 		= entryState & 0x3f;
					lfnFilled	= true;
					lfnChk		= entryPtr->lfn.checkSum;
					if ((13 * lfnSeq) >= lfnBfrLen) lfnSeq = 0;
				}
				
				if (entryPtr->lfn.checkSum != lfnChk) lfnSeq = 0;
				
				if (lfnSeq == 0)
				{
					memset(lfnBfr, 0, lfnBfrLen);
					lfnFilled	= false;
					continue;
				}
				
				lfnSeq--;
				
				lfnBfrPtr = lfnBfr + (13 * lfnSeq);
				
				for (lfnIdx = 0; lfnIdx < 13; lfnIdx++)
				{
					     if (lfnIdx <  5) lfnUni = entryPtr->lfn.part1[lfnIdx];
					else if (lfnIdx < 11) lfnUni = entryPtr->lfn.part2[lfnIdx -  5];
					else                  lfnUni = entryPtr->lfn.part3[lfnIdx - 11];
					
					if (lfnUni == 0xffff) continue;
					if (lfnUni == 0x0000) continue;
					
					lfnAsc = lfnUni & 0xff;
					
					if (lfnAsc < 0x20) lfnAsc = '_';
					if (lfnAsc > 0x7f) lfnAsc = 0x7f;
					
					*lfnBfrPtr++ = lfnAsc;
				}
#endif

				continue;
			}
			
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
			if (lfnBfr)
			{
				if (lfnFilled)
				{
					// calculate name[] checksum in lfnASC
					lfnAsc = 0;
					for (lfnIdx = 0; lfnIdx < 11; lfnIdx++)
					{
						lfnAsc = (((lfnAsc & 0x01) << 7) | ((lfnAsc & 0xfe) >> 1)) + entryPtr->normal.name[lfnIdx];
					}
				
					if (lfnAsc != lfnChk)
					{
						memset(lfnBfr, 0, lfnBfrLen);
						lfnFilled	= false;
					}
				}
			
				if (!lfnFilled)
				{
					`$j1LIB_NAME`_NameToString(entryPtr, lfnBfr, lfnBfrLen);
				}
			}
#else
			if (lfnBfr)
			{
				`$j1LIB_NAME`_NameToString(entryPtr, lfnBfr, lfnBfrLen);
			}
#endif
			if (matchAttrib == 0xff)
			{
				if (entryPtr->normal.attrib & `$j1LIB_NAME`_DIR_ATTRIB_DIRECTORY)
				{
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
					lfnFilled = false;
#endif
					continue;
				}
			}
			else if (matchAttrib)
			{
				if ((entryPtr->normal.attrib & matchAttrib) == 0)
				{
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
					lfnFilled = false;
#endif
					continue;
				}
			}
			
			if (match)
			{
				if (lfnBfr)
				{
					if (strcasecmp(lfnBfr, match))
					{
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
						lfnFilled = false;
#endif
						continue;
					}
				}
				else
				{
					`$j1LIB_NAME`_NameToString(entryPtr, nameBfr, sizeof(nameBfr));
					if (strcasecmp(nameBfr, match))
					{
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
						lfnFilled = false;
#endif
						continue;
					}
				}
			}
			
			if (extMatch)
			{
				if (lfnBfr)
				{
					namePtr = strrchr(lfnBfr, '.');
					if (namePtr == NULL)
					{
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
						lfnFilled = false;
#endif
						continue;
					}
					
					if (strcasecmp(namePtr, extMatch))
					{
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
						lfnFilled = false;
#endif
						continue;
					}
				}
				else
				{
					`$j1LIB_NAME`_NameToString(entryPtr, nameBfr, sizeof(nameBfr));

					namePtr = strrchr(nameBfr, '.');
					if (namePtr == NULL)
					{
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
						lfnFilled = false;
#endif
						continue;
					}
					
					if (strcasecmp(namePtr, extMatch))
					{
#if `$j1LIB_NAME`_PARAM_j1LFN_ENABLED
						lfnFilled = false;
#endif
						continue;
					}
				}
			}

			memcpy(dirEntryBfr, entryPtr, sizeof(`$j1LIB_NAME`_DIR_ENTRY));
			success = true;
				
			// next entry logic
			if (++entryInSec < (sizeof(secBfr.dir.dirEntries) / sizeof(secBfr.dir.dirEntries[0])))
			{
				f32C->cwdEntryInSec = entryInSec;
				return true;
			}
			break;
		}
		
		// next sector logic
		f32C->cwdEntryInSec = 0;
		
		if (++(f32C->cwdSecInCluster) < f32C->secsPerCluster)
		{
			f32C->cwdCurrLBA++;
		}
		else
		{
			if ((f32C->cwdCurrCluster = `$j1LIB_NAME`_FollowFAT32(f32C, f32C->cwdCurrCluster)) != 0)
			{
				f32C->cwdCurrLBA		= `$j1LIB_NAME`_GetClusterStartLBA(f32C, f32C->cwdCurrCluster);
				f32C->cwdSecInCluster	= 0;
			}
			else
			{
				f32C->cwdEndNotReached = false;
			}
		}
		
		if (success) return true;
	}
	
	return false;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_RewindDir()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_RewindDir(`$j1LIB_NAME`_CTRL *f32C)
{
	return `$j1LIB_NAME`_OpenDirCluster(f32C, f32C->cwdStartCluster);
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_CloseDir()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_CloseDir(`$j1LIB_NAME`_CTRL *f32C)
{
	f32C->cwdStartCluster	= 0;
	f32C->cwdCurrCluster	= 0;
	f32C->cwdCurrLBA		= 0;
	f32C->cwdEntryInSec		= 0;
	f32C->cwdSecInCluster	= 0;
	f32C->cwdEndNotReached	= false;
	
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_OpenFile()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_OpenFile(`$j1LIB_NAME`_CTRL *f32C, char *fileSpec, `$j1LIB_NAME`_FCB *fcb)
{
	return false;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_OpenFileCluster()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_OpenFileCluster(`$j1LIB_NAME`_CTRL *f32C, uint32 cluster, uint32 fileSize, `$j1LIB_NAME`_FCB *fcb)
{
	fcb->f32C			= f32C;
	fcb->startCluster	= cluster;
	fcb->fileSize		= fileSize;
	fcb->currCluster	= cluster;
	fcb->currLBA		= `$j1LIB_NAME`_GetClusterStartLBA(f32C, cluster);
	fcb->currRecAddr	= 0;
	fcb->secInCluster	= 0;
	
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_SeekFile()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_SeekFile(`$j1LIB_NAME`_FCB *fcb, uint32 recAddr)
{
	return false;
}

//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_ReadFile()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_ReadFile(`$j1LIB_NAME`_FCB *fcb, uint8 *bfrPtr, uint16 bfrSiz, uint16 *bfrRead)
{
	`$j1LIB_NAME`_CTRL	*f32C;
	uint32				nextCluster;
	
	if (bfrSiz != 0x200) return false;
	if (fcb->currRecAddr >= fcb->fileSize) return false;
	
	f32C = fcb->f32C;
	
	if (fcb->secInCluster == f32C->secsPerCluster)
	{
		nextCluster = `$j1LIB_NAME`_FollowFAT32(f32C, fcb->currCluster);
		if (nextCluster == 0) return false;
		
		fcb->currCluster	= nextCluster;
		fcb->currLBA		= `$j1LIB_NAME`_GetClusterStartLBA(f32C, nextCluster);
		fcb->secInCluster	= 0;
	}
	

	if (!f32C->readSector(f32C->deviceCtrl, fcb->currLBA, bfrPtr)) return false;
	
	fcb->currRecAddr += 0x200;
	fcb->currLBA++;
	fcb->secInCluster++;
	
	if (fcb->currRecAddr > fcb->fileSize)
	{
		*bfrRead = fcb->fileSize - (fcb->currRecAddr - 0x200);
	}
	else *bfrRead = bfrSiz;
	
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_RewindFile()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_RewindFile(`$j1LIB_NAME`_FCB *fcb)
{
	`$j1LIB_NAME`_CTRL	*f32C;
	uint32				cluster;
	
	f32C	= fcb->f32C;
	cluster = fcb->startCluster;
	
	fcb->currCluster	= cluster;
	fcb->currLBA		= `$j1LIB_NAME`_GetClusterStartLBA(f32C, cluster);
	fcb->currRecAddr	= 0;
	fcb->secInCluster	= 0;
	
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_CloseFile()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_CloseFile(`$j1LIB_NAME`_FCB *fcb)
{
	memset(fcb, 0, sizeof(`$j1LIB_NAME`_FCB));
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_FollowFAT32()
//---------------------------------------------------------------------

uint32 `$j1LIB_NAME`_FollowFAT32(`$j1LIB_NAME`_CTRL *f32C, uint32 cluster)
{
	`$j1LIB_NAME`_SEC_BFR	secBfr;
	
	if (f32C->readSector)
	  if (cluster)
	    if (f32C->readSector(f32C->deviceCtrl, (cluster >> 7) + f32C->fatStartLBA, secBfr.bfr.bytes))
	{
		return secBfr.fat.fatNextCluster[cluster & 127];
	}

	return 0;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_GetClusterStartLBA()
//---------------------------------------------------------------------

uint32 `$j1LIB_NAME`_GetClusterStartLBA(`$j1LIB_NAME`_CTRL *f32C, uint32 cluster)
{
	uint32	lba;
	
	lba = f32C->clusterStartLBA + (f32C->secsPerCluster * (cluster - 2));
	
	return lba;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_ReadDeviceSector() - read a device sector
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_ReadDeviceSector(`$j1LIB_NAME`_CTRL *f32C, uint32 lba, uint8 *secBfr)
{
	if (f32C->readSector)
	{
		return f32C->readSector(f32C->deviceCtrl, lba, secBfr);
	}
	
	return false;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_ReadVolumeSector() - read a sector in the volume
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_ReadVolumeSector(`$j1LIB_NAME`_CTRL *f32C, uint32 lba, uint8 *secBfr)
{
	if (f32C->volumeMounted)
	{
		return f32C->readSector(f32C->deviceCtrl, lba + f32C->partStartLBA, secBfr);
	}
	
	return false;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_NameToString() - convert dir entry name to a string
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_NameToString(`$j1LIB_NAME`_DIR_ENTRY *dirEntry, char *nameStringBfr, uint8 nameStringBfrLen)
{
	char	*bfrPtr;
	uint8	i;
	
	
	if (nameStringBfr == NULL || nameStringBfrLen < 13) return false;
	
	memset(nameStringBfr, 0, nameStringBfrLen);
	
	if (dirEntry->normal.attrib & (`$j1LIB_NAME`_DIR_ATTRIB_VOLUME_ID | `$j1LIB_NAME`_DIR_ATTRIB_DIRECTORY))
	{
		bfrPtr = nameStringBfr;
		for (i = 0; i < 11; i++)
		{
			if (dirEntry->normal.name[i] > ' ') *bfrPtr++ = dirEntry->normal.name[i];
		}
	}
	else
	{
		bfrPtr = nameStringBfr;
		for (i = 0; i < 11; i++)
		{
			if (i == 8) if (dirEntry->normal.name[i] > ' ') *bfrPtr++ = '.';
							
			if (dirEntry->normal.name[i] > ' ') *bfrPtr++ = dirEntry->normal.name[i];
		}
	}
	
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_GetStartCluster() - build startCluster from dir entry
//---------------------------------------------------------------------

uint32 `$j1LIB_NAME`_GetStartCluster(`$j1LIB_NAME`_DIR_ENTRY *dirEntry)
{
	uint32	startCluster;
	
	startCluster   = dirEntry->normal.firstClusterHi;
	startCluster <<= 16;
	startCluster  |= dirEntry->normal.firstClusterLo;
	
	return startCluster;
}
