//=====================================================================
//
//	`$INSTANCE_NAME`.h - j1sys fat32 component - read-only currently
//
//	version 1.0 - 15 mar 2015
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JOSHUA 1 SYSTEMS INC. OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#if !defined(`$INSTANCE_NAME`_H)
#define `$INSTANCE_NAME`_H
	
#include <j1Types.h>

//---------------------------------------------------------------------
//	stdlib includes
//---------------------------------------------------------------------

#include <string.h>
	
//---------------------------------------------------------------------
//	`$j1LIB_NAME`_PARAM - component options exposed as defines
//---------------------------------------------------------------------
	
#define	`$j1LIB_NAME`_PARAM_j1LFN_ENABLED	`$j1LFN_ENABLED`
	
//---------------------------------------------------------------------
//	`$LIB_NAME`_CTRL
//---------------------------------------------------------------------
	
typedef struct _`$j1LIB_NAME`_CTRL
{
	bool	(*readSector)(void *deviceCtrl, uint32 lba, uint8 *secBfr);
	void	*deviceCtrl;
	bool	volumeMounted;
	uint8	secsPerCluster;
	uint32	partStartLBA;
	uint32	fatStartLBA;
	uint32	clusterStartLBA;
	uint32	rootStartCluster;
	uint32	cwdStartCluster;
	uint32	cwdCurrCluster;
	uint32	cwdCurrLBA;
	uint8	cwdEntryInSec;
	uint8	cwdSecInCluster;
	bool	cwdEndNotReached;
	
} `$j1LIB_NAME`_CTRL;

//---------------------------------------------------------------------
//	`$LIB_NAME`_FCB
//---------------------------------------------------------------------

typedef struct _`$j1LIB_NAME`_FCB
{
	`$j1LIB_NAME`_CTRL	*f32C;
	uint32			startCluster;
	uint32			fileSize;
	uint32			currCluster;
	uint32			currLBA;
	uint32			currRecAddr;
	uint8			secInCluster;
	
} `$j1LIB_NAME`_FCB;

//---------------------------------------------------------------------
//	`$LIB_NAME`_SEC_BFR
//---------------------------------------------------------------------

typedef struct __attribute__ ((__packed__))
{
	uint8	partState;		// partition state
	uint8	partStartHead;	// partition start head
	uint16	partStartCyl;	// partition start cylinder
	uint8	partType;		// partition type
	uint8	partEndHead;	// partition end head
	uint16	partEndCyl;		// partition end cylinder
	uint32	partStartLBA;	// partition start LBA
	uint32	partSecCount;	// partition sector count
} `$j1LIB_NAME`_PART_TABLE_ENTRY;

typedef union
{
	struct __attribute__ ((__packed__))
	{
		uint8	name[11];		// name and ext field
		uint8	attrib;			// attributes byte
		uint8	fill1[8];		// fill #1
		uint16	firstClusterHi;	// first cluster high word
		uint16	time;			// file time
		uint16	date;			// file date
		uint16	firstClusterLo;	// first cluster low  word
		uint32	fileSize;		// file size in bytes
	} normal;

	struct __attribute__ ((__packed__))
	{
		uint8	seq;			// sequence number
		uint16	part1[5];		// lfn part 1
		uint8	attrib;			// attributes byte
		uint8	fill1;			// fill #1
		uint8	checkSum;		// check sum
		uint16	part2[6];		// lfn part 2
		uint8	fill2[2];		// fill #2
		uint16	part3[2];		// lfn part 3
	} lfn;
} `$j1LIB_NAME`_DIR_ENTRY;

#define	`$j1LIB_NAME`_DIR_ATTRIB_READ_ONLY	0x01
#define	`$j1LIB_NAME`_DIR_ATTRIB_HIDDEN		0x02
#define	`$j1LIB_NAME`_DIR_ATTRIB_SYSTEM		0x04
#define	`$j1LIB_NAME`_DIR_ATTRIB_VOLUME_ID	0x08
#define	`$j1LIB_NAME`_DIR_ATTRIB_DIRECTORY	0x10
#define	`$j1LIB_NAME`_DIR_ATTRIB_ARCHIVE	0x20
#define	`$j1LIB_NAME`_DIR_ATTRIB_LFN		0x0f	// all low nibble

typedef union
{
	struct __attribute__ ((__packed__))
	{
		uint8	bytes[512];			// the buffer
	} bfr;
	
	struct __attribute__ ((__packed__))
	{
		uint8	execCode[446];		// executable area

		`$j1LIB_NAME`_PART_TABLE_ENTRY	partEntries[4];
		
		uint16	signature;			// signature
	} mbr;
	
	struct __attribute__ ((__packed__))
	{
		uint8	jumpCode[3];		// jump code + NOP
		uint8	oemName[8];			// oem name
		uint16	bytesPerSec;		// bytes per sector
		uint8	secsPerCluster;		// sectors per cluster
		uint16	resvdSecs;			// reserved sectors
		uint8	fatCopies;			// number of fat copies
		uint16	maxRootDirEntries;	// maximum root directory entries
		uint16	_secsInPartition;	// old sectors in partition
		uint8	mediaDesc;			// media descriptor
		uint16	_secsPerFAT;		// old sectors per FAT
		uint16	secsPerTrack;		// sectors per track
		uint16	heads;				// number of heads
		uint32	hiddenSecs;			// hidden sectors in partition
		uint32	secsInPartition;	// sectors in partition
		uint32	secsPerFAT;			// sectors per FAT
		uint16	flagBits;			// misc flag bits
		uint16	fatVer;				// fat version
		uint32	rootStartCluster;	// root directory start cluster
		uint16	fisSec;				// filesytem info sector number
		uint16	backupBootSec;		// backup boot sector number
		uint8	reserved[12];		// reserved area
		uint8	logDrive;			// logical drive number
		uint8	unused;				// unused area
		uint8	extSig;				// extended signature
		uint32	serNo;				// serial number
		uint8	volName[11];		// volume name
		uint8	fatName[8];			// fat name
		uint8	execCode[420];		// executable code area
		uint16	signature;			// signature
	} fat32;

	struct __attribute__ ((__packed__))
	{
		uint32	fatNextCluster[128];	// fat next cluster
	} fat;
	
	struct __attribute__ ((__packed__))
	{
		`$j1LIB_NAME`_DIR_ENTRY	dirEntries[16];
	} dir;
	
} `$j1LIB_NAME`_SEC_BFR;

//---------------------------------------------------------------------
//	function prototypes
//---------------------------------------------------------------------

bool	`$j1LIB_NAME`_Start				(`$j1LIB_NAME`_CTRL *f32C, bool (*readSector)(void *deviceCtrl, uint32 lba, uint8 *secBfr), void *deviceCtrl);
bool	`$j1LIB_NAME`_End				(`$j1LIB_NAME`_CTRL *f32C);

bool	`$j1LIB_NAME`_GetVolumeID		(`$j1LIB_NAME`_CTRL *f32C, char *volumeIDBfr, uint8 bfrLen);

bool	`$j1LIB_NAME`_OpenDir			(`$j1LIB_NAME`_CTRL *f32C, char *dirPath);
bool	`$j1LIB_NAME`_OpenDirCluster	(`$j1LIB_NAME`_CTRL *f32C, uint32 cluster);
bool	`$j1LIB_NAME`_ReadNextDirEntry	(`$j1LIB_NAME`_CTRL *f32C, `$j1LIB_NAME`_DIR_ENTRY *dirEntryBfr, char *lfnBfr, uint8 lfnBfrLen, char *match, uint8 matchAttrib, char *extMatch);
bool	`$j1LIB_NAME`_RewindDir			(`$j1LIB_NAME`_CTRL *f32C);
bool	`$j1LIB_NAME`_CloseDir			(`$j1LIB_NAME`_CTRL *f32C);

bool	`$j1LIB_NAME`_OpenFile			(`$j1LIB_NAME`_CTRL *f32C, char *fileSpec, `$j1LIB_NAME`_FCB *fcb);
bool	`$j1LIB_NAME`_OpenFileCluster	(`$j1LIB_NAME`_CTRL *f32C, uint32 cluster, uint32 fileSize, `$j1LIB_NAME`_FCB *fcb);
bool	`$j1LIB_NAME`_SeekFile			(`$j1LIB_NAME`_FCB *fcb, uint32 recAddr);
bool	`$j1LIB_NAME`_ReadFile			(`$j1LIB_NAME`_FCB *fcb, uint8 *bfrPtr, uint16 bfrSiz, uint16 *bfrRead);
bool	`$j1LIB_NAME`_RewindFile		(`$j1LIB_NAME`_FCB *fcb);
bool	`$j1LIB_NAME`_CloseFile			(`$j1LIB_NAME`_FCB *fcb);

uint32	`$j1LIB_NAME`_FollowFAT32		(`$j1LIB_NAME`_CTRL *f32C, uint32 cluster);
uint32	`$j1LIB_NAME`_GetClusterStartLBA(`$j1LIB_NAME`_CTRL *f32C, uint32 cluster);
bool	`$j1LIB_NAME`_ReadDeviceSector	(`$j1LIB_NAME`_CTRL *f32C, uint32 lba, uint8 *secBfr);
bool	`$j1LIB_NAME`_ReadVolumeSector	(`$j1LIB_NAME`_CTRL *f32C, uint32 lba, uint8 *secBfr);
bool	`$j1LIB_NAME`_NameToString		(`$j1LIB_NAME`_DIR_ENTRY *dirEntry, char *nameStringBfr, uint8 nameStringBfrLen);
uint32	`$j1LIB_NAME`_GetStartCluster	(`$j1LIB_NAME`_DIR_ENTRY *dirEntry);

#endif	// `$INSTANCE_NAME`_H
