//=====================================================================
//
//	`$INSTANCE_NAME`.c - j1sys fs lib component
//
//	version 1.0 - 16 dec 2013
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#include <project.h>

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_Start() - init the control
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_Start(`$j1LIB_NAME`_CTRL *j1FS_Ctrl, bool (*readArray)(void *deviceCtrl, uint32 startAddr, uint8 *readBfr, uint16 readCnt), void *deviceCtrl, uint32 deviceOffset)
{
	`$j1LIB_NAME`_DIR_ENTRY	dirEntry;
	
	memset(j1FS_Ctrl, 0, sizeof(`$j1LIB_NAME`_CTRL));
	
	j1FS_Ctrl->readArray	= readArray;
	j1FS_Ctrl->deviceCtrl	= deviceCtrl;
	j1FS_Ctrl->deviceOffset	= deviceOffset;
	
	`$j1LIB_NAME`_ReadDeviceArray(j1FS_Ctrl, 0, (uint8 *) &dirEntry, sizeof(`$j1LIB_NAME`_DIR_ENTRY));
	
	if (dirEntry.attrib != 16 && dirEntry.attrib != 32 && dirEntry.attrib != 64) return false;
	
	j1FS_Ctrl->bytesPerBlock = dirEntry.attrib;
	j1FS_Ctrl->rootStart     = dirEntry.startBlock;
	j1FS_Ctrl->rootSize      = dirEntry.fileSize;

	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_GetVolumeID()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_GetVolumeID(`$j1LIB_NAME`_CTRL *j1FS_Ctrl, char *volumeIDBfr, uint8 bfrLen)
{
	`$j1LIB_NAME`_DIR_ENTRY	dirEntry;

	`$j1LIB_NAME`_ReadDeviceArray(j1FS_Ctrl, 0, (uint8 *) &dirEntry, sizeof(`$j1LIB_NAME`_DIR_ENTRY));
	return `$j1LIB_NAME`_NameToString(true, &dirEntry, volumeIDBfr, bfrLen);
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_OpenFile()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_OpenFile(`$j1LIB_NAME`_CTRL *j1FS_Ctrl, char *fileSpec, `$j1LIB_NAME`_FCB *fcb)
{
	char	target[13];
	char	entry[13];
	char	*sepPtr;
	uint8	targetLen;
	uint16	read;
	
	`$j1LIB_NAME`_DIR_ENTRY	dirEntry;
	`$j1LIB_NAME`_FCB		dirFCB;
	
	if (*fileSpec++ != '/') return false;
	
	dirFCB.j1FS_Ctrl		= j1FS_Ctrl;
	dirFCB.fileStartAddr	= j1FS_Ctrl->rootStart * j1FS_Ctrl->bytesPerBlock;
	dirFCB.fileSize			= j1FS_Ctrl->rootSize;
	dirFCB.currReadAddr		= 0;
	
	while (true)
	{
		sepPtr = strchr(fileSpec, '/');
		
		if (sepPtr)
		{
			targetLen = sepPtr - fileSpec;
			if (targetLen == 0 || targetLen > 12) return false;
			memcpy(target, fileSpec, targetLen);
			target[targetLen] = 0;
		}
		
		else
		{
			targetLen = strlen(fileSpec);
			if (targetLen == 0 || targetLen > 12) return false;
			strcpy(target, fileSpec);
		}
		
		while (`$j1LIB_NAME`_ReadFile(&dirFCB, (uint8 *) &dirEntry, sizeof(dirEntry), &read))
		{
			if (sepPtr)
			{
				if (dirEntry.attrib != `$j1LIB_NAME`_DIR_ATTRIB_DIRECTORY) return false;
			}
			else
			{
				if (dirEntry.attrib == `$INSTANCE_NAME`_DIR_ATTRIB_DIRECTORY) continue;
			}
			
			if (`$INSTANCE_NAME`_NameToString(false, &dirEntry, entry, sizeof(entry)) == false) return false;
			
			if (strcmp(target, entry) == 0)
			{
				if (sepPtr)
				{
					dirFCB.fileStartAddr	= dirEntry.startBlock * j1FS_Ctrl->bytesPerBlock;
					dirFCB.fileSize			= dirEntry.fileSize;
					dirFCB.currReadAddr		= 0;
					fileSpec = sepPtr + 1;
					targetLen = 0;			// signal success
					break;
				}
				
				else
				{
					fcb->j1FS_Ctrl		= j1FS_Ctrl;
					fcb->fileStartAddr	= dirEntry.startBlock * j1FS_Ctrl->bytesPerBlock;
					fcb->fileSize		= dirEntry.fileSize;
					fcb->currReadAddr	= 0;
					return true;
				}
			}
		}
		
		if (targetLen) return false;
	}

	return false;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_SeekFile()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_SeekFile(`$j1LIB_NAME`_FCB *fcb, uint16 readAddr)
{
	fcb->currReadAddr = readAddr;
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_ReadFile()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_ReadFile(`$j1LIB_NAME`_FCB *fcb, uint8 *readBfr, uint16 readCnt, uint16 *read)
{
	uint16	remaining;
	
	`$j1LIB_NAME`_CTRL	*j1FS_Ctrl;
	
	*read	  = readCnt;
	remaining = fcb->fileSize - fcb->currReadAddr;
	
	if (remaining < readCnt) *read = remaining;
	if (*read == 0) return false;
	
	j1FS_Ctrl = fcb->j1FS_Ctrl;
	
	`$j1LIB_NAME`_ReadDeviceArray(j1FS_Ctrl, fcb->fileStartAddr + fcb->currReadAddr, readBfr, *read);
	
	fcb->currReadAddr += *read;
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_RewindFile()
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_RewindFile(`$j1LIB_NAME`_FCB *fcb)
{
	fcb->currReadAddr = 0;	
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_Remaining()
//---------------------------------------------------------------------

uint16 `$j1LIB_NAME`_Remaining(`$j1LIB_NAME`_FCB *fcb)
{
	uint16	remaining;
	
	remaining = fcb->fileSize - fcb->currReadAddr;
	
	return remaining;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_ReadDeviceArray() - read a device array
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_ReadDeviceArray(`$j1LIB_NAME`_CTRL *j1FS_Ctrl, uint32 startAddress, uint8 *readBfr, uint16 readCnt)
{
	uint32	deviceAddr;
	
	deviceAddr = j1FS_Ctrl->deviceOffset + startAddress;
	
	if (j1FS_Ctrl->readArray)
	{
		return j1FS_Ctrl->readArray(j1FS_Ctrl->deviceCtrl, deviceAddr, readBfr, readCnt);
	}
	
	memcpy(readBfr, (uint8 *) deviceAddr, readCnt);
	return true;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_NameToString() - convert dir entry name to a string
//---------------------------------------------------------------------

bool `$j1LIB_NAME`_NameToString(bool volName, `$j1LIB_NAME`_DIR_ENTRY *dirEntry, char *nameStringBfr, uint8 nameStringBfrLen)
{
	char	*bfrPtr;
	uint8	i;
	
	
	if (nameStringBfr == NULL || nameStringBfrLen < 13) return false;
	
	memset(nameStringBfr, 0, nameStringBfrLen);
	
	if (volName)
	{
		bfrPtr = nameStringBfr;
		for (i = 0; i < 11; i++)
		{
			if (dirEntry->nameExt[i] > ' ') *bfrPtr++ = dirEntry->nameExt[i];
		}
	}
	else
	{
		bfrPtr = nameStringBfr;
		for (i = 0; i < 11; i++)
		{
			if (i == 8) if (dirEntry->nameExt[i] > ' ') *bfrPtr++ = '.';
							
			if (dirEntry->nameExt[i] > ' ') *bfrPtr++ = dirEntry->nameExt[i];
		}
	}
	
	return true;
}
