//=====================================================================
//
//	`$INSTANCE_NAME`.c - j1sys device independent graphics lib
//
//	loosely based on a re-write of the adafruit graphics library
//
//	version 1.1 - 15 feb 2014
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#include <project.h>

#include "`$INSTANCE_NAME`.h"

#include "`$INSTANCE_NAME`_Font5x7.inc"
#include "`$INSTANCE_NAME`_Font8x12.inc"

//#define	j1GFX_ABS(x)			((x)<0) ? -(x) : (x)
//#define	j1GFX_VARMIN(a,b)		((a)<(b)) ? (a) : (b)
#define j1GFX_VARSWAP(a,b,x)	x=a; a=b; b=x

//---------------------------------------------------------------------
//	j1GFX_SetRegion() - set a new region
//---------------------------------------------------------------------

void j1GFX_SetRegion(j1GFX_CTRL *j1GFX_Template, j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, int16 h)
{
	if (j1GFX_Template)
	{
		memcpy(j1GFX_Ctrl, j1GFX_Template, sizeof(j1GFX_CTRL));
	}
	
	if (x < 0 || j1GFX_Ctrl->displayWidth  <= x) return;
	if (y < 0 || j1GFX_Ctrl->displayHeight <= y) return;
	
	if (x + w > j1GFX_Ctrl->displayWidth ) w = j1GFX_Ctrl->displayWidth  - x;
	if (y + h > j1GFX_Ctrl->displayHeight) h = j1GFX_Ctrl->displayHeight - y;
	
	j1GFX_Ctrl->regOffsetX	= x;
	j1GFX_Ctrl->regOffsetY	= y;
	j1GFX_Ctrl->regWidth	= w;
	j1GFX_Ctrl->regHeight	= h;
}

//---------------------------------------------------------------------
//	j1GFX_DrawPixel() - draw a pixel relative to region
//---------------------------------------------------------------------

void j1GFX_DrawPixel(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, uint32 rgb)
{
	j1GFX_HOOKS	*hooks;
	
	if (x < 0 || j1GFX_Ctrl->regWidth  <= x) return;
	if (y < 0 || j1GFX_Ctrl->regHeight <= y) return;
	
	x += j1GFX_Ctrl->regOffsetX;
	y += j1GFX_Ctrl->regOffsetY;
	
	hooks = j1GFX_Ctrl->hardware;
	
	if (hooks->DrawPixel) hooks->DrawPixel(j1GFX_Ctrl, x, y, rgb);
}

//---------------------------------------------------------------------
//	j1GFX_DrawVLine() - draw a vertical line relative to region
//---------------------------------------------------------------------

void j1GFX_DrawVLine(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 h, uint32 rgb)
{
	j1GFX_HOOKS	*hooks;
	
	if (x < 0 || j1GFX_Ctrl->regWidth  <= x) return;
	if (y < 0 || j1GFX_Ctrl->regHeight <= y) return;
	
	if (y + h > j1GFX_Ctrl->regHeight) h = j1GFX_Ctrl->regHeight - y;
	
	x += j1GFX_Ctrl->regOffsetX;
	y += j1GFX_Ctrl->regOffsetY;
	
	hooks = j1GFX_Ctrl->hardware;
	
	if (hooks->DrawVLine) hooks->DrawVLine(j1GFX_Ctrl, x, y, h, rgb);
}

//---------------------------------------------------------------------
//	j1GFX_DrawHLine() - draw a horizontal line relative to region
//---------------------------------------------------------------------

void j1GFX_DrawHLine(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, uint32 rgb)
{
	j1GFX_HOOKS	*hooks;
	
	if (x < 0 || j1GFX_Ctrl->regWidth  <= x) return;
	if (y < 0 || j1GFX_Ctrl->regHeight <= y) return;
	
	if (x + w > j1GFX_Ctrl->regWidth) w = j1GFX_Ctrl->regWidth - x;
	
	x += j1GFX_Ctrl->regOffsetX;
	y += j1GFX_Ctrl->regOffsetY;
	
	hooks = j1GFX_Ctrl->hardware;
	
	if (hooks->DrawHLine) hooks->DrawHLine(j1GFX_Ctrl, x, y, w, rgb);
}

//---------------------------------------------------------------------
//	j1GFX_DrawLine() - draw arbitrary line
//---------------------------------------------------------------------

void j1GFX_DrawLine(j1GFX_CTRL *j1GFX_Ctrl, int16 x0, int16 y0, int16 x1, int16 y1, uint32 rgb)
{
	bool	steep;
	int16	dx, dy, err, ystep;
	
	dx	= x1 - x0; if (dx < 0) dx = -dx;
	dy	= y1 - y0; if (dy < 0) dy = -dy;
	
	if (dy == 0)
	{
		if (x1 < x0) x0 = x1;
		j1GFX_DrawHLine(j1GFX_Ctrl, x0, y0, dx + 1, rgb);
		return;
	}
	
	if (dx == 0)
	{
		if (y1 < y0) y0 = y1;
		j1GFX_DrawVLine(j1GFX_Ctrl, x0, y0, dy + 1, rgb);
		return;
	}
	
	steep = (dy > dx);
	
	if (steep)
	{
		j1GFX_VARSWAP(x0, y0, err);
		j1GFX_VARSWAP(x1, y1, err);
	}
	
	if (x0 > x1)
	{
		j1GFX_VARSWAP(x0, x1, err);
		j1GFX_VARSWAP(y0, y1, err);
	}
	
	dx		= x1 - x0;
	dy		= y1 - y0; if (dy < 0) dy = -dy;
	err		= dx / 2;
	ystep	= (y0 < y1) ? 1 : -1;
	
	while (x0 <= x1)
	{
		if (steep) j1GFX_DrawPixel(j1GFX_Ctrl, y0, x0, rgb);
			  else j1GFX_DrawPixel(j1GFX_Ctrl, x0, y0, rgb);
		
		err -= dy;
		if (err < 0)
		{
			y0	+= ystep;
			err	+= dx;
		}
		
		x0++;
	}
}

//---------------------------------------------------------------------
//	j1GFX_DrawCircle() - draw a circle
//---------------------------------------------------------------------

void j1GFX_DrawCircle(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 r, uint32 rgb)
{
}

//---------------------------------------------------------------------
//	j1GFX_DrawTriangle() - draw a triangle
//---------------------------------------------------------------------

void j1GFX_DrawTriangle(j1GFX_CTRL *j1GFX_Ctrl, int16 x0, int16 y0, int16 x1, int16 y1, int16 x2, int16 y2, uint32 rgb)
{
	j1GFX_DrawLine(j1GFX_Ctrl, x0, y0, x1, y1, rgb);
	j1GFX_DrawLine(j1GFX_Ctrl, x1, y1, x2, y2, rgb);
	j1GFX_DrawLine(j1GFX_Ctrl, x2, y2, x0, y0, rgb);
}

//---------------------------------------------------------------------
//	j1GFX_GetFontDims() - get the width/height of the font
//---------------------------------------------------------------------

void j1GFX_GetFontDims(j1GFX_FONT font, uint8 scale, uint16 *w, uint16 *h)
{
	uint16	ww, hh;
	
	switch (font)
	{
#if j1GFX_PARAM_FONT_5X7 || j1GFX_PARAM_NUM_5X7
	case j1GFX_FONT_5X7:
		ww = 6;
		hh = 8;
		break;
#endif

#if j1GFX_PARAM_FONT_8X12 || j1GFX_PARAM_NUM_8X12
	case j1GFX_FONT_8X12:
		ww =  8;
		hh = 12;
		break;
#endif
		
	default:
		ww = 0;
		hh = 0;
	}
	
	ww *= scale;
	hh *= scale;
	
	*w = ww;
	*h = hh;
}

//---------------------------------------------------------------------
//	j1GFX_DrawFont() - draw a font character relative to region
//---------------------------------------------------------------------

void j1GFX_DrawFont(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, char c, j1GFX_FONT font, uint8 scale, uint32 fg, uint32 bg)
{
	j1GFX_HOOKS	*hooks;
	uint16		w, h;
	
	j1GFX_GetFontDims(font, scale, &w, &h);

	if (x + w > j1GFX_Ctrl->regWidth ) { x = 0; y += h; }
	if (y + h > j1GFX_Ctrl->regHeight) { x = 0; y  = 0; }
	
	x += j1GFX_Ctrl->regOffsetX;
	y += j1GFX_Ctrl->regOffsetY;
	
	hooks = j1GFX_Ctrl->hardware;
	
	if (hooks->DrawFont) hooks->DrawFont(j1GFX_Ctrl, x, y, c, font, w, h, scale, fg, bg);
	
}

//---------------------------------------------------------------------
//	j1GFX_DrawRect() - draw a rectangle relative to region
//---------------------------------------------------------------------

void j1GFX_DrawRect(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, int16 h, uint32 rgb)
{
	j1GFX_HOOKS	*hooks;
	
	if (x < 0 || j1GFX_Ctrl->regWidth  <= x) return;
	if (y < 0 || j1GFX_Ctrl->regHeight <= y) return;
	
	if (x + w > j1GFX_Ctrl->regWidth ) w = j1GFX_Ctrl->regWidth  - x;
	if (y + h > j1GFX_Ctrl->regHeight) h = j1GFX_Ctrl->regHeight - y;
	
	x += j1GFX_Ctrl->regOffsetX;
	y += j1GFX_Ctrl->regOffsetY;
	
	hooks = j1GFX_Ctrl->hardware;
	
	if (hooks->DrawHLine)
	{
		hooks->DrawHLine(j1GFX_Ctrl, x, y, w, rgb);
		hooks->DrawHLine(j1GFX_Ctrl, x, y + h - 1, w, rgb);
	}
	
	if (hooks->DrawVLine)
	{
		hooks->DrawVLine(j1GFX_Ctrl, x, y, h, rgb);
		hooks->DrawVLine(j1GFX_Ctrl, x + w - 1, y, h, rgb);
	}
}

//---------------------------------------------------------------------
//	j1GFX_FillRect() - fill a rectangle relative to region
//---------------------------------------------------------------------

void j1GFX_FillRect(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, int16 h, uint32 rgb)
{
	j1GFX_HOOKS	*hooks;
	
	if (x < 0 || j1GFX_Ctrl->regWidth  <= x) return;
	if (y < 0 || j1GFX_Ctrl->regHeight <= y) return;
	
	if (x + w > j1GFX_Ctrl->regWidth ) w = j1GFX_Ctrl->regWidth  - x;
	if (y + h > j1GFX_Ctrl->regHeight) h = j1GFX_Ctrl->regHeight - y;
	
	x += j1GFX_Ctrl->regOffsetX;
	y += j1GFX_Ctrl->regOffsetY;
	
	hooks = j1GFX_Ctrl->hardware;
	
	if (hooks->FillRect)
	{
		hooks->FillRect(j1GFX_Ctrl, x, y, w, h, rgb);
	}
}

//---------------------------------------------------------------------
//	j1GFX_FillRegion() - fill the region
//---------------------------------------------------------------------

void j1GFX_FillRegion(j1GFX_CTRL *j1GFX_Ctrl, uint32 rgb)
{
	j1GFX_HOOKS	*hooks;
	
	hooks = j1GFX_Ctrl->hardware;
	
	if (hooks->FillRect)
	{
		hooks->FillRect(j1GFX_Ctrl, j1GFX_Ctrl->regOffsetX, j1GFX_Ctrl->regOffsetY, j1GFX_Ctrl->regWidth, j1GFX_Ctrl->regHeight, rgb);
	}
}

//---------------------------------------------------------------------
//	j1GFX_FillDisplay() - fill the display
//---------------------------------------------------------------------

void j1GFX_FillDisplay(j1GFX_CTRL *j1GFX_Ctrl, uint32 rgb)
{
	j1GFX_HOOKS	*hooks;
	
	hooks = j1GFX_Ctrl->hardware;
	
	if (hooks->FillRect)
	{
		hooks->FillRect(j1GFX_Ctrl, 0, 0, j1GFX_Ctrl->displayWidth, j1GFX_Ctrl->displayHeight, rgb);
	}
}

//---------------------------------------------------------------------
//	j1GFX_SetRotation() - set the display rotation
//---------------------------------------------------------------------

void j1GFX_SetRotation(j1GFX_CTRL *j1GFX_Ctrl, j1GFX_ROTATE rotate)
{
	j1GFX_HOOKS	*hooks;
	
	hooks = j1GFX_Ctrl->hardware;
		
	if (hooks->SetRotation) hooks->SetRotation(j1GFX_Ctrl, rotate);
}

//---------------------------------------------------------------------
//	j1GFX_SetFontFgBg() - set the font, fg rgb, bg rgb
//---------------------------------------------------------------------

void j1GFX_SetFontFgBg(j1GFX_CTRL *j1GFX_Ctrl, j1GFX_FONT font, uint8 scale, uint32 fg, uint32 bg)
{
	j1GFX_Ctrl->textFont	= font;
	j1GFX_Ctrl->textScale	= scale;
	j1GFX_GetFontDims(font, scale, &j1GFX_Ctrl->textFontW, &j1GFX_Ctrl->textFontH);
	j1GFX_Ctrl->textFG		= fg;
	j1GFX_Ctrl->textBG		= bg;
}

//---------------------------------------------------------------------
//	j1GFX_SetTextRowCol() - set the row and column for text output
//---------------------------------------------------------------------

void j1GFX_SetTextRowCol(j1GFX_CTRL *j1GFX_Ctrl, uint8 row, uint8 col)
{
	uint16	x, y;
	
	x = j1GFX_Ctrl->textFontW;
	y = j1GFX_Ctrl->textFontH;
	
	x *= col;
	y *= row;
	
	j1GFX_SetTextXY(j1GFX_Ctrl, x, y);
}

//---------------------------------------------------------------------
//	j1GFX_SetTextXY() - set the x and y for text output
//---------------------------------------------------------------------

void j1GFX_SetTextXY(j1GFX_CTRL *j1GFX_Ctrl, uint16 x, uint16 y)
{
	if (j1GFX_Ctrl->regWidth  <= x) x = 0;
	if (j1GFX_Ctrl->regHeight <= y) y = 0;
	
	j1GFX_Ctrl->textX = x;
	j1GFX_Ctrl->textY = y;
}

//---------------------------------------------------------------------
//	j1GFX_PutChar() - put a char using font,fg,bg,row,col and region
//---------------------------------------------------------------------

void j1GFX_PutChar(j1GFX_CTRL *j1GFX_Ctrl, char c)
{
	j1GFX_HOOKS	*hooks;
	uint16		x, y, w, h;
	
	x = j1GFX_Ctrl->textX;
	y = j1GFX_Ctrl->textY;
	
	w = j1GFX_Ctrl->textFontW;
	h = j1GFX_Ctrl->textFontH;
	
	if (x + w > j1GFX_Ctrl->regWidth ) { x = 0; y += h; j1GFX_Ctrl->textX = x; j1GFX_Ctrl->textY = y; }
	if (y + h > j1GFX_Ctrl->regHeight) { x = 0; y  = 0; j1GFX_Ctrl->textX = x; j1GFX_Ctrl->textY = y;}
	
	x += j1GFX_Ctrl->regOffsetX;
	y += j1GFX_Ctrl->regOffsetY;
	
	hooks = j1GFX_Ctrl->hardware;
	
	if (hooks->DrawFont) hooks->DrawFont(j1GFX_Ctrl, x, y, c, j1GFX_Ctrl->textFont, w, h, j1GFX_Ctrl->textScale, j1GFX_Ctrl->textFG, j1GFX_Ctrl->textBG);
	
	x = j1GFX_Ctrl->textX + w;
	y = j1GFX_Ctrl->textY;
	
	if (x + w > j1GFX_Ctrl->regWidth ) { x = 0; y += h; }
	if (y + h > j1GFX_Ctrl->regHeight) { x = 0; y  = 0; }
	
	j1GFX_Ctrl->textX = x;
	j1GFX_Ctrl->textY = y;
}

//---------------------------------------------------------------------
//	j1GFX_PutString() - simple iterative string output through j1GFX_PutChar
//---------------------------------------------------------------------

void j1GFX_PutString(j1GFX_CTRL *j1GFX_Ctrl, char *string)
{
	unsigned char	c;
	
	while ((c = *string++)) j1GFX_PutChar(j1GFX_Ctrl, c);
}

//---------------------------------------------------------------------
//	j1GFX_Text() - with options: fill rectangle, border, center text
//---------------------------------------------------------------------

void j1GFX_Text(j1GFX_CTRL *j1GFX_Ctrl, uint8 opts, int16 x, int16 y, int16 w, int16 h, j1GFX_FONT font, uint8 scale, uint32 fg, uint32 bg, char *text)
{
	uint16	textLen;
	char	*textPtr, *newlinePtr;
	uint16	lineCnt;
	uint16	fontW, fontH;
	uint16	offX = 0, offY = 0;
	
	if (opts & j1GFX_SDL_TEXT_OPTS_FILL)  j1GFX_FillRect(j1GFX_Ctrl, x, y, w, h, bg);
	if (opts & j1GFX_SDL_TEXT_OPTS_FRAME) j1GFX_DrawRect(j1GFX_Ctrl, x, y, w, h, fg);
	
	j1GFX_GetFontDims(font, scale, &fontW, &fontH);
	j1GFX_SetFontFgBg(j1GFX_Ctrl, font, scale, fg, bg);
	
	if (opts & j1GFX_SDL_TEXT_OPTS_VCENT)
	{
		lineCnt	= 0;
		textPtr	= text;
		
		while ((newlinePtr = strchr(textPtr, '\n')))
		{
			lineCnt++;
			textPtr = newlinePtr + 1;
		}
		
		lineCnt++;
		
		offY	= (h - lineCnt * fontH) / 2;
	}

	textPtr	= text;
	while ((newlinePtr = strchr(textPtr, '\n')))
	{
		textLen	 = newlinePtr - textPtr;
		if (opts & j1GFX_SDL_TEXT_OPTS_HCENT) offX = (w - textLen * fontW) / 2;
		j1GFX_SetTextXY(j1GFX_Ctrl, x + offX, y + offY);
		while (textLen--) j1GFX_PutChar(j1GFX_Ctrl, *textPtr++);
		textPtr  = newlinePtr + 1;
		offY	+= fontH;
	}
		
	textLen  = strlen(textPtr);
	if (opts & j1GFX_SDL_TEXT_OPTS_HCENT) offX = (w - textLen * fontW) / 2;
	j1GFX_SetTextXY(j1GFX_Ctrl, x + offX, y + offY);
	while (textLen--) j1GFX_PutChar(j1GFX_Ctrl, *textPtr++);
}

//---------------------------------------------------------------------
//	j1GFX_SDL_Draw() - draw an SDL table onto the screen
//---------------------------------------------------------------------

void j1GFX_SDL_Draw(j1GFX_CTRL *j1GFX_Ctrl, j1GFX_SDL_ENTRY const *sdlTable, uint8 sdlCount)
{
#if j1GFX_PARAM_SDL_TEXT
	union
	{
#if j1GFX_PARAM_SDL_TEXT
		_j1GFX_SDL_Text			*text;
#endif
		void const				*sdlPtr;
		
	} sdlPtr;
	
	while (sdlCount--)
	{
		sdlPtr.sdlPtr = sdlTable->sdlPtr;

		switch (sdlTable->sdlType)
		{
#if j1GFX_PARAM_SDL_TEXT
		case j1GFX_SDL_TEXT:
			j1GFX_Text(j1GFX_Ctrl, sdlPtr.text->opts, sdlPtr.text->x, sdlPtr.text->y, sdlPtr.text->w, sdlPtr.text->h, sdlPtr.text->font, sdlPtr.text->scale, sdlPtr.text->fg, sdlPtr.text->bg, sdlPtr.text->text);
			break;
#endif
		
		default:
			break;
		}
		
		sdlTable++;
	}
#endif
}

//---------------------------------------------------------------------
//	j1GFX_SDL_HitTest() - test for touch hits against SDL table
//---------------------------------------------------------------------

void j1GFX_SDL_HitTest(j1GFX_CTRL *j1GFX_Ctrl, j1GFX_SDL_ENTRY const *sdlTable, uint8 sdlCount, j1GFX_SDL_ENTRY *hit)
{
}
