//=====================================================================
//
//	`$INSTANCE_NAME`_LCD.hh - driver file for HAOYU ILI9325C 2.8" TFT
//
//	version 1.1 - 15 feb 2014
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#if !defined(`$INSTANCE_NAME`_LCD_HH)
#define `$INSTANCE_NAME`_LCD_HH
	
//#define	`$INSTANCE_NAME`_LCD_PAR_MODE	`$LCD_PAR_MODE`
//#define	`$INSTANCE_NAME`_LCD_SPI_MODE	`$LCD_SPI_MODE`
//#define	`$INSTANCE_NAME`_SENDER			`$SENDER`
//#define	`$INSTANCE_NAME`_RECEIVER		`$RECEIVER`
	
//#include "`$J1GFX_INSTANCE`.h"

//---------------------------------------------------------------------
//	FreeRTOS includes
//---------------------------------------------------------------------

//#include <FreeRTOS.h>
//#include <list.h>
//#include <queue.h>
//#include <semphr.h>
//#include <task.h>
//#include <timers.h>
	
//---------------------------------------------------------------------
//	defines for HY28B
//---------------------------------------------------------------------

#define	`$INSTANCE_NAME`_LCD_DelayMS(ms)	CyDelay(ms)

#define `$INSTANCE_NAME`_LCD_TFTWIDTH	240
#define `$INSTANCE_NAME`_LCD_TFTHEIGHT	320

#define `$INSTANCE_NAME`_LCD_START   (0x70)              /* Start byte for SPI transfer        */
#define `$INSTANCE_NAME`_LCD_RD      (0x01)              /* WR bit 1 within start              */
#define	`$INSTANCE_NAME`_LCD_WR      (0x00)              /* WR bit 0 within start              */
#define `$INSTANCE_NAME`_LCD_DATA    (0x02)              /* RS bit 1 within start byte         */
#define `$INSTANCE_NAME`_LCD_INDEX   (0x00)              /* RS bit 0 within start byte         */

#define	`$INSTANCE_NAME`_LCD_EM_03_TRI		0x8000			/* tri mode									*/
#define	`$INSTANCE_NAME`_LCD_EM_03_DFM		0x4000			/* dfm mode if tri = 1						*/
#define	`$INSTANCE_NAME`_LCD_EM_03_BGR		0x1000			/* bgr=0 (RGB), bgr=1 (GRB)					*/
#define	`$INSTANCE_NAME`_LCD_EM_03_ORG		0x0080			/* org=0 leave org, org=1 follow directions	*/
#define	`$INSTANCE_NAME`_LCD_EM_03_ID1		0x0020			/* i/d1 - inc/dec bit 1						*/
#define	`$INSTANCE_NAME`_LCD_EM_03_ID0		0x0010			/* i/d0 - inc/dec bit 0						*/
#define	`$INSTANCE_NAME`_LCD_EM_03_AM		0x0008			/* am - addressing mode						*/

// control register masks

#define	`$INSTANCE_NAME`_CTRL_RESET		0x01
#define	`$INSTANCE_NAME`_CTRL_SS		0x02
#define `$INSTANCE_NAME`_CTRL_TSS		0x04
#define	`$INSTANCE_NAME`_CTRL_TCLK_LCLK	0x08
#define	`$INSTANCE_NAME`_CTRL_RS		0x10
#define	`$INSTANCE_NAME`_CTRL_RD		0x20
#define	`$INSTANCE_NAME`_CTRL_WR		0x40
#define	`$INSTANCE_NAME`_CTRL_PAR_SER	0x80

//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_LCD hardware ctrl
//---------------------------------------------------------------------

typedef struct _`$INSTANCE_NAME`_LCD_CTRL
{
	uint8			dispType;					// display type (driver dependent)
												// not used for HY28B
												// but we need at least a 1 byte struct
												// for compatibility with other
												// drivers
	
} `$INSTANCE_NAME`_LCD_CTRL;

//---------------------------------------------------------------------
//	function prototypes
//---------------------------------------------------------------------

void `$INSTANCE_NAME`_LCD_SetRotation(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, `$J1GFX_LIB`_ROTATE rotate);
void `$INSTANCE_NAME`_LCD_DrawPixel(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, int16 x, int16 y, uint32 rgb);
void `$INSTANCE_NAME`_LCD_DrawVLine(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 h, uint32 rgb);
void `$INSTANCE_NAME`_LCD_DrawHLine(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, uint32 rgb);
void `$INSTANCE_NAME`_LCD_DrawFont(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, int16 x, int16_t y, char c, `$J1GFX_LIB`_FONT font, uint16 w, uint16 h, uint8 scale, uint32 fgRGB, uint32 bgRGB);
void `$INSTANCE_NAME`_LCD_FillRect(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, int16 h, uint32 rgb);

#endif	// `$INSTANCE_NAME`_LCD_HH
