//=====================================================================
//
//	`$INSTANCE_NAME`_Touch.c - HAOYU 2.8" GFX Driver
//
//	version 1.1 - 15 feb 2014
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#include <project.h>

//---------------------------------------------------------------------
// `$INSTANCE_NAME`_Touch_Touched() - simple touch detector
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Touch_Touched(void)
{
	int		i;
	
	for (i = 0; i < 8; i++)
	{
		if (`$INSTANCE_NAME`_iTIRQ_Read() == 1) return false;
	}
	
	return true;
}

//---------------------------------------------------------------------
//	EVERYTHING else is only compiled if the TOUCH_SPI_ENABLE is on
//---------------------------------------------------------------------

#if `$INSTANCE_NAME`_PARAM_TOUCH_SPI_ENABLE

static void		DrawCross(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, uint16 Xpos, uint16 Ypos);
static uint8	WR_CMD(uint8 cmd);
static uint16	RD_ADC(uint8 cmd);
static void		Get_XY_ADC(`$INSTANCE_NAME`_Touch_Point *pointADC);

#define	CHX		0x90	// channel X+ selection command
#define	CHY		0xd0	// channel Y+ selection command

//---------------------------------------------------------------------
//	private variables
//---------------------------------------------------------------------

static `$INSTANCE_NAME`_Touch_Point	XY_Points[3] =
{
	{  45,  45 },
	{ 270,  45 },
	{ 190, 190 }
};

//---------------------------------------------------------------------
// `$INSTANCE_NAME`_Touch_SelectHi()/Lo()
//---------------------------------------------------------------------

#define	`$INSTANCE_NAME`_Touch_SelectHi()	*`$INSTANCE_NAME`_iCtrl_Control_PTR |= `$INSTANCE_NAME`_CTRL_TSS
#define	`$INSTANCE_NAME`_Touch_SelectLo()	*`$INSTANCE_NAME`_iCtrl_Control_PTR &= ~`$INSTANCE_NAME`_CTRL_TSS

//---------------------------------------------------------------------
// `$INSTANCE_NAME`_Touch_StartHelper() - start the touch part of component
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Touch_StartHelper(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, `$INSTANCE_NAME`_Touch_CTRL *preload)
{
//	`$INSTANCE_NAME`_Touch_Point	readADC;

	`$INSTANCE_NAME`_iTSPI_Start();

	`$INSTANCE_NAME`_Touch_SelectLo();
	WR_CMD(0);
	WR_CMD(0);
	WR_CMD(0);
	WR_CMD(0);
	`$INSTANCE_NAME`_Touch_SelectHi();
//	Get_XY_ADC(&readADC);	// do a read to 'init' the touch system

	((`$INSTANCE_NAME`_CTRL *) (j1GFX_Ctrl->hardware))->Touch_Ctrl.XY_Points = XY_Points;
	
	`$INSTANCE_NAME`_Touch_Calibrate(j1GFX_Ctrl, preload);
	
	return true;
}

//---------------------------------------------------------------------
// `$INSTANCE_NAME`_Touch_ReadADC() - read the current point adc values
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Touch_ReadADC(`$INSTANCE_NAME`_Touch_Point *point)
{
	int		m0;
	uint8	count = 0;
	int		buffer[2][2], *bfrPtr;
	int		result[2];
	`$INSTANCE_NAME`_Touch_Point	readADC;
	
	if (point == NULL) return false;
	
	bfrPtr = buffer[0];
	bfrPtr[0] = 0;
	bfrPtr[1] = 0;
	
	do
	{
		if (count == 3)
		{
			bfrPtr = buffer[1];
			bfrPtr[0] = 0;
			bfrPtr[1] = 0;
		}
		
		Get_XY_ADC(&readADC);
		bfrPtr[0] += readADC.x;
		bfrPtr[1] += readADC.y;
		count++;
	} while (count < 6 && !`$INSTANCE_NAME`_iTIRQ_Read());
	
	if (count < 6) return false;
	
	for (count = 0; count < 2; count++)
	{
		buffer[0][count] /= 3;
		buffer[1][count] /= 3;

		// delta of two groups
		m0 = buffer[0][count] - buffer[1][count];
		
		// abs the delta
		if (m0 < 0) m0 = -m0;
		
		// check treshold to discard
		if (m0 > `$INSTANCE_NAME`_PARAM_TOUCH_DELTA_MAX) return false;
		
		// calculate final average
		result[count] = (buffer[0][count] + buffer[1][count]) / 2;
	}
		
	point->x = result[0];
	point->y = result[1];
		
	return true;
}

//---------------------------------------------------------------------
// `$INSTANCE_NAME`_Touch_Calibrate() - calibrate the screen
//---------------------------------------------------------------------

void `$INSTANCE_NAME`_Touch_Calibrate(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, `$INSTANCE_NAME`_Touch_CTRL *preload)
{
	uint8	i;
	uint16	upCnt;
	`$INSTANCE_NAME`_CTRL			*hCtrl;
	`$INSTANCE_NAME`_Touch_CTRL		*tC;
	`$INSTANCE_NAME`_Touch_Point	touch;
	
	hCtrl	= (`$INSTANCE_NAME`_CTRL *) j1GFX_Ctrl->hardware;
	tC		= &(hCtrl->Touch_Ctrl);
	
	if (preload)
	{
		memcpy(tC->ADC_Points, preload->ADC_Points, sizeof(tC->ADC_Points));
	}
	
	else
	{
		`$J1GFX_LIB`_SetRegion(NULL, j1GFX_Ctrl, 0, 0, hCtrl->currWidth, hCtrl->currHeight);

		for (i = 0; i < 3; i++)
		{
			upCnt = `$INSTANCE_NAME`_PARAM_TOUCH_UP_PRE_CNT;
			while (upCnt)
			{
				if (`$INSTANCE_NAME`_iTIRQ_Read() == 0) upCnt = `$INSTANCE_NAME`_PARAM_TOUCH_UP_PRE_CNT;
				CyDelay(1);
				upCnt--;
			}
		
			`$J1GFX_LIB`_FillDisplay(j1GFX_Ctrl, 0x000000);
			`$J1GFX_LIB`_SetFontFgBg(j1GFX_Ctrl, j1GFX_FONT_5X7, 1, 0xffffff, 0x000000);
			`$J1GFX_LIB`_SetTextXY(j1GFX_Ctrl, 10, 10);
			`$J1GFX_LIB`_PutString(j1GFX_Ctrl, "Touch crosshair to calibrate");
		
			DrawCross(j1GFX_Ctrl, tC->XY_Points[i].x, tC->XY_Points[i].y);
		
			while (!`$INSTANCE_NAME`_Touch_ReadADC(&touch)) {}
		
			tC->ADC_Points[i].x = touch.x;
			tC->ADC_Points[i].y = touch.y;

			upCnt = `$INSTANCE_NAME`_PARAM_TOUCH_UP_PST_CNT;
			while (upCnt)
			{
				if (`$INSTANCE_NAME`_iTIRQ_Read() == 0) upCnt = `$INSTANCE_NAME`_PARAM_TOUCH_UP_PST_CNT;
				CyDelay(1);
				upCnt--;
			}
		}
	
		`$J1GFX_LIB`_FillDisplay(j1GFX_Ctrl, 0x000000);
	}
	
	`$INSTANCE_NAME`_Touch_CalcCalibrationMatrix(tC);
}

//---------------------------------------------------------------------
// `$INSTANCE_NAME`_Touch_CalcCalibrationMatrix()
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Touch_CalcCalibrationMatrix(`$INSTANCE_NAME`_Touch_CTRL *tC)
{
	tC->matrix.Divider	= ((tC->ADC_Points[0].x - tC->ADC_Points[2].x) * (tC->ADC_Points[1].y - tC->ADC_Points[2].y)) -
						  ((tC->ADC_Points[1].x - tC->ADC_Points[2].x) * (tC->ADC_Points[0].y - tC->ADC_Points[2].y));
						
	if (tC->matrix.Divider == 0) return false;
	
	tC->matrix.An	= ((tC->XY_Points[0].x  - tC->XY_Points[2].x ) * (tC->ADC_Points[1].y - tC->ADC_Points[2].y)) -
					  ((tC->XY_Points[1].x  - tC->XY_Points[2].x ) * (tC->ADC_Points[0].y - tC->ADC_Points[2].y));
	
	tC->matrix.Bn	= ((tC->ADC_Points[0].x - tC->ADC_Points[2].x) * (tC->XY_Points[1].x  - tC->XY_Points[2].x )) -
					  ((tC->XY_Points[0].x  - tC->XY_Points[2].x ) * (tC->ADC_Points[1].x - tC->ADC_Points[2].x));
					
	tC->matrix.Cn	= (tC->ADC_Points[2].x * tC->XY_Points[1].x - tC->ADC_Points[1].x * tC->XY_Points[2].x) * tC->ADC_Points[0].y +
					  (tC->ADC_Points[0].x * tC->XY_Points[2].x - tC->ADC_Points[2].x * tC->XY_Points[0].x) * tC->ADC_Points[1].y +
					  (tC->ADC_Points[1].x * tC->XY_Points[0].x - tC->ADC_Points[0].x * tC->XY_Points[1].x) * tC->ADC_Points[2].y;
					
	tC->matrix.Dn	= ((tC->XY_Points[0].y  - tC->XY_Points[2].y ) * (tC->ADC_Points[1].y - tC->ADC_Points[2].y)) -
					  ((tC->XY_Points[1].y  - tC->XY_Points[2].y ) * (tC->ADC_Points[0].y - tC->ADC_Points[2].y));
	
	tC->matrix.En	= ((tC->ADC_Points[0].x - tC->ADC_Points[2].x) * (tC->XY_Points[1].y  - tC->XY_Points[2].y )) -
					  ((tC->XY_Points[0].y  - tC->XY_Points[2].y ) * (tC->ADC_Points[1].x - tC->ADC_Points[2].x));
					
	tC->matrix.Fn	= (tC->ADC_Points[2].x * tC->XY_Points[1].y - tC->ADC_Points[1].x * tC->XY_Points[2].y) * tC->ADC_Points[0].y +
					  (tC->ADC_Points[0].x * tC->XY_Points[2].y - tC->ADC_Points[2].x * tC->XY_Points[0].y) * tC->ADC_Points[1].y +
					  (tC->ADC_Points[1].x * tC->XY_Points[0].y - tC->ADC_Points[0].x * tC->XY_Points[1].y) * tC->ADC_Points[2].y;
					
	return true;
}

//---------------------------------------------------------------------
// `$INSTANCE_NAME`_Touch_GetTouchXY() - ALWAYS in 'N' Rotation
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Touch_GetTouchXY(`$INSTANCE_NAME`_Touch_CTRL *tC, bool waitUp, `$INSTANCE_NAME`_Touch_Point *tXY)
{
	`$INSTANCE_NAME`_Touch_Point	tADC;
	uint16							upCnt;
	
	tXY->x = 0;
	tXY->y = 0;
	
	if (waitUp)
	{
		upCnt = `$INSTANCE_NAME`_PARAM_TOUCH_UP_PRE_CNT;
		while (upCnt)
		{
			if (`$INSTANCE_NAME`_iTIRQ_Read() == 0) upCnt = `$INSTANCE_NAME`_PARAM_TOUCH_UP_PRE_CNT;
			CyDelay(1);
			upCnt--;
		}
	}
		
	while (!`$INSTANCE_NAME`_Touch_ReadADC(&tADC)) {}
		
	if (tC->matrix.Divider == 0) return false;
	
	tXY->x	= ( (tC->matrix.An * tADC.x) +
			    (tC->matrix.Bn * tADC.y) +
				 tC->matrix.Cn
			  ) / tC->matrix.Divider;
	
	tXY->y	= ( (tC->matrix.Dn * tADC.x) +
			    (tC->matrix.En * tADC.y) +
				 tC->matrix.Fn
			  ) / tC->matrix.Divider;
	
	if (waitUp)
	{
		upCnt = `$INSTANCE_NAME`_PARAM_TOUCH_UP_PST_CNT;
		while (upCnt)
		{
			if (`$INSTANCE_NAME`_iTIRQ_Read() == 0) upCnt = `$INSTANCE_NAME`_PARAM_TOUCH_UP_PST_CNT;
			CyDelay(1);
			upCnt--;
		}
	}
		
	return true;
}

//---------------------------------------------------------------------
// DrawCross()
//---------------------------------------------------------------------

static void DrawCross(`$J1GFX_LIB`_CTRL *j1GFX_Ctrl, uint16 Xpos, uint16 Ypos)
{
	`$J1GFX_LIB`_DrawHLine(j1GFX_Ctrl, Xpos - 15, Ypos, 14, 0xffffff);
	`$J1GFX_LIB`_DrawHLine(j1GFX_Ctrl, Xpos +  2, Ypos, 14, 0xffffff);
	`$J1GFX_LIB`_DrawVLine(j1GFX_Ctrl, Xpos, Ypos - 15, 14, 0xffffff);
	`$J1GFX_LIB`_DrawVLine(j1GFX_Ctrl, Xpos, Ypos +  2, 14, 0xffffff);
}

//---------------------------------------------------------------------
// WR_CMD()
//---------------------------------------------------------------------

static uint8 WR_CMD(uint8 cmd)
{
	`$INSTANCE_NAME`_iTSPI_WriteTxData(cmd);
	
	while ((`$INSTANCE_NAME`_iTSPI_ReadRxStatus() & `$INSTANCE_NAME`_iTSPI_STS_RX_FIFO_NOT_EMPTY) == 0) {}
	
	return `$INSTANCE_NAME`_iTSPI_ReadRxData();
}

//---------------------------------------------------------------------
// RD_ADC()
//---------------------------------------------------------------------

static uint16 RD_ADC(uint8 cmd)
{
	uint16	val, byt;
	
	WR_CMD(cmd);
	CyDelayUs(1);
	
	`$INSTANCE_NAME`_iTSPI_WriteTxData(0x00);
	`$INSTANCE_NAME`_iTSPI_WriteTxData(0x00);
	
	while ((`$INSTANCE_NAME`_iTSPI_ReadRxStatus() & `$INSTANCE_NAME`_iTSPI_STS_RX_FIFO_NOT_EMPTY) == 0) {}
	byt = `$INSTANCE_NAME`_iTSPI_ReadRxData();
	val = byt << 8;
	
	while ((`$INSTANCE_NAME`_iTSPI_ReadRxStatus() & `$INSTANCE_NAME`_iTSPI_STS_RX_FIFO_NOT_EMPTY) == 0) {}
	byt = `$INSTANCE_NAME`_iTSPI_ReadRxData();
	val |= byt;
	val >>= 3;
	val &= 0xfff;
	
	return val;
}

//---------------------------------------------------------------------
// Get_XY_ADC()
//---------------------------------------------------------------------

static void Get_XY_ADC(`$INSTANCE_NAME`_Touch_Point *pointADC)
{
	`$INSTANCE_NAME`_Touch_SelectLo();
	CyDelayUs(1);
	pointADC->x = RD_ADC(CHX);
	pointADC->y = RD_ADC(CHY);
	`$INSTANCE_NAME`_Touch_SelectHi();
}

//---------------------------------------------------------------------
// `$INSTANCE_NAME`_Touch_CtrlToString()
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Touch_CtrlToString(`$INSTANCE_NAME`_Touch_CTRL *touch_Ctrl, char *bfr, uint8 bfrSize)
{
	uint8	i;
	
	if (bfrSize < 32) return false;
	
	for (i = 0; i < 3; i++)
	{
		if (i) *bfr++ = ':';
		bfr = j1_u16tohexstr(touch_Ctrl->ADC_Points[i].x, bfr);
		*bfr++ = '.';
		bfr = j1_u16tohexstr(touch_Ctrl->ADC_Points[i].y, bfr);
	}
	
	*bfr = 0;
	
	return true;
}

//---------------------------------------------------------------------
// `$INSTANCE_NAME`_Touch_StringToCtrl()
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Touch_StringToCtrl(char *bfr, `$INSTANCE_NAME`_Touch_CTRL *touch_Ctrl)
{
	uint8	i;
	
	if (strlen(bfr) != 29) return false;
	
	for (i = 0; i < 3; i++)
	{
		if (i) if (*bfr++ != ':') return false;
		
		if (!j1_hexfldtouint16(bfr, &touch_Ctrl->ADC_Points[i].x)) return false;
		bfr += 4;
		if (*bfr++ != '.') return false;
		if (!j1_hexfldtouint16(bfr, &touch_Ctrl->ADC_Points[i].y)) return false;
		bfr += 4;
	}
	
	return true;
}

#endif
