//=====================================================================
//
//	`$INSTANCE_NAME`.c - j1sys device independent put lib
//
//	version 1.2 - 15 jan 2015
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JOSHUA 1 SYSTEMS INC. OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#include "`$INSTANCE_NAME`.h"

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_Start() - init the control
//---------------------------------------------------------------------

void `$j1LIB_NAME`_Start(`$j1LIB_NAME`_CTRL *pC, void *Put_Char, void *deviceCtrl)
{
	pC->Put_Char	= Put_Char;
	pC->deviceCtrl	= deviceCtrl;
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_Char() - put a char using control
//---------------------------------------------------------------------

void `$j1LIB_NAME`_Char(`$j1LIB_NAME`_CTRL *pC, char c)
{
	if (pC->Put_Char) pC->Put_Char(pC->deviceCtrl, c);
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_String() - simple iterative string via _Char
//---------------------------------------------------------------------

void `$j1LIB_NAME`_String(`$j1LIB_NAME`_CTRL *pC, char *string)
{
	char	c;
	
	if (pC->Put_Char)
	{
		while ((c = *string++)) pC->Put_Char(pC->deviceCtrl, c);
	}
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_StringOpt() - simple iterative string via _Char w/options
//---------------------------------------------------------------------

void `$j1LIB_NAME`_StringOpt(`$j1LIB_NAME`_CTRL *pC, char *string, uint16 options)
{
	char	c;
	
	if (pC->Put_Char)
	{
		if (options & `$j1LIB_NAME`_PRESPC) pC->Put_Char(pC->deviceCtrl, ' ');
		while ((c = *string++)) pC->Put_Char(pC->deviceCtrl, c);
		if (options & `$j1LIB_NAME`_PSTSPC) pC->Put_Char(pC->deviceCtrl, ' ');
	}
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_StringField() - string output limited/padded w/options
//---------------------------------------------------------------------

void `$j1LIB_NAME`_StringField(`$j1LIB_NAME`_CTRL *pC, char *string, uint16 length, uint16 options)
{
	char	c;
	
	if (pC->Put_Char)
	{
		if (options & `$j1LIB_NAME`_PRESPC) pC->Put_Char(pC->deviceCtrl, ' ');

		while ((c = *string++))
		{
			if (length)
			{
				pC->Put_Char(pC->deviceCtrl, c);
				length--;
			}
		}
		
		while (length--) pC->Put_Char(pC->deviceCtrl, ' ');

		if (options & `$j1LIB_NAME`_PSTSPC) pC->Put_Char(pC->deviceCtrl, ' ');
	}
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_Field() - simple iterative field output w/options
//---------------------------------------------------------------------

void `$j1LIB_NAME`_Field(`$j1LIB_NAME`_CTRL *pC, char *field, uint16 length, uint16 options)
{
	if (pC->Put_Char)
	{
		if (options & `$j1LIB_NAME`_PRESPC) pC->Put_Char(pC->deviceCtrl, ' ');
		while (length--) pC->Put_Char(pC->deviceCtrl, *field++);
		if (options & `$j1LIB_NAME`_PSTSPC) pC->Put_Char(pC->deviceCtrl, ' ');
	}
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_Hex8() - output 8 bit hex w/options
//---------------------------------------------------------------------

void `$j1LIB_NAME`_Hex8(`$j1LIB_NAME`_CTRL *pC, uint8 hex, uint16 options)
{
	char	nib;
	
	if (options & `$j1LIB_NAME`_PRESPC)
	{
		`$j1LIB_NAME`_Char(pC, ' ');
		options &= ~`$j1LIB_NAME`_PRESPC;
	}
	
	if (options & `$j1LIB_NAME`_HEX_PREFIX)
	{
		`$j1LIB_NAME`_String(pC, "0x");
		options &= ~`$j1LIB_NAME`_HEX_PREFIX;
	}
	
	nib = (hex >> 4) + '0';
	if (nib > '9')
	{
		if (options & `$j1LIB_NAME`_HEX_LOWER) nib += 'a' - '9' - 1;
								  else nib += 'A' - '9' - 1;
	}
	`$j1LIB_NAME`_Char(pC, nib);

	nib = (hex & 0x0f) + '0';
	if (nib > '9')
	{
		if (options & `$j1LIB_NAME`_HEX_LOWER) nib += 'a' - '9' - 1;
								  else nib += 'A' - '9' - 1;
	}
	`$j1LIB_NAME`_Char(pC, nib);
	
	if (options & `$j1LIB_NAME`_PSTSPC)
	{
		`$j1LIB_NAME`_Char(pC, ' ');
		options &= ~`$j1LIB_NAME`_PSTSPC;
	}
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_Hex16() - output 16 bit hex w/options
//---------------------------------------------------------------------

void `$j1LIB_NAME`_Hex16(`$j1LIB_NAME`_CTRL *pC, uint16 hex, uint16 options)
{
	if (options & `$j1LIB_NAME`_PRESPC)
	{
		`$j1LIB_NAME`_Char(pC, ' ');
		options &= ~`$j1LIB_NAME`_PRESPC;
	}

	if (options & `$j1LIB_NAME`_HEX_PREFIX)
	{
		`$j1LIB_NAME`_String(pC, "0x");
		options &= ~`$j1LIB_NAME`_HEX_PREFIX;
	}
	
	`$j1LIB_NAME`_Hex8(pC, hex >> 8  , options & ~`$j1LIB_NAME`_PSTSPC);
	`$j1LIB_NAME`_Hex8(pC, hex & 0xff, options & ~`$j1LIB_NAME`_PSTSPC);
	
	if (options & `$j1LIB_NAME`_PSTSPC)
	{
		`$j1LIB_NAME`_Char(pC, ' ');
		options &= ~`$j1LIB_NAME`_PSTSPC;
	}
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_Hex32() - output 32 bit hex w/options
//---------------------------------------------------------------------

void `$j1LIB_NAME`_Hex32(`$j1LIB_NAME`_CTRL *pC, uint32 hex, uint16 options)
{
	if (options & `$j1LIB_NAME`_PRESPC)
	{
		`$j1LIB_NAME`_Char(pC, ' ');
		options &= ~`$j1LIB_NAME`_PRESPC;
	}

	if (options & `$j1LIB_NAME`_HEX_PREFIX)
	{
		`$j1LIB_NAME`_String(pC, "0x");
		options &= ~`$j1LIB_NAME`_HEX_PREFIX;
	}

	`$j1LIB_NAME`_Hex16(pC, hex >> 16   , options & ~`$j1LIB_NAME`_PSTSPC);
	`$j1LIB_NAME`_Hex16(pC, hex & 0xffff, options & ~`$j1LIB_NAME`_PSTSPC);
	
	if (options & `$j1LIB_NAME`_PSTSPC)
	{
		`$j1LIB_NAME`_Char(pC, ' ');
		options &= ~`$j1LIB_NAME`_PSTSPC;
	}
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_HexDump() - output hex dump line w/options
//---------------------------------------------------------------------

void `$j1LIB_NAME`_HexDump(`$j1LIB_NAME`_CTRL *pC, uint32 addrHex, uint8 *bfr, uint8 lo, uint8 hi, uint16 options)
{
	uint8	i, len, *bfrPtr, c;
	
	if (options & `$j1LIB_NAME`_HEXDUMPA32) `$j1LIB_NAME`_Hex32(pC, addrHex, options);
	else `$j1LIB_NAME`_Hex16(pC, (uint16) addrHex, options);
	
	if (options & `$j1LIB_NAME`_HEXDUMP16) len = 16;
	else len = 8;
	
	bfrPtr = bfr;
	for (i = 0; i < len; i++)
	{
		if (i < lo || hi < i) `$j1LIB_NAME`_StringOpt(pC, "..", `$j1LIB_NAME`_PRESPC);
		else `$j1LIB_NAME`_Hex8(pC, *bfrPtr++, options | `$j1LIB_NAME`_PRESPC);
	}
	
	if (options & `$j1LIB_NAME`_HEXDUMPASC)
	{
		`$j1LIB_NAME`_String(pC, " |");
		
		bfrPtr = bfr;
		for (i = 0; i < len; i++)
		{
			if (i < lo || hi < i) `$j1LIB_NAME`_Char(pC, ' ');
			else
			{
				c = *bfrPtr++;
				if (c < 0x20 || 0x7f < c) c = 0x20;
				`$j1LIB_NAME`_Char(pC, c);
			}
		}
		
		`$j1LIB_NAME`_Char(pC, '|');
	}
}

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_FillRegion() - special shortcut to GFX
//---------------------------------------------------------------------

#if `$j1GFX_ENABLED`
void `$j1LIB_NAME`_FillRegion(`$j1LIB_NAME`_CTRL *pC, uint32 rgb)
{
	`$j1GFX_LIB`_CTRL	*gC;
	
	if (pC)
	{
		gC = (`$j1GFX_LIB`_CTRL *) pC->deviceCtrl;
		if (gC)
		{
			`$j1GFX_LIB`_FillRegion(gC, rgb);
		}
	}
}
#endif

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_SetFontFgBg() - special shortcut to GFX
//---------------------------------------------------------------------

#if `$j1GFX_ENABLED`
void `$j1LIB_NAME`_SetFontFgBg(`$j1LIB_NAME`_CTRL *pC, `$j1GFX_LIB`_FONT font, uint8 scale, uint32 fg, uint32 bg)
{
	`$j1GFX_LIB`_CTRL	*gC;
	
	if (pC)
	{
		gC = (`$j1GFX_LIB`_CTRL *) pC->deviceCtrl;
		if (gC)
		{
			`$j1GFX_LIB`_SetFontFgBg(gC, font, scale, fg, bg);
		}
	}
}
#endif

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_SetTextRowCol() - special shortcut to GFX
//---------------------------------------------------------------------

#if `$j1GFX_ENABLED`
void `$j1LIB_NAME`_SetTextRowCol(`$j1LIB_NAME`_CTRL *pC, uint8 row, uint8 col)
{
	`$j1GFX_LIB`_CTRL	*gC;
	
	if (pC)
	{
		gC = (`$j1GFX_LIB`_CTRL *) pC->deviceCtrl;
		if (gC)
		{
			`$j1GFX_LIB`_SetTextRowCol(gC, row, col);
		}
	}
}
#endif

//---------------------------------------------------------------------
//	`$j1LIB_NAME`_SetTextXY() - special shortcut to GFX
//---------------------------------------------------------------------

#if `$j1GFX_ENABLED`
void `$j1LIB_NAME`_SetTextXY(`$j1LIB_NAME`_CTRL *pC, uint16 x, uint16 y)
{
	`$j1GFX_LIB`_CTRL	*gC;
	
	if (pC)
	{
		gC = (`$j1GFX_LIB`_CTRL *) pC->deviceCtrl;
		if (gC)
		{
			`$j1GFX_LIB`_SetTextXY(gC, x, y);
		}
	}
}
#endif
