//=====================================================================
//
//	`$INSTANCE_NAME`.c - j1sys spi flash for SST25 series
//
//	version 1.0 - 16 dec 2013
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================


//#include "`$INSTANCE_NAME`.h"
#include <project.h>

//---------------------------------------------------------------------
//	timeout magic
//---------------------------------------------------------------------

//extern	`$TICK_TYPE`	`$TICK_FUNC`
#define	TICK_TYPE	`$TICK_TYPE`
#define	TICK_FUNC	`$TICK_FUNC`
#if `$TICK_ASCEND`
#define TICKS(x)	(TICK_FUNC - x)
#else
#define TICKS(x)	(x - TICK_FUNC)
#endif
#define TIMEOUT(x,y)	(TICKS(x) > y)

//---------------------------------------------------------------------
//	private defines
//---------------------------------------------------------------------

#define READ				(0x03)    // SPI Flash opcode: Read up up to 25MHz
#define READ_FAST			(0x0B)    // SPI Flash opcode: Read up to 50MHz with 1 dummy byte
#define ERASE_4K			(0x20)    // SPI Flash opcode: 4KByte sector erase
#define ERASE_32K			(0x52)    // SPI Flash opcode: 32KByte block erase
#define ERASE_SECTOR		(0xD8)    // SPI Flash opcode: 64KByte block erase
#define ERASE_ALL			(0x60)    // SPI Flash opcode: Entire chip erase
#define WRITE				(0x02)    // SPI Flash opcode: Write one byte (or a page of up to 256 bytes, depending on device)
#define WRITE_WORD_STREAM	(0xAD)    // SPI Flash opcode: Write continuous stream of 16-bit words (AAI mode); available on SST25VF016B (but not on SST25VF010A)
#define WRITE_BYTE_STREAM	(0xAF)    // SPI Flash opcode: Write continuous stream of bytes (AAI mode); available on SST25VF010A (but not on SST25VF016B)
#define RDSR				(0x05)    // SPI Flash opcode: Read Status Register
#define EWSR				(0x50)    // SPI Flash opcode: Enable Write Status Register
#define WRSR				(0x01)    // SPI Flash opcode: Write Status Register
#define WREN				(0x06)    // SPI Flash opcode: Write Enable
#define WRDI				(0x04)    // SPI Flash opcode: Write Disable / End AAI mode
#define RDID				(0x90)    // SPI Flash opcode: Read ID
#define JEDEC_ID			(0x9F)    // SPI Flash opcode: Read JEDEC ID
#define EBSY				(0x70)    // SPI Flash opcode: Enable write BUSY status on SO pin
#define DBSY				(0x80)    // SPI Flash opcode: Disable write BUSY status on SO pin

#define BUSY    (0x01)    // Mask for Status Register BUSY (Internal Write Operaiton in Progress status) bit
#define WEL     (0x02)    // Mask for Status Register WEL (Write Enable status) bit
#define BP0     (0x04)    // Mask for Status Register BP0 (Block Protect 0) bit
#define BP1     (0x08)    // Mask for Status Register BP1 (Block Protect 1) bit
#define BP2     (0x10)    // Mask for Status Register BP2 (Block Protect 2) bit
#define BP3     (0x20)    // Mask for Status Register BP3 (Block Protect 3) bit
#define AAI     (0x40)    // Mask for Status Register AAI (Auto Address Increment Programming status) bit
#define BPL     (0x80)    // Mask for Status Register BPL (BPx block protect bit read-only protect) bit


//---------------------------------------------------------------------
//	private functions
//---------------------------------------------------------------------

static bool		InitFlash(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl);
static void		SelectFlash(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl);
static void		DeselectFlash(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl);
static bool		WaitWhileBusy(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl);
static uint8	SendCmd(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl, uint8 cmd);
static uint8	WriteReadSPI(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl, uint8 writeVal);

//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_Start() - startup the component
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Start(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl)
{
	bool	result;
	
	`$INSTANCE_NAME`_iSPIM_Start();

	result = InitFlash(j1SpiFlash_Ctrl);
	
	return result;
}

//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_ReadArray()
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_ReadArray(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl, DWORD dwAddress, BYTE *vData, WORD wLength)
{
    // Ignore operations when the destination is NULL or nothing to read
    if (vData == NULL || wLength == 0) return false;

    // Activate chip select
    SelectFlash(j1SpiFlash_Ctrl);

    // Send READ opcode
	WriteReadSPI(j1SpiFlash_Ctrl, READ);

    // Send address
	WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &dwAddress)[2]);
	WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &dwAddress)[1]);
	WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &dwAddress)[0]);

    // Read data
    while (wLength--)
    {
		*vData++ = WriteReadSPI(j1SpiFlash_Ctrl, 0x00);
    }

    // Deactivate chip select
    DeselectFlash(j1SpiFlash_Ctrl);
	
	return true;
}

//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_BeginWrite()
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_BeginWrite(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl, DWORD dwAddr)
{
    j1SpiFlash_Ctrl->writeAddr = dwAddr;
	return true;
}

//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_Write()
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Write(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl, BYTE vData)
{
    // If address is a boundary, erase a sector first
    if ((j1SpiFlash_Ctrl->writeAddr & `$INSTANCE_NAME`_SECTOR_MASK) == 0u)
	{
        `$INSTANCE_NAME`_EraseSector(j1SpiFlash_Ctrl, j1SpiFlash_Ctrl->writeAddr);
	}

    // Enable writing
    SendCmd(j1SpiFlash_Ctrl, WREN);

    // Activate the chip select
	SelectFlash(j1SpiFlash_Ctrl);

    // Send WRITE opcode
	WriteReadSPI(j1SpiFlash_Ctrl, WRITE);

    // Send address
	WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &(j1SpiFlash_Ctrl->writeAddr))[2]);
	WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &(j1SpiFlash_Ctrl->writeAddr))[1]);
	WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &(j1SpiFlash_Ctrl->writeAddr))[0]);

    // Write the byte
	WriteReadSPI(j1SpiFlash_Ctrl, vData);
    j1SpiFlash_Ctrl->writeAddr++;

    // Deactivate chip select and wait for write to complete
	DeselectFlash(j1SpiFlash_Ctrl);
	WaitWhileBusy(j1SpiFlash_Ctrl);
	
	return true;
}

//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_WriteArray()
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_WriteArray(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl, BYTE *vData, WORD wLen)
{
    BOOL	isStarted;
    BYTE	vOpcode;
    BYTE	i;

	// Do nothing if no data to process
	if (wLen == 0u) return false;
	
    // If starting at an odd address, write a single byte
    if (j1SpiFlash_Ctrl->writeAddr & 0x01)
    {
        `$INSTANCE_NAME`_Write(j1SpiFlash_Ctrl, *vData);
        vData++;
        wLen--;
    }

	// Assume we are using AAI Word program mode unless changed later
	vOpcode = WRITE_WORD_STREAM;	
    isStarted = FALSE;

    // Loop over all remaining WORDs
    while (wLen > 1)
    {
        // Don't do anything until chip is ready
        WaitWhileBusy(j1SpiFlash_Ctrl);

        // If address is a sector boundary
        if ((j1SpiFlash_Ctrl->writeAddr & `$INSTANCE_NAME`_SECTOR_MASK) == 0)
		{
            `$INSTANCE_NAME`_EraseSector(j1SpiFlash_Ctrl, j1SpiFlash_Ctrl->writeAddr);
		}

        // If not yet started, initiate AAI mode
        if (!isStarted)
        {
            // Enable writing
            SendCmd(j1SpiFlash_Ctrl, WREN);

			// Select appropriate programming opcode.  The WRITE_WORD_STREAM 
			// mode is the default if neither of these flags are set.
	        if (j1SpiFlash_Ctrl->deviceCaps.bits.bWriteByteStream)
			{
				vOpcode = WRITE_BYTE_STREAM;
			}
			else if(j1SpiFlash_Ctrl->deviceCaps.bits.bPageProgram)
			{
				// Note: Writing one byte at a time is extremely slow (ex: ~667 
				// bytes/second write speed on SST SST25VF064C).  You can 
				// improve this by over a couple of orders of magnitude by 
				// writing a function to write full pages of up to 256 bytes at 
				// a time.  This is implemented this way only because I don't 
				// have an SST25VF064C handy to test with right now. -HS
				while (wLen--)
				{
			        `$INSTANCE_NAME`_Write(j1SpiFlash_Ctrl, *vData++);
				}
				return true;
			}

            // Activate the chip select
			SelectFlash(j1SpiFlash_Ctrl);

		    // Send WRITE_xxx opcode
			WriteReadSPI(j1SpiFlash_Ctrl, vOpcode);

    		// Send address
			WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &(j1SpiFlash_Ctrl->writeAddr))[2]);
			WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &(j1SpiFlash_Ctrl->writeAddr))[1]);
			WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &(j1SpiFlash_Ctrl->writeAddr))[0]);

            isStarted = TRUE;
        }
		
        // Otherwise, just write the AAI command again
        else
        {
            // Assert the chip select pin
			SelectFlash(j1SpiFlash_Ctrl);

            // Issue the WRITE_STREAM command for continuation
			WriteReadSPI(j1SpiFlash_Ctrl, vOpcode);
        }

        // Write a byte or two
        for (i = 0; i <= j1SpiFlash_Ctrl->deviceCaps.bits.bWriteWordStream; i++)
        {
			WriteReadSPI(j1SpiFlash_Ctrl, *vData++);
			j1SpiFlash_Ctrl->writeAddr++;
			wLen--;
		}

        // Release the chip select to begin the write
		DeselectFlash(j1SpiFlash_Ctrl);

        // If a boundary was reached, end the write
        if ((j1SpiFlash_Ctrl->writeAddr & `$INSTANCE_NAME`_SECTOR_MASK) == 0)
        {
            WaitWhileBusy(j1SpiFlash_Ctrl);
            SendCmd(j1SpiFlash_Ctrl, WRDI);
            isStarted = FALSE;
        }
    }

    // Wait for write to complete, then exit AAI mode
    WaitWhileBusy(j1SpiFlash_Ctrl);
    SendCmd(j1SpiFlash_Ctrl, WRDI);

    // If a byte remains, write the odd address
    if (wLen)
	{
		`$INSTANCE_NAME`_Write(j1SpiFlash_Ctrl, *vData);
	}
	
	return true;
}


//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_EraseSector()
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_EraseSector(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl, DWORD dwAddr)
{
    // Enable writing
    SendCmd(j1SpiFlash_Ctrl, WREN);

    // Activate the chip select
	SelectFlash(j1SpiFlash_Ctrl);

    // Send ERASE opcode
	WriteReadSPI(j1SpiFlash_Ctrl, ERASE_4K);

    // Send address
	WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &dwAddr)[2]);
	WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &dwAddr)[1]);
	WriteReadSPI(j1SpiFlash_Ctrl, ((BYTE *) &dwAddr)[0]);

    // Deactivate chip select to perform the erase
 	DeselectFlash(j1SpiFlash_Ctrl);
	
    // Wait for erase to complete
    WaitWhileBusy(j1SpiFlash_Ctrl);
	
	return true;
}


//---------------------------------------------------------------------
//	InitFlash()
//---------------------------------------------------------------------

bool InitFlash(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl)
{
	if (j1SpiFlash_Ctrl == NULL) return false;
	
	memset(j1SpiFlash_Ctrl, 0, sizeof(`$INSTANCE_NAME`_CTRL));
	
	DeselectFlash(j1SpiFlash_Ctrl);		// make sure SS is high

    // Activate chip select
	SelectFlash(j1SpiFlash_Ctrl);
	
    // Send instruction
	WriteReadSPI(j1SpiFlash_Ctrl, RDID);
		
	// Send 3 byte address (0x000000)
	WriteReadSPI(j1SpiFlash_Ctrl, 0x00);
	WriteReadSPI(j1SpiFlash_Ctrl, 0x00);
	WriteReadSPI(j1SpiFlash_Ctrl, 0x00);
		
	// get manufacture id
	j1SpiFlash_Ctrl->mfgID = WriteReadSPI(j1SpiFlash_Ctrl, 0x00);
		
	// get device id
	j1SpiFlash_Ctrl->devID = WriteReadSPI(j1SpiFlash_Ctrl, 0x00);
		
	// Deactivate chip select
	DeselectFlash(j1SpiFlash_Ctrl);
		
	// Decode Device Capabilities Flags from Device ID
	j1SpiFlash_Ctrl->deviceCaps.v = 0x00;

	switch (j1SpiFlash_Ctrl->devID)
	{
	case 0x43:	// SST25LF020(A)	(2 Mbit)	0xAF, 14us, AAI Byte
	case 0x48:	// SST25VF512(A)	(512 Kbit)	0xAF, 14us, AAI Byte
	case 0x49:	// SST25VF010A		(1 Mbit)	0xAF, 14us, AAI Byte
		j1SpiFlash_Ctrl->deviceCaps.bits.bWriteByteStream = 1;
		break;
				
	case 0x4B:	// SST25VF064C		(64 Mbit)	0x02, 1.5ms/256 byte page, no AAI
		j1SpiFlash_Ctrl->deviceCaps.bits.bPageProgram = 1;
		break;

	//case 0x01:	// SST25WF512		(512 Kbit)	0xAD, 50us, AAI Word
	//case 0x02:	// SST25WF010		(1 Mbit)	0xAD, 50us, AAI Word
	//case 0x03:	// SST25WF020		(2 Mbit)	0xAD, 50us, AAI Word
	//case 0x04:	// SST25WF040		(4 Mbit)	0xAD, 50us, AAI Word
	//case 0x05:	// SST25WF080		(8 Mbit)	0xAD, 14us, AAI Word
	//case 0x41:	// SST25VF016B		(16 Mbit)	0xAD,  7us, AAI Word
	//case 0x4A:	// SST25VF032B		(32 Mbit)	0xAD,  7us, AAI Word
	//case 0x8C:	// SST25VF020B		(2 Mbit)	0xAD,  7us, AAI Word
	//case 0x8D:	// SST25VF040B		(4 Mbit)	0xAD,  7us, AAI Word
	//case 0x8E:	// SST25VF080B		(8 Mbit)	0xAD,  7us, AAI Word				
	// Assume AAI Word programming is supported for the above commented 
	// devices and unknown devices.
	default:	
		j1SpiFlash_Ctrl->deviceCaps.bits.bWriteWordStream = 1;
	}

    // Clear any pre-existing AAI write mode
    // This may occur if the Flash is not tied to hardware reset.
    SendCmd(j1SpiFlash_Ctrl, WRDI);

    // Execute Enable-Write-Status-Register (EWSR) instruction
    SendCmd(j1SpiFlash_Ctrl, EWSR);

    // Clear Write-Protect on all memory locations
	SelectFlash(j1SpiFlash_Ctrl);
	WriteReadSPI(j1SpiFlash_Ctrl, WRSR);
	WriteReadSPI(j1SpiFlash_Ctrl, 0x00);	// Clear all block protect bits
    DeselectFlash(j1SpiFlash_Ctrl);
	
	return true;
}

//---------------------------------------------------------------------
//	SelectFlash() - select the flash
//---------------------------------------------------------------------

static void SelectFlash(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl)
{
	`$SS_INSTANCE`_Write(0);			// take it low
}

//---------------------------------------------------------------------
//	DeselectFlash() - deselect the flash
//---------------------------------------------------------------------

static void DeselectFlash(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl)
{
	`$SS_INSTANCE`_Write(1);			// take it high
}

//---------------------------------------------------------------------
//	WaitWhileBusy() - wait for flash to finish op
//---------------------------------------------------------------------

static bool WaitWhileBusy(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl)
{
	uint8	status;
	
    // Activate chip select
	SelectFlash(j1SpiFlash_Ctrl);

    // Send Read Status Register instruction
	WriteReadSPI(j1SpiFlash_Ctrl, RDSR);

	// Poll the BUSY bit
    do
    {
		status = WriteReadSPI(j1SpiFlash_Ctrl, 0x00);
    } while(status & BUSY);

    // Deactivate chip select
	DeselectFlash(j1SpiFlash_Ctrl);
	
	return true;
}

//---------------------------------------------------------------------
//	SendCmd() - send a command and get response
//---------------------------------------------------------------------

static uint8 SendCmd(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl, uint8 cmd)
{
    // Activate chip select
	SelectFlash(j1SpiFlash_Ctrl);

    // Send instruction
	WriteReadSPI(j1SpiFlash_Ctrl, cmd);

    // Deactivate chip select
	DeselectFlash(j1SpiFlash_Ctrl);
	
	return true;
}

//---------------------------------------------------------------------
//	WriteReadSPI() - write a byte to spi and get read
//---------------------------------------------------------------------

static uint8 WriteReadSPI(`$INSTANCE_NAME`_CTRL *j1SpiFlash_Ctrl, uint8 writeVal)
{
	`$INSTANCE_NAME`_iSPIM_ClearTxBuffer();
	`$INSTANCE_NAME`_iSPIM_ClearRxBuffer();
	`$INSTANCE_NAME`_iSPIM_WriteTxData(writeVal);
	while (`$INSTANCE_NAME`_iSPIM_GetRxBufferSize() == 0) {}
	return `$INSTANCE_NAME`_iSPIM_ReadRxData();
	
//    *`$INSTANCE_NAME`_iSPIM_TXDATA_PTR = writeVal;
//	while ((`$INSTANCE_NAME`_iSPIM_TX_STATUS_REG & `$INSTANCE_NAME`_iSPIM_STS_SPI_DONE) == 0) {}
//	return *`$INSTANCE_NAME`_iSPIM_RXDATA_PTR;
}
