//=====================================================================
//
//	j1Types.h - j1 standard data types and macros
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JOSHUA 1 SYSTEMS INC. OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#if !defined(`$INSTANCE_NAME`_H)
#define `$INSTANCE_NAME`_H

#include <cytypes.h>
#include <stdbool.h>
	
//---------------------------------------------------------------------
// define j1GLOBALQUALIFY for use with main application vs. modules
//
//	this can be used in .h files to instantiate variables
//	only one time in a project (when included from main.c)
//
//---------------------------------------------------------------------

#if !defined(j1THIS_IS_MAIN_APPLICATION)
	#define	j1GLOBALQUALIFY	extern
#else
	#define	j1GLOBALQUALIFY
#endif

//---------------------------------------------------------------------
// Specify an extension for GCC based compilers
//---------------------------------------------------------------------
	
#if defined(__GNUC__)
#define __EXTENSION __extension__
#else
#define __EXTENSION
#endif

//---------------------------------------------------------------------
// define an empty __PACKED macro if not already defined
//---------------------------------------------------------------------

#if !defined(__PACKED)
    #define __PACKED
#endif

//---------------------------------------------------------------------
// uint8/uint16/uint32 unions for each splitting
//---------------------------------------------------------------------

typedef union
{
    uint8 Val;
    struct
    {
        __EXTENSION uint8 b0:1;
        __EXTENSION uint8 b1:1;
        __EXTENSION uint8 b2:1;
        __EXTENSION uint8 b3:1;
        __EXTENSION uint8 b4:1;
        __EXTENSION uint8 b5:1;
        __EXTENSION uint8 b6:1;
        __EXTENSION uint8 b7:1;
    } bits;
} j1UINT8_VAL;

typedef union 
{
    uint16 Val;
    uint8 B[2] __PACKED;
    struct __PACKED
    {
        uint8 B0;
        uint8 B1;
    } byte;
    struct __PACKED
    {
        j1UINT8_VAL B0;
        j1UINT8_VAL B1;
    } byteUnion;
    struct __PACKED
    {
        __EXTENSION uint8 b0:1;
        __EXTENSION uint8 b1:1;
        __EXTENSION uint8 b2:1;
        __EXTENSION uint8 b3:1;
        __EXTENSION uint8 b4:1;
        __EXTENSION uint8 b5:1;
        __EXTENSION uint8 b6:1;
        __EXTENSION uint8 b7:1;
        __EXTENSION uint8 b8:1;
        __EXTENSION uint8 b9:1;
        __EXTENSION uint8 b10:1;
        __EXTENSION uint8 b11:1;
        __EXTENSION uint8 b12:1;
        __EXTENSION uint8 b13:1;
        __EXTENSION uint8 b14:1;
        __EXTENSION uint8 b15:1;
    } bits;
} j1UINT16_VAL;

typedef union
{
    uint32 Val;
    uint16 W[2] __PACKED;
    uint8  B[4] __PACKED;
    struct __PACKED
    {
        uint8 B0;
        uint8 B1;
        uint8 B2;
        uint8 B3;
    } byte;
    struct __PACKED
    {
        j1UINT8_VAL B0;
        j1UINT8_VAL B1;
        j1UINT8_VAL B2;
        j1UINT8_VAL B3;
    } byteUnion;
    struct __PACKED
    {
        uint16 W0;
        uint16 W1;
    } word;
    struct __PACKED
    {
        j1UINT16_VAL W0;
        j1UINT16_VAL W1;
    } wordUnion;
    struct __PACKED
    {
        __EXTENSION uint8 b0:1;
        __EXTENSION uint8 b1:1;
        __EXTENSION uint8 b2:1;
        __EXTENSION uint8 b3:1;
        __EXTENSION uint8 b4:1;
        __EXTENSION uint8 b5:1;
        __EXTENSION uint8 b6:1;
        __EXTENSION uint8 b7:1;
        __EXTENSION uint8 b8:1;
        __EXTENSION uint8 b9:1;
        __EXTENSION uint8 b10:1;
        __EXTENSION uint8 b11:1;
        __EXTENSION uint8 b12:1;
        __EXTENSION uint8 b13:1;
        __EXTENSION uint8 b14:1;
        __EXTENSION uint8 b15:1;
        __EXTENSION uint8 b16:1;
        __EXTENSION uint8 b17:1;
        __EXTENSION uint8 b18:1;
        __EXTENSION uint8 b19:1;
        __EXTENSION uint8 b20:1;
        __EXTENSION uint8 b21:1;
        __EXTENSION uint8 b22:1;
        __EXTENSION uint8 b23:1;
        __EXTENSION uint8 b24:1;
        __EXTENSION uint8 b25:1;
        __EXTENSION uint8 b26:1;
        __EXTENSION uint8 b27:1;
        __EXTENSION uint8 b28:1;
        __EXTENSION uint8 b29:1;
        __EXTENSION uint8 b30:1;
        __EXTENSION uint8 b31:1;
    } bits;
} j1UINT32_VAL;

//---------------------------------------------------------------------
// for easier reg modifies
//---------------------------------------------------------------------

#define	j1REG8(x)		(*(reg8 *) x)
#define	j1REG16(x)		(*(reg16 *) x)

//---------------------------------------------------------------------
// legacy code & compiler independent
//---------------------------------------------------------------------

#if `$LEGACY_DEFS`
typedef	int8	INT8;
typedef	int16	INT16;
typedef	int32	INT32;
typedef	uint8	BYTE;
typedef	uint16	WORD;
typedef	uint32	DWORD;
typedef	uint8	UINT8;
typedef	uint16	UINT16;
typedef uint32	UINT32;
typedef	bool	BOOL;
#define	FALSE	false
#define	TRUE	true

typedef	j1UINT8_VAL		BYTE_VAL;
typedef	j1UINT16_VAL	WORD_VAL;
typedef	j1UINT32_VAL	DWORD_VAL;
#endif	// LEGACY_DEFS

//---------------------------------------------------------------------
// vector sizeof
//---------------------------------------------------------------------

#define	j1VSIZEOF(x)	(sizeof(x)/sizeof(x[0]))

//---------------------------------------------------------------------
//
//	some simple macros to manage simple fifo queues
//
//		user care needs to address encapsulation in braces as needed
//
//		all items are psuedo methods and user must terminate with
//		semicolon
//
//
//		examples:
//
//			j1QUEUE(testQ,  16, BYTE, BYTE);	// declare a queue struct (BYTE queue[], BYTE count etc)
//			j1QUEUE(testQ,  16, WORD, BYTE);	// declare a queue struct (WORD queue[], BYTE count etc)
//			j1QUEUE(testQ, 512, BYTE, WORD);	// declare a queue struct (BYTE queue[], WORD count etc)
//			j1QUEUE(testQ,  32, DWORD, BYTE);	// declare a queue struct (DWORD queue[], BYTE count etc)
//
//			j1QueueInit(testQ);				// init the queue
//
//			if (j1QueueCount(testQ) ...		// using QueueCount in an if
//			x = j1QueueCount(testQ);		// getting QueueCount
//			if (j1QueueEmpty(testQ)) ...	// using QueueEmpty in an if
//			if (j1QueueAvail(testQ)) ...	// using QueueAvail in an if
//			if (j1QueueFull(testQ)) ...		// using QueueFull in an if
//
//			{ x = j1QueuePeek(testQ); }		// using QueuePeek, braces recommended
//			{ x = j1QueueGet(testQ); }		// using QueueGet, braces recommended
//			{ j1QueueAdd(testQ, x); }		// using QueueAdd, braces recommended
//
//---------------------------------------------------------------------

#define	j1QUEUE(qName, qSize, qVType, qCType) \
struct	__##qName\
{	\
	qVType	queue[qSize];	\
	qCType	next;	\
	qCType	last;	\
	qCType	count;	\
	qCType	size;	\
	int		vSize;	\
} qName

#define	j1QueueInit(qName)		qName.next = 0; qName.last = 0; qName.count = 0; qName.size = sizeof(qName.queue)/sizeof(qName.queue[0]); qName.vSize = sizeof(qName.queue[0])

#define	j1QueueCount(qName)		qName.count
#define	j1QueueEmpty(qName)		(qName.count == 0)
#define	j1QueueAvail(qName)		(qName.count > 0)
#define	j1QueueFull(qName)		(qName.count == qName.size)
#define	j1QueueNotFull(qName)	(qName.count < qName.size)
#define	j1QueueSize(qName)		(qName.size)

#define	j1QueuePeek(qName)		qName.queue[qName.next]
#define	j1QueueGet(qName)		qName.queue[qName.next++]; qName.count--; if (qName.next == qName.size) qName.next = 0
#define	j1QueueAdd(qName,val)	qName.queue[qName.last++] = (val); qName.count++; if (qName.last == qName.size) qName.last = 0
#define j1QueueAddString(qName,str) { int __idx = 0; while (str[__idx]) { if (qName.count < qName.size) { qName.queue[qName.last++] = (str[__idx]); qName.count++; if (qName.last == qName.size) qName.last = 0; } __idx++; }}

#define	j1QueuePtrInit(qName)		qName->next = 0; qName->last = 0; qName->count = 0; qName->size = sizeof(qName->queue)/sizeof(qName->queue[0]); qName->vSize = sizeof(qName->queue[0])

#define	j1QueuePtrCount(qName)		qName->count
#define	j1QueuePtrEmpty(qName)		(qName->count == 0)
#define	j1QueuePtrAvail(qName)		(qName->count > 0)
#define	j1QueuePtrFull(qName)		(qName->count == qName->size)
#define	j1QueuePtrNotFull(qName)	(qName->count < qName->size)
#define	j1QueuePtrSize(qName)		(qName->size)

#define	j1QueuePtrPeek(qName)		qName->queue[qName->next]
#define	j1QueuePtrGet(qName)		qName->queue[qName->next++]; qName->count--; if (qName->next == qName->size) qName->next = 0
#define	j1QueuePtrAdd(qName,val)	qName->queue[qName->last++] = (val); qName->count++; if (qName->last == qName->size) qName->last = 0
#define j1QueuePtrAddString(qName,str) { int __idx = 0; while (str[__idx]) { if (qName->count < qName->size) { qName->queue[qName->last++] = (str[__idx]); qName->count++; if (qName->last == qName->size) qName->last = 0; } __idx++; }}

#endif	// `$INSTANCE_NAME`_H
