//=====================================================================
//
//	`$INSTANCE_NAME`_main.c - j1UBL project design main() code
//
//	version 1.2 - 16 apr 2015
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

//---------------------------------------------------------------------
//	we are the main application - instantiate global data for instances
//---------------------------------------------------------------------

#define	j1THIS_IS_MAIN_APPLICATION

#include <project.h>
//#include "`$INSTANCE_NAME`_main.h"

//---------------------------------------------------------------------
//	internal functions
//---------------------------------------------------------------------

bool Init_ublConfigBlock();
#if PD_PARAM_ETH_ENABLED
bool Init_Eth_MAC_IP();
#endif

#if PD_PARAM_HY28B_ENABLED
void MainScreen(j1Put_CTRL *pC, uint32 fgText, uint32 bg);
#endif

#if PD_PARAM_SD_ENABLED
void uSD(j1Put_CTRL *pC);
bool uSD_GetAppInfo(j1Put_CTRL *pC, j1FAT32_CTRL *f32C, uint32 startCluster, uint32 fileSize, uint16 fileTime, uint16 fileDate, APP_INFO *appInfo);
bool uSD_FlashApp(j1Put_CTRL *pC, j1FAT32_CTRL *f32C, uint32 startCluster, uint32 fileSize, uint16 fileTime, uint16 fileDate);
bool uSD_OpenAppGet(j1FAT32_CTRL *f32C, uint32 startCluster, uint32 fileSize, j1FAT32_FCB *fcbPtr, j1Get_CTRL *getC);
bool uSD_ReadAppHeader(j1Get_CTRL *getC, APP_INFO *appInfo);
bool uSD_ReadAppFlashRow(j1Put_CTRL *pC, j1Get_CTRL *getC, APP_FLASH_ROW *appFlashRow);
#endif

//---------------------------------------------------------------------
//	main()
//---------------------------------------------------------------------

int main()
{
	j1UINT16_VAL		meta16;
	j1UINT32_VAL		meta32;
//#if PD_PARAM_HY28B_ENABLED || PD_PARAM_SD_ENABLED
//	j1Put_CTRL			j1Put_Ctrl, *pC = &j1Put_Ctrl;
//#endif

#if PD_PARAM_HY28B_ENABLED
	j1GFX_CTRL			j1GFX_Ctrl, *gC;
	PD_j1HY28B_CTRL		j1HY28B_Ctrl;
	j1Put_CTRL			j1Put_Ctrl, *pC;
#endif

	gC = &j1GFX_Ctrl;
	pC = &j1Put_Ctrl;

	/* CyGlobalIntEnable; */ /* Uncomment this line to enable global interrupts. */

	if (!Init_ublConfigBlock()) memset(&ublConfigBlock.token, 0, sizeof(ublConfigBlock.token));
	
#if PD_PARAM_HY28B_ENABLED
	if (ublConfigBlock.token.attr2.bits.hy28b) if (strcmp(ublConfigBlock.token.hy28b, "off") == 0)
	{
		gC = NULL;
		pC = NULL;
	}
#endif
	
#if PD_PARAM_ETH_ENABLED
	Init_Eth_MAC_IP();
#endif

	// partially init the currAppInfo struct
	memset(&currAppInfo, 0, sizeof(APP_INFO));
	
	meta16.Val = PD_j1UBL_GetMetadata(PD_j1UBL_GET_METADATA_BTLDB_LENGTH, 0);
	if (meta16.Val)
	{
		// GetMetaData does everything big endian, we need little endian
		currAppInfo.appLength	= ((uint16) meta16.B[0] << 8) | (uint16) meta16.B[1];
		meta16.Val				= PD_j1UBL_GetMetadata(PD_j1UBL_GET_METADATA_BTLDB_APP_ID, 0);
		currAppInfo.appID		= ((uint16) meta16.B[0] << 8) | (uint16) meta16.B[1];
		meta16.Val				= PD_j1UBL_GetMetadata(PD_j1UBL_GET_METADATA_BTLDB_APP_VERSION, 0);
		currAppInfo.appVer		= ((uint16) meta16.B[0] << 8) | (uint16) meta16.B[1];
		meta32.Val				= PD_j1UBL_GetMetadata(PD_j1UBL_GET_METADATA_BTLDB_APP_CUST_ID, 0);
		currAppInfo.appCustom	= ((uint32) meta32.B[0] << 24) | ((uint32) meta32.B[1] << 16) | ((uint32) meta32.B[2] << 8) | (uint32) meta32.B[3];
		meta32.Val				= PD_j1UBL_GetMetadata(PD_j1UBL_GET_METADATA_BTLDB_APP_TIME_DATE, 0);
		currAppInfo.appTime		= ((uint16) meta32.B[2] << 8) | (uint16) meta32.B[3];
		currAppInfo.appDate		= ((uint16) meta32.B[0] << 8) | (uint16) meta32.B[1];
	}

#if PD_PARAM_HY28B_ENABLED
#if PD_PARAM_XP_ENABLED
	if (strcmp(ublConfigBlock.token.xp, "hy28b") == 0)
	{
#endif
		if (gC)
		{
			// init the LCD
			PD_j1HY28B_Start(&j1HY28B_Ctrl, gC, 0, j1GFX_ROTATE_N, 0x0000ff, NULL);
			#if PD_PARAM_HY28B_BL_ENABLED
				PD_LCD_BL_Write(1);
			#endif
			j1Put_Start(pC, j1GFX_PutChar, gC);
			MainScreen(pC, 0xffffff, 0x0000ff);
		}
		__pC = pC;
#if PD_PARAM_XP_ENABLED
	}
	else __pC = NULL;
#endif
#endif

#if PD_PARAM_SD_ENABLED
	uSD(pC);
#endif

#if PD_PARAM_SF_ENABLED
	PD_j1SpiFlash_Start(&PD_j1SpiFlash_Ctrl);
#endif

#if (PD_PARAM_BL_SELECT == PD_PARAM_BL_SELECT_CYBL && PD_PARAM_USB_ENABLED)
	PD_BL_Start();
#endif	// (PD_PARAM_BL_SELECT == PD_PARAM_BL_SELECT_CYBL && PD_PARAM_USB_ENABLED)

#if (PD_PARAM_BL_SELECT == PD_PARAM_BL_SELECT_J1UBL)
	PD_j1UBL_Start();
#endif	// (PD_PARAM_BL_SELECT == PD_PARAM_BL_SELECT_J1UBL)

while (true) {}
}

//---------------------------------------------------------------------
//	MainScreen()
//---------------------------------------------------------------------

#if PD_PARAM_HY28B_ENABLED
void MainScreen(j1Put_CTRL *pC, uint32 fgText, uint32 bg)
{
//	j1GFX_CTRL	*gC;
#if PD_PARAM_ETH_ENABLED
	char		bfr[32];
#endif

//	gC = (j1GFX_CTRL *) pC->deviceCtrl;
	
	j1Put_FillRegion(pC, bg);
	j1Put_SetFontFgBg(pC, j1GFX_FONT_8X12, 2, fgText, bg);
	j1Put_SetTextRowCol(pC, 0, (20 - (uint8) (sizeof(PD_PARAM_DEVICE_NAME) - 1)) / 2);
	j1Put_String(pC, PD_PARAM_DEVICE_NAME);
//	j1GFX_Text(gC, j1GFX_SDL_TEXT_OPTS_HCENT, 0, 0, 320, 24, j1GFX_FONT_8X12, 2, fgText, bg, PD_PARAM_DEVICE_NAME);
	j1Put_SetFontFgBg(pC, j1GFX_FONT_8X12, 1, fgText, bg);
	j1Put_SetTextRowCol(pC, 2, 5);
	j1Put_String(pC, "Universal BootLoader (UBL) v1.0");
//	j1GFX_Text(gC, j1GFX_SDL_TEXT_OPTS_VCENT, 0, 24, 320, 12, j1GFX_FONT_8X12, 1, fgText, bg, "Universal Bootloader (UBL) v1.1");
//	j1Put_SetFontFgBg(pC, j1GFX_FONT_8X12, 1, fgText, bg);
#if PD_PARAM_ETH_ENABLED
	j1Put_SetTextRowCol(pC, 3, 5);
	j1Put_String(pC, "IP:");
	j1_iptostr(PD_ETH0_ipv4_addr.octets, bfr);
	j1Put_String(pC, bfr);
#endif
	j1Put_SetTextRowCol(pC, 14, 0);
	j1Put_String(pC, "XML: ");
	j1Put_String(pC, (char *) CY_EEPROM_BASE);
	j1Put_SetTextRowCol(pC, 17, 8);
	j1Put_String(pC, "Time:");
	j1Put_Hex16(pC, currAppInfo.appTime, j1Put_HEX_PREFIX | j1Put_PSTSPC);
	j1Put_String(pC, "Date:");
	j1Put_Hex16(pC, currAppInfo.appDate, j1Put_HEX_PREFIX);
	j1Put_SetTextRowCol(pC, 18, 2);
	j1Put_String(pC, "Dev:");
	j1Put_Hex32(pC, CYDEV_CHIP_JTAG_ID, j1Put_HEX_PREFIX | j1Put_PSTSPC);
	j1Put_String(pC, "App:");
	j1Put_Hex16(pC, currAppInfo.appID, j1Put_HEX_PREFIX | j1Put_PSTSPC);
	j1Put_String(pC, "Ver:");
	j1Put_Hex16(pC, currAppInfo.appVer, j1Put_HEX_PREFIX);
	j1Put_SetTextRowCol(pC, 19, 0);
	j1Put_String(pC, "(c)2014 Joshua 1 Systems - www.j1sys.com");
}
#endif

//---------------------------------------------------------------------
//	Init_ublConfigBlock()
//---------------------------------------------------------------------

bool Init_ublConfigBlock()
{
	uint16	eStack[1];
	int		depth;
	
	char	*xmlPtr;
	uint16	xmlLen;
	j1XML_ELEMENT_GEN(element, 8, 16, 8, 32);
	
	CyEEPROM_Start();
	CyDelay(1);
	xmlPtr 	= (char *) CY_EEPROM_BASE;
	xmlLen 	= 0;
	depth	= 0;
	
	memset(&ublConfigBlock, 0, sizeof(ublConfigBlock));
	
	while (*xmlPtr++ != 0x00) { xmlLen++; }
	if (xmlLen == 0) return false;

	// open the eeprom xml
	j1XML_Open(j1XML_IOMODE_READ_MEM, CY_EEPROM_BASE, xmlLen);
	
	// read the element
	if (j1XML_ReadElement(0, &element, false) != j1XML_RESULT_OK)
	{
		return false;
	}
	if (element.eType != j1XML_ETYPE_EMPTY) return false;
	
	// parse the element
	if (j1XML_ParseElement(&element, j1UBL_XML_eTbl_size, j1UBL_XML_eTbl, &depth, j1VSIZEOF(eStack), eStack, false) != j1XML_RESULT_OK)
	{
		return false;
	}
	
	// validate the element
	if (j1UBL_XML_Validate(&element, (j1UBL_TOKEN_XXX *) &ublConfigBlock.token, depth, eStack) != j1XML_RESULT_OK)
	{
		return false;
	}
	
	return true;
}

//---------------------------------------------------------------------
//	Init_ETH_MAC_IP()
//---------------------------------------------------------------------

#if PD_PARAM_ETH_ENABLED
	
bool Init_Eth_MAC_IP()
{
	if (ublConfigBlock.token.an)
	{
		memset(&PD_ETH0_mac_addr, 0, sizeof(PD_ETH0_mac_addr));
		PD_ETH0_Start();
		
		PD_ETH0_ipv4_addr.octets[0] = ublConfigBlock.token.an;
		PD_ETH0_ipv4_addr.octets[1] = ((PD_PARAM_ARTNET_OEM_CODE >> 8 & 0xff) + (PD_PARAM_ARTNET_OEM_CODE & 0xff) + PD_ETH0_mac_addr.v[3]) & 0xff;
		PD_ETH0_ipv4_addr.octets[2] = PD_ETH0_mac_addr.v[4];
		PD_ETH0_ipv4_addr.octets[3] = PD_ETH0_mac_addr.v[5];
	}
	else if (ublConfigBlock.token.attr1.bits.ip)
	{
		PD_ETH0_ipv4_addr.dwIP = ublConfigBlock.token.ip.dwIP;
	}
	else
	{
		PD_ETH0_ipv4_addr.dwIP = 0x0a0a0a0a;
	}
	
	memset(&PD_ETH0_mac_addr, 0, sizeof(PD_ETH0_mac_addr));
	return true;
}

#endif

//---------------------------------------------------------------------
//	uSD()
//---------------------------------------------------------------------

#if PD_PARAM_SD_ENABLED
	
void uSD(j1Put_CTRL *pC)
{
#if PD_PARAM_HY28B_ENABLED
	j1GFX_CTRL				*gC = NULL;
	char					*msg;
#endif

	PD_microSD_CTRL			microSD_Ctrl;
	j1FAT32_CTRL			j1FAT32_Ctrl , *f32C  = &j1FAT32_Ctrl;
	j1FAT32_DIR_ENTRY		dirEntry;
	uint32					startCluster;
	APP_INFO				appInfo;
	
	#if j1FAT32_PARAM_J1LFN_ENABLED
		char				lfn[255];
		#define				EXT_MASK		".cyacd"
	#else
		char				lfn[13];
		#define				EXT_MASK		".cya"
	#endif

#if PD_PARAM_HY28B_ENABLED
	if (pC)
	{
		gC = pC->deviceCtrl;
	}
	msg = "no card detected";
#endif

	PD_TickTimer_Start();

#if PD_PARAM_HY28B_ENABLED
	if (pC)
	{
		j1GFX_SetTextRowCol(gC, 4, 0);
		j1Put_String(pC, "uSD: ");
	}
#endif

	if (PD_microSD_Start(&microSD_Ctrl))
	{
		if (j1FAT32_Start(f32C, (bool (*)(void *, uint32, uint8 *)) PD_microSD_ReadSector, &microSD_Ctrl))
		{
#if PD_PARAM_HY28B_ENABLED
			msg = "no different firmware found";
#endif
			
			if (j1FAT32_GetVolumeID(f32C, lfn, sizeof(lfn)))
			{
#if PD_PARAM_HY28B_ENABLED
				if (pC)
				{
					j1Put_StringOpt(pC, lfn, j1Put_PSTSPC);
				}
#endif
			}
			
#if PD_PARAM_HY28B_ENABLED
			if (pC)
			{
				j1GFX_SetTextRowCol(gC, 5, 0);
			}
#endif

			if (j1FAT32_OpenDir(f32C, "\\boot"))
			{
				while (j1FAT32_ReadNextDirEntry(f32C, &dirEntry, lfn, sizeof(lfn), NULL, 0x00, EXT_MASK))
				{
					if (dirEntry.normal.attrib & j1FAT32_DIR_ATTRIB_HIDDEN) continue;
					if (dirEntry.normal.attrib & j1FAT32_DIR_ATTRIB_DIRECTORY) continue;

					startCluster = j1FAT32_GetStartCluster(&dirEntry);
					if (uSD_GetAppInfo(pC, f32C, startCluster, dirEntry.normal.fileSize, dirEntry.normal.time, dirEntry.normal.date, &appInfo))
					{
						if (appInfo.devID  != CYDEV_CHIP_JTAG_ID) continue;
						
						if (currAppInfo.appLength)
						{
							if (appInfo.appID     == currAppInfo.appID
							 && appInfo.appVer    == currAppInfo.appVer
							 && appInfo.appCustom == currAppInfo.appCustom
							 && appInfo.appTime   == currAppInfo.appTime
							 && appInfo.appDate   == currAppInfo.appDate) continue;
						}

#if PD_PARAM_HY28B_ENABLED
						if (pC)
						{
							j1Put_StringOpt(pC, lfn, j1Put_PSTSPC);
							j1GFX_SetTextRowCol(gC, 6, 0);
							j1Put_Hex32(pC, appInfo.devID    , j1Put_HEX_PREFIX | j1Put_PSTSPC);
							j1Put_Hex16(pC, appInfo.appID    , j1Put_HEX_PREFIX | j1Put_PSTSPC);
							j1Put_Hex16(pC, appInfo.appVer   , j1Put_HEX_PREFIX | j1Put_PSTSPC);
							j1Put_Hex32(pC, appInfo.appCustom, j1Put_HEX_PREFIX | j1Put_PSTSPC);
							j1GFX_SetTextRowCol(gC, 7, 0);
							j1Put_Hex16(pC, appInfo.appTime  , j1Put_HEX_PREFIX | j1Put_PSTSPC);
							j1Put_Hex16(pC, appInfo.appDate  , j1Put_HEX_PREFIX);
						}
#endif
						
						if (uSD_FlashApp(pC, f32C, startCluster, dirEntry.normal.fileSize, appInfo.appTime, appInfo.appDate))
						{
#if PD_PARAM_HY28B_ENABLED
							if (pC)
							{
								j1GFX_SetTextRowCol(gC, 9, 0);
								j1Put_String(pC, "Firmware Flashed. Restarting ...");
							}
#endif

							CyDelay(2000);

							if (CYRET_SUCCESS == PD_j1UBL_ValidateBootloadable(0))
							{
								PD_j1UBL_SET_RUN_TYPE(PD_j1UBL_START_APP);
							}

                			CySoftwareReset();
						}
					}
					break;
				}
			}
		}
	}
	
#if PD_PARAM_HY28B_ENABLED
	if (pC)
	{
		j1Put_String(pC, msg);
	}
#endif
}

#endif

//---------------------------------------------------------------------
//	uSD_GetAppInfo()
//---------------------------------------------------------------------

#if PD_PARAM_SD_ENABLED
	
bool uSD_GetAppInfo(j1Put_CTRL *pC, j1FAT32_CTRL *f32C, uint32 startCluster, uint32 fileSize, uint16 fileTime, uint16 fileDate, APP_INFO *appInfo)
{
	APP_FLASH_ROW	appFlashRow;
	j1FAT32_FCB		fcb, *fcbPtr = &fcb;
	j1Get_CTRL		getCtrl, *getC = &getCtrl;
	j1UINT16_VAL	u16Val;
	j1UINT32_VAL	u32Val;
	
	memset(appInfo, 0, sizeof(APP_INFO));
	
	if (uSD_OpenAppGet(f32C, startCluster, fileSize, fcbPtr, getC))
	 if (uSD_ReadAppHeader(getC, appInfo))
	{
		while (uSD_ReadAppFlashRow(pC, getC, &appFlashRow))
		{
			if (appFlashRow.arrayID != ((CYDEV_FLASH_SIZE / CYDEV_FLS_SECTOR_SIZE)   - 1)) continue;
			if (appFlashRow.rowNum  != ((CYDEV_FLS_SECTOR_SIZE / CYDEV_FLS_ROW_SIZE) - 1)) continue;
			if (appFlashRow.dataLen != CYDEV_FLS_ROW_SIZE) break;
			
			memcpy(u16Val.B, &appFlashRow.dataRow[(CYDEV_FLS_ROW_SIZE - 0x40) + 0x14], sizeof(u16Val));
			appInfo->appID		= u16Val.Val;
			memcpy(u16Val.B, &appFlashRow.dataRow[(CYDEV_FLS_ROW_SIZE - 0x40) + 0x16], sizeof(u16Val));
			appInfo->appVer		= u16Val.Val;
			memcpy(u32Val.B, &appFlashRow.dataRow[(CYDEV_FLS_ROW_SIZE - 0x40) + 0x18], sizeof(u32Val));
			appInfo->appCustom	= u32Val.Val;
			appInfo->appTime	= fileTime;
			appInfo->appDate	= fileDate;
			return true;
		}
	}
	
	return false;
}

#endif

//---------------------------------------------------------------------
//	uSD_FlashApp()
//---------------------------------------------------------------------

#if PD_PARAM_SD_ENABLED
	
bool uSD_FlashApp(j1Put_CTRL *pC, j1FAT32_CTRL *f32C, uint32 startCluster, uint32 fileSize, uint16 fileTime, uint16 fileDate)
{
#if PD_PARAM_HY28B_ENABLED
	j1GFX_CTRL		*gC;
#endif

	APP_INFO		appInfo;
	APP_FLASH_ROW	appFlashRow;
	j1FAT32_FCB		fcb, *fcbPtr = &fcb;
	j1Get_CTRL		getCtrl, *getC = &getCtrl;
	uint8			result;
	uint32			fieldPtr;
	j1UINT32_VAL	timeDate;
	
#if PD_PARAM_HY28B_ENABLED
	if (pC)
	{
		gC = pC->deviceCtrl;
	}
#endif

	if (uSD_OpenAppGet(f32C, startCluster, fileSize, fcbPtr, getC))
	 if (uSD_ReadAppHeader(getC, &appInfo))
	{
		result = CySetTemp();

		while (uSD_ReadAppFlashRow(pC, getC, &appFlashRow))
		{
#if PD_PARAM_HY28B_ENABLED
			if (pC)
			{
				j1GFX_SetTextRowCol(gC, 8, 0);
				j1Put_Hex8(pC, appFlashRow.arrayID, j1Put_PSTSPC);
				j1Put_Hex16(pC, appFlashRow.rowNum, j1Put_PSTSPC);
			}
#endif
			result = CyWriteRowFull(appFlashRow.arrayID, appFlashRow.rowNum, appFlashRow.dataRow, appFlashRow.dataLen);
			if (result != CYRET_SUCCESS) return false;
#if PD_PARAM_HY28B_ENABLED
			if (pC)
			{
				j1Put_String(pC, "+ ");
			}
#endif
		}
		
	    fieldPtr  = PD_j1UBL_MD_BTLDB_APP_TIME_DATE_OFFSET(0);
		timeDate.W[0] = fileTime;
		timeDate.W[1] = fileDate;
		PD_j1UBL_SetFlashBytes(fieldPtr, timeDate.B, sizeof(timeDate));

#if PD_PARAM_HY28B_ENABLED
		if (pC)
		{
			j1GFX_SetTextRowCol(gC, 11, 0);
			j1Put_Hex32(pC, fieldPtr, j1Put_HEX_PREFIX | j1Put_PSTSPC);
			j1Put_Hex16(pC, fileTime, j1Put_HEX_PREFIX | j1Put_PSTSPC);
			j1Put_Hex16(pC, fileDate, j1Put_HEX_PREFIX | j1Put_PSTSPC);
		}
#endif
		return true;
	}
	
	return false;
}

#endif

//---------------------------------------------------------------------
//	uSD_OpenAppGet()
//---------------------------------------------------------------------

#if PD_PARAM_SD_ENABLED

bool uSD_OpenAppGet(j1FAT32_CTRL *f32C, uint32 startCluster, uint32 fileSize, j1FAT32_FCB *fcbPtr, j1Get_CTRL *getC)
{
	if (j1FAT32_OpenFileCluster(f32C, startCluster, fileSize, fcbPtr))
	 if (j1Get_Start(getC, j1FAT32_ReadFile, fcbPtr))
	  return true;
	
	return false;
}

#endif

//---------------------------------------------------------------------
//	uSD_ReadAppHeader()
//---------------------------------------------------------------------

#if PD_PARAM_SD_ENABLED
	
bool uSD_ReadAppHeader(j1Get_CTRL *getC, APP_INFO *appInfo)
{
	char	lineBfr[32];
	uint16	lineLen;
	
	if (j1Get_Line(getC, lineBfr, sizeof(lineBfr), &lineLen))
	{
		if (lineLen == 12)
		 if (j1_hexfldtouint32(&lineBfr[0], &appInfo->devID))
		  if (j1_hexfldtouint8(&lineBfr[8], &appInfo->devRev))
		   if (j1_hexfldtouint8(&lineBfr[10], &appInfo->chkSum))
		{
			return true;
		}
	}

	return false;
}

#endif

//---------------------------------------------------------------------
//	uSD_ReadAppFlashRow()
//---------------------------------------------------------------------

#if PD_PARAM_SD_ENABLED

bool uSD_ReadAppFlashRow(j1Put_CTRL *pC, j1Get_CTRL *getC, APP_FLASH_ROW *appFlashRow)
{
	char	lineBfr[2 * CYDEV_FLS_ROW_SIZE + 32];
	uint16	lineLen;
	uint8	chkSum;
	char	*bfrPtr;
	uint8	*dataPtr;
	uint8	dataByte;
	uint16	i;
	
	if (j1Get_Line(getC, lineBfr, sizeof(lineBfr), &lineLen))
	{
		if (lineLen >= 13)
		 if (lineBfr[0] == ':')
		  if (j1_hexfldtouint8(&lineBfr[1], &appFlashRow->arrayID))
		   if (j1_hexfldtouint16(&lineBfr[3], &appFlashRow->rowNum))
		    if (j1_hexfldtouint16(&lineBfr[7], &appFlashRow->dataLen))
			 if (appFlashRow->dataLen <= CYDEV_FLS_ROW_SIZE)
			  if (lineLen == 2 * appFlashRow->dataLen + 13)
			{
				bfrPtr	= &lineBfr[11];
				dataPtr	= appFlashRow->dataRow;
				chkSum	= appFlashRow->arrayID + (appFlashRow->rowNum >> 8) + (appFlashRow->rowNum & 0xff) + (appFlashRow->dataLen >> 8) + (appFlashRow->dataLen & 0xff);
				i		= appFlashRow->dataLen;
				while (i--)
				{
					if (!j1_hexfldtouint8(bfrPtr, &dataByte)) return false;
					*dataPtr++	 = dataByte;
					chkSum		+= dataByte;
					bfrPtr		+= 2;
				}
				
				if (j1_hexfldtouint8(bfrPtr, &dataByte))
				{
					appFlashRow->chkSum = dataByte;
					chkSum += dataByte;
					if (chkSum == 0) return true;
				}
			}
	}
	
	return false;
}

#endif
