/*=====================================================================
/
/	`$INSTANCE_NAME`.h - header file for j1sys XML
/
/		simple xml reader, parser, writer
/
/		also includes a command buffer parser as a psuedo
/		xml element
/
/====================================================================*/

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#ifndef `$INSTANCE_NAME`_H		// include just once
#define `$INSTANCE_NAME`_H

#include <stdbool.h>

#include <cytypes.h>
#include <cyfitter.h>
#include <CyLib.h>

#include <j1Types.h>

//#include <j1Conversion.h>

//---------------------------------------------------------------------
//
//	the following allows for external flash
//
//---------------------------------------------------------------------

//#define j1XML_EXTFLASH

//---------------------------------------------------------------------
//
//	xml specs say that element and attribute names have to be case
//	sensitive. to make it easier on hand editing we allow for forcing
//	element and attribute names to all lower case if the following
//	definition is true.
//
//	xml specs say that an attribute HAS to have a value. the option
//	of using it without is called bareword. it might help on some
//	tight flags.
//
//	there are separate controls for j1XML_CMD psuedo element
//
//---------------------------------------------------------------------

#define	j1XML_PARAM_LOWER_CASE_NAMES		`$LOWER_CASE_NAMES`
#define	j1XML_PARAM_ALLOW_BARE_WORD			`$ALLOW_BARE_WORD`

#define j1XML_PARAM_CMD_LOWER_CASE_NAMES	`$CMD_LOWER_CASE_NAMES`
#define	j1XML_PARAM_CMD_ALLOW_BARE_WORD		`$CMD_ALLOW_BARE_WORD`

//---------------------------------------------------------------------
//
//	j1XML_RESULT - enum for all function call results
//
//---------------------------------------------------------------------

typedef enum
{
	j1XML_RESULT_OK = 0,

	j1XML_RESULT_OPEN_ERROR,
	j1XML_RESULT_CLOSE_ERROR,

	j1XML_RESULT_READ_EOF,
	j1XML_RESULT_READ_NOT_OPENED_CORRECTLY,
	j1XML_RESULT_READ_ERROR,

	j1XML_RESULT_READ_PARSE_ERROR_TEXT_AFTER,
	j1XML_RESULT_READ_PARSE_ERROR_OPEN_IS_LAST,
	j1XML_RESULT_READ_PARSE_ERROR_COMMENT_TOO_BIG,
	j1XML_RESULT_READ_PARSE_ERROR_OPEN_COMMENT,

	j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_FIRST_NOT_ALPHA,
	j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_TOO_BIG,
	j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_NOT_ALPHANUMERIC,
	j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_EMPTY,

	j1XML_RESULT_READ_PARSE_ERROR_COMMAND_FIRST_NOT_ALPHA,
	j1XML_RESULT_READ_PARSE_ERROR_COMMAND_TOO_BIG,
	j1XML_RESULT_READ_PARSE_ERROR_COMMAND_NOT_ALPHANUMERIC,
	j1XML_RESULT_READ_PARSE_ERROR_COMMAND_EMPTY,

	j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_TOO_MANY,
	j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_FIRST_NOT_ALPHA,
	j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_TOO_BIG,
	j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_NOT_ALPHANUMERIC,
	j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_EMPTY,
	j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_VALUE_INVALID_QUOTE,
	j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_VALUE_TOO_BIG,

	j1XML_RESULT_READ_PARSE_ERROR_DECL_END_MISMATCH,
	j1XML_RESULT_READ_PARSE_ERROR_NO_CHAR_AFTER_DECL_END,
	j1XML_RESULT_READ_PARSE_ERROR_NO_ANGLE_AFTER_DECL_END,
	j1XML_RESULT_READ_PARSE_ERROR_EMPTY_END_MISMATCH,
	j1XML_RESULT_READ_PARSE_ERROR_NO_CHAR_AFTER_EMPTY_END,
	j1XML_RESULT_READ_PARSE_ERROR_NO_ANGLE_AFTER_EMPTY_END,
	j1XML_RESULT_READ_PARSE_ERROR_DECL_MISSING_END,
	j1XML_RESULT_READ_PARSE_ERROR_COMM_LOGIC,
	j1XML_RESULT_READ_PARSE_ERROR_EMPTY_LOGIC,
	j1XML_RESULT_READ_PARSE_ERROR_LOGIC,

	j1XML_RESULT_WRITE_NOT_OPENED_CORRECTLY,
	j1XML_RESULT_WRITE_NO_ROOM,
	j1XML_RESULT_WRITE_UNKNOWN_ETYPE,
	j1XML_RESULT_WRITE_NO_POSSIBLE_QUOTE,
	j1XML_RESULT_WRITE_ETYPE_LOGIC,
	j1XML_RESULT_WRITE_ERROR,

	j1XML_RESULT_PARSE_ELEM_INVALID,
	j1XML_RESULT_PARSE_CMD_INVALID,
	j1XML_RESULT_PARSE_ATTR_INVALID,
	j1XML_RESULT_PARSE_ATTR_DUPE,
	j1XML_RESULT_PARSE_ATTR_MINLEN,
	j1XML_RESULT_PARSE_ATTR_MAXLEN,
	j1XML_RESULT_PARSE_ATTR_ATYPE,
	j1XML_RESULT_PARSE_ATTR_STRING_INVALID,
	j1XML_RESULT_PARSE_ATTR_BYTE_INVALID,
	j1XML_RESULT_PARSE_ATTR_WORD_INVALID,
	j1XML_RESULT_PARSE_ATTR_DWORD_INVALID,
	j1XML_RESULT_PARSE_ATTR_INT16_INVALID,
	j1XML_RESULT_PARSE_ATTR_INT32_INVALID,
	j1XML_RESULT_PARSE_ATTR_BOOL_INVALID,
	j1XML_RESULT_PARSE_ATTR_IP_INVALID,
	j1XML_RESULT_PARSE_ATTR_MAC_INVALID,
	j1XML_RESULT_PARSE_ESTACK_EXCEEDED,
	j1XML_RESULT_PARSE_ESTACK_EXHAUSTED,
	j1XML_RESULT_PARSE_ESTACK_MISMATCH,
	j1XML_RESULT_PARSE_ETYPE_ERROR,

	j1XML_RESULT_VALIDATE_INVALID,

	j1XML_RESULT_SET_ELEMENT_NAME_ERROR,
	j1XML_RESULT_SET_ATTRIBUTE_ERROR,

	j1XML_RESULT__MAXVALUE__

} j1XML_RESULT;

//---------------------------------------------------------------------
//
//	open/close structures
//
//---------------------------------------------------------------------

typedef enum
{
	j1XML_IOMODE_CLOSED = 0,

	j1XML_IOMODE_READ_MEM,
	j1XML_IOMODE_READ_EXTFLASH,

	j1XML_IOMODE_WRITE_MEM,
	j1XML_IOMODE_WRITE_EXTFLASH
} j1XML_IOMODE;

//---------------------------------------------------------------------
//
//	element structures - used for read/write
//
//		this is a control structure for use with the reader/writer
//		it contains pointers to data areas along with the
//		size/limits of the data areas. consistency of size vs buffers
//		is VERY critical!!!
//
//		to reduce the footprint for embedded use we declare maximum
//		sizes of element name, attribute name, attribute value and
//		maximum number of name/value attribute pairs. this can vary
//		on a case by case basis as needed.
//
//		for safety zero fill the structure before filling your fields.
//		that way any defaults of zero/null SHOULD be safe.
//
//---------------------------------------------------------------------

//-------------------------------------------------
//	this will create and init an element structure
//	element variable name will be prepended with an
//	'_' to keep macro preprocessor happy
//
//	no trailing ';' so user has to add one
//-------------------------------------------------

#define j1XML_E_AID		uint16
#define	j1XML_E_VAL		uint32

#define j1XML_ELEMENT_GEN(v, _eNameSize, _aCount, _aNameSize, _aValueSize) \
	char	eNameBfr_##v[_eNameSize]; \
	char	aBfr_##v[_aCount * (_aNameSize + _aValueSize)]; \
	j1XML_E_AID		aIDTbl_##v[_aCount]; \
	j1XML_E_VAL		vTbl_##v[_aCount]; \
	j1XML_ELEMENT	v; \
	memset(&v, 0, sizeof(j1XML_ELEMENT)); \
	v.eNameSize = _eNameSize; \
	v.eNameBfr = eNameBfr_##v; \
	v.aCount = _aCount; \
	v.aNameSize = _aNameSize; \
	v.aValueSize = _aValueSize; \
	v.aBfr = aBfr_##v; \
	v.aIDTbl = aIDTbl_##v; \
	v.vTbl = vTbl_##v; \
	memset(aBfr_##v, 0, _aCount * (_aNameSize + _aValueSize)); \
	memset(vTbl_##v, 0, sizeof(vTbl_##v))
	
typedef enum
{
	j1XML_ETYPE_DECL = 0,
	j1XML_ETYPE_COMM,
	j1XML_ETYPE_START,
	j1XML_ETYPE_STOP,
	j1XML_ETYPE_EMPTY,
	j1XML_ETYPE_CMD			// command buffer as a psuedo element
} j1XML_ETYPE;

typedef struct _j1XML_ELEMENT
{
	uint32		tStartOffset;	// offset to first text (pre-element)
	uint32		tEndOffset;		// offset to last text (pre-element)

	uint32		eStartOffset;	// offset to opening '<' in datastream
	uint32		eEndOffset;		// offset to closing '>' in datastream

	j1XML_ETYPE	eType;			// element type

	int			eNameSize;		// element name bfr size
	char		*eNameBfr;		// element name bfr

	int			aCount;			// # of attribute rows in aTable
	int			aNameSize;		// attribute name bfr size
	int			aValueSize;		// attribute value bfr size
	char		*aBfr;			// the entire attribute table
								// MUST be count * (nameSize + valueSize)
								// stored in rows of name,value data
								// as a trick - comments are stored in
								// the same buffer (upto total room)

	uint16		eID;			// element ID
	j1XML_E_AID	*aIDTbl;		// attribute ID table

	j1XML_E_VAL	*vTbl;			// pointer to values converted/validated
								// table size is assumed the same as aCount
								// values is stored as a dword but
								// should be cast to its proper
								// value. numeric values are implied.
								// the string attributes that have a
								// valid string table will store an index (0-x)
								// value to indicate which string matched

} j1XML_ELEMENT;

//---------------------------------------------------------------------
//
//	dictionary structures - these will be used by validate/convert
//
//---------------------------------------------------------------------

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR_GEN(__vName, __attrID, __name, __aType, __minLen, __maxLen, __vCnt, __vTbl) \
	const	j1XML_DICT_ATTR const __vName = { __attrID, __name, __aType, __minLen, __maxLen, __vCnt, __vTbl }
#else
#define j1XML_DICT_ATTR_GEN(__vName, __attrID, __name, __aType, __minLen, __maxLen, __vCnt, __vTbl) \
	extern	j1XML_DICT_ATTR const __vName
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR1_GEN(__vName, p1) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1 }
#else
#define j1XML_DICT_ATTR1_GEN(__vName, p1) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR2_GEN(__vName, p1, p2) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2 }
#else
#define j1XML_DICT_ATTR2_GEN(__vName, p1, p2) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR3_GEN(__vName, p1, p2, p3) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3 }
#else
#define j1XML_DICT_ATTR3_GEN(__vName, p1, p2, p3) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR4_GEN(__vName, p1, p2, p3, p4) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4 }
#else
#define j1XML_DICT_ATTR4_GEN(__vName, p1, p2, p3, p4) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR5_GEN(__vName, p1, p2, p3, p4, p5) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5 }
#else
#define j1XML_DICT_ATTR5_GEN(__vName, p1, p2, p3, p4, p5) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR6_GEN(__vName, p1, p2, p3, p4, p5, p6) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6 }
#else
#define j1XML_DICT_ATTR6_GEN(__vName, p1, p2, p3, p4, p5, p6) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR7_GEN(__vName, p1, p2, p3, p4, p5, p6, p7) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7 }
#else
#define j1XML_DICT_ATTR7_GEN(__vName, p1, p2, p3, p4, p5, p6, p7) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR8_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8 }
#else
#define j1XML_DICT_ATTR8_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR9_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9 }
#else
#define j1XML_DICT_ATTR9_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR10_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10 }
#else
#define j1XML_DICT_ATTR10_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR11_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11 }
#else
#define j1XML_DICT_ATTR11_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR12_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11, &p12 }
#else
#define j1XML_DICT_ATTR12_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR13_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11, &p12, &p13 }
#else
#define j1XML_DICT_ATTR13_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR14_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11, &p12, &p13, &p14 }
#else
#define j1XML_DICT_ATTR14_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR15_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11, &p12, &p13, &p14, &p15 }
#else
#define j1XML_DICT_ATTR15_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR16_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11, &p12, &p13, &p14, &p15, &p16 }
#else
#define j1XML_DICT_ATTR16_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR17_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16, p17) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11, &p12, &p13, &p14, &p15, &p16, &p17 }
#else
#define j1XML_DICT_ATTR17_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16, p17) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR18_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16, p17, p18) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11, &p12, &p13, &p14, &p15, &p16, &p17, &p18 }
#else
#define j1XML_DICT_ATTR18_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16, p17, p18) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR19_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16, p17, p18, p19) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11, &p12, &p13, &p14, &p15, &p16, &p17, &p18, &p19 }
#else
#define j1XML_DICT_ATTR19_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16, p17, p18, p19) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ATTR20_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16, p17, p18, p19, p20) \
	const	j1XML_DICT_ATTR const * const __vName[] = { &p1, &p2, &p3, &p4, &p5, &p6, &p7, &p8, &p9, &p10, &p11, &p12, &p13, &p14, &p15, &p16, &p17, &p18, &p19, &p20 }
#else
#define j1XML_DICT_ATTR20_GEN(__vName, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16, p17, p18, p19) \
	extern	j1XML_DICT_ATTR const * const __vName[]
#endif

#ifdef j1THIS_IS_MAIN_APPLICATION
#define j1XML_DICT_ELEM_GEN(__vName, __elemID, __name, __aTblSize, __aTbl) \
	const	j1XML_DICT_ELEM const __vName = { __elemID, __name, __aTblSize, __aTbl }
#else
#define j1XML_DICT_ELEM_GEN(__vName, __elemID, __name, __aTbleSize, __aTbl) \
	extern	j1XML_DICT_ELEM const __vName
#endif

typedef enum
{
	j1XML_DICT_ATYPE_BARE = 0,
	j1XML_DICT_ATYPE_STRING,
	j1XML_DICT_ATYPE_CHAR,
	j1XML_DICT_ATYPE_BYTE,
	j1XML_DICT_ATYPE_HEXBYTE,
	j1XML_DICT_ATYPE_WORD,
	j1XML_DICT_ATYPE_DWORD,
	j1XML_DICT_ATYPE_INT16,
	j1XML_DICT_ATYPE_INT32,
	j1XML_DICT_ATYPE_BOOL,
	j1XML_DICT_ATYPE_IP,
	j1XML_DICT_ATYPE_MAC,
} j1XML_DICT_ATYPE;

typedef struct _j1XML_DICT_ATTR
{
	j1XML_E_AID			attrID;		// attribute id to make switch/case work
	char				*name;		// attribute name
	j1XML_DICT_ATYPE	aType;		// attribute type
	uint8				minLen;		// attribute min text length
	uint8				maxLen;		// attribute max text length
	uint8				vCnt;		// attribute validation count
	const void const * const vTbl;	// attribute validation table
									// for strings it is a table
									// of 'vCnt' string pointers
									// for numbers it is a table
									// of 'vCnt=2' pointers to
									// two numbers which represent
									// the lo/hi range of valid numbers
} j1XML_DICT_ATTR;

typedef struct _j1XML_DICT_ELEM
{
	uint16						elemID;		// element id to make switch/case work
	char						*name;		// element name
	uint8						aCnt;		// element attribute dictionary count
	const j1XML_DICT_ATTR const	* const * const aTbl;		// element attribute dictionary table
} j1XML_DICT_ELEM;

//---------------------------------------------------------------------
//	function prototypes
//---------------------------------------------------------------------

j1XML_RESULT	j1XML_Open(j1XML_IOMODE, uint32 baseOffset, uint32 size);
bool			j1XML_ReadChar(uint32 *offset, char *value, bool skipWhite);
bool			j1XML_WriteChar(uint32 *offset, char value);
bool			j1XML_WriteString(uint32 *offset, char *value);
j1XML_RESULT	j1XML_Close();

j1XML_RESULT	j1XML_ReadElement(uint32 offset, j1XML_ELEMENT *element, bool cmdParse);
j1XML_RESULT	j1XML_WriteElement(uint32 offset, uint8 inset, j1XML_ELEMENT *element, uint32 *nextOffset);

j1XML_RESULT	j1XML_ParseElement(j1XML_ELEMENT *element, int eTbl_Size, const j1XML_DICT_ELEM const * const *eTbl, int *depth, int maxDepth, uint16 *eStack, bool cmdParse);

j1XML_RESULT	j1XML_SetElementName(j1XML_ELEMENT *element, j1XML_DICT_ELEM const *dictElem);
j1XML_RESULT	j1XML_SetAttribute(j1XML_ELEMENT *element, uint16 *aCnt, void *valPtr, j1XML_DICT_ATTR const *dictAttr);

char 			*j1XML_eTypeDecode(j1XML_ETYPE eType);
char 			*j1XML_ErrorDecode(j1XML_RESULT result);

void			j1XML_Element_etxtcpy(char *bfr, int bfrSize, j1XML_ELEMENT *element);

j1XML_RESULT	j1XML_StrToIP(char *valPtr, int valLen, uint8 *ip);
j1XML_RESULT	j1XML_StrToMAC(char *valPtr, int valLen, uint8 *mac);

#endif		// j1XML_H
