//=====================================================================
//
//	`$INSTANCE_NAME`.c - microSD Driver
//
//	version 1.0 - 15 feb 2015
//
//=====================================================================


//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#include "`$INSTANCE_NAME`.h"

//---------------------------------------------------------------------
//	timeout magic
//---------------------------------------------------------------------

//extern	`$TICK_TYPE`	`$TICK_FUNC`
#define	TICK_TYPE	`$TICK_TYPE`
#define	TICK_FUNC	`$TICK_FUNC`
#if `$TICK_ASCEND`
#define TICKS(x)	(TICK_FUNC - x)
#else
#define TICKS(x)	(x - TICK_FUNC)
#endif
#define TIMEOUT(x,y)	(TICKS(x) > y)

//---------------------------------------------------------------------
//	private defines
//---------------------------------------------------------------------

#define	CMD_GO_IDLE			(0x00)			// go idle
#define	CMD_SEND_OP			(0x01)			// send op cond (mmc)
#define	CMD_SEND_OP_APP		(0x80 | 0x29)	// send op cond (sdc)
#define	CMD_SEND_IF			(0x08)			// send if cond
#define	CMD_SEND_CSD		(0x09)			// send csd
#define	CMD_SEND_CID		(0x0a)			// send cid
#define	CMD_STOP			(0x0c)			// stop transmission
#define	CMD_STATUS_APP		(0x80 | 0x0d)	// status (sdc)
#define	CMD_BLOCKLEN		(0x10)			// set blocklen
#define	CMD_READ			(0x11)			// read block
#define	CMD_READ_MUL		(0x12)			// read multiple blocks
#define	CMD_BLOCKCNT		(0x17)			// set blockcnt (mmc)
#define	CMD_BLOCKCNT_APP	(0x80 | 0x17)	// set blockcnt (sdc)
#define	CMD_WRITE			(0x18)			// write block
#define	CMD_WRITE_MUL		(0x19)			// write multiple blocks
#define	CMD_ERASE_BLKSTART	(0x20)			// erase block start
#define	CMD_ERASE_BLKEND	(0x21)			// erase block end
#define	CMD_ERASE			(0x26)			// erase
#define	CMD_APP				(0x37)			// app cmd
#define	CMD_READ_OCR		(0x3a)			// read ocr

#define	CMD_ARG_HCS			(1ul << 30)		// hcs argument

//---------------------------------------------------------------------
//	private functions
//---------------------------------------------------------------------

static bool		InitCard(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl);
static bool		SelectCard(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl);
static void		DeselectCard(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl);
static bool		WaitReady(`$INSTANCE_NAME`_CTRL *j1micrpSD_Ctrl);
static uint8	SendCmd(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl, uint8 cmd, uint32 val);
static uint8	WriteReadSPI(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl, uint8 writeVal);
static bool		ReadDataPacket(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl, uint8 *bfrPtr, uint16 len);
static void		SlowSPI(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl);
static void		FastSPI(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl);

//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_Start() - startup the component
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_Start(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl)
{
	bool	result;
	
//	j1PC = j1Put_Ctrl;
	
//	j1Put_String(j1PC, "iSPIM_Start()/", 0);
	`$INSTANCE_NAME`_iSPIM_Start();
//	j1Put_String(j1PC, "Done ", 0);
	
//	j1Put_String(j1PC, "InitCard() ", 0);
	result = InitCard(j1microSD_Ctrl);
//	j1Put_String(j1PC, "_InitCard ", 0);
	
	return result;
}

//---------------------------------------------------------------------
//	`$INSTANCE_NAME`_ReadSector() - read a sector from the card
//---------------------------------------------------------------------

bool `$INSTANCE_NAME`_ReadSector(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl, uint32 lba, uint8 *bfrPtr)
{
	if (j1microSD_Ctrl->cardType == 0) return false;
	
	if (!(j1microSD_Ctrl->cardType & `$INSTANCE_NAME`_CARD_TYPE_BLOCK)) lba *= 512;
	
	if (SendCmd(j1microSD_Ctrl, CMD_READ, lba) == 0)
	{
		return ReadDataPacket(j1microSD_Ctrl, bfrPtr, 512);
	}
	
	return false;
}

//---------------------------------------------------------------------
//	InitCard() - initialize the card
//---------------------------------------------------------------------

static bool InitCard(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl)
{
	uint8		i, cmd, ocr[4];
	TICK_TYPE	tickStart;

	SlowSPI(j1microSD_Ctrl);		// set spi clock to low speed
	
	`$SS_INSTANCE`_Write(1);		// make sure SS is high
	
//	j1Put_String(j1PC, "Dum80 ", 0);
	// send 80 dummy clocks
	for (i = 0; i < 10; i++) WriteReadSPI(j1microSD_Ctrl, 0xff);
	
	j1microSD_Ctrl->cardType = 0x00;
	
//	j1Put_String(j1PC, "GO_IDLE ", 0);
	if (SendCmd(j1microSD_Ctrl, CMD_GO_IDLE,0) != 1) return false;
//	j1Put_String(j1PC, "! ", 0);

	tickStart = TICK_FUNC;
//	j1Put_String(j1PC, "tick=", 0);
//	j1Put_Hex16(j1PC, tickStart, j1Put_PSTSPC);
	
//	j1Put_String(j1PC, "SDV2? ", 0);
	if (SendCmd(j1microSD_Ctrl, CMD_SEND_IF, 0x1aa) == 1)	// SDV2?
	{
//		j1Put_String(j1PC, "Y,OCR=", 0);
		// read in the 32 bit OCR
		for (i = 0; i < 4; i++)
		{
			ocr[i] = WriteReadSPI(j1microSD_Ctrl, 0xff);
//			j1Put_Hex8(j1PC, ocr[i], j1Put_PSTSPC);
		}
		
		if (ocr[2] == 0x01 && ocr[3] == 0xaa)
		{
			// wait for end of initialization
//			j1Put_String(j1PC, "EOI",0);
			while (!TIMEOUT(tickStart, `$TICKS_INIT`) && SendCmd(j1microSD_Ctrl, CMD_SEND_OP_APP, CMD_ARG_HCS)) {}
//			j1Put_String(j1PC, "! ", 0);
			
			// check ccs bit in the ocr
//			j1Put_String(j1PC, "ReadOCR ", 0);
			if (!TIMEOUT(tickStart, `$TICKS_INIT`)) if (SendCmd(j1microSD_Ctrl, CMD_READ_OCR, 0) == 0)
			{
				j1microSD_Ctrl->cardType = `$INSTANCE_NAME`_CARD_TYPE_SD2;
				
//				j1Put_String(j1PC, "OCR=", 0);
				for (i = 0; i < 4; i++)
				{
					ocr[i] = WriteReadSPI(j1microSD_Ctrl, 0xff);
//					j1Put_Hex8(j1PC, ocr[i], j1Put_PSTSPC);
				}
				if (ocr[0] & 0x40) j1microSD_Ctrl->cardType = `$INSTANCE_NAME`_CARD_TYPE_SD2 | `$INSTANCE_NAME`_CARD_TYPE_BLOCK;
							  else j1microSD_Ctrl->cardType = `$INSTANCE_NAME`_CARD_TYPE_SD2;
			}
		}
	}
	else
	{
//		j1Put_String(j1PC, "SDV1orMMC? ",0);
		if (SendCmd(j1microSD_Ctrl, CMD_SEND_OP_APP, 0) <= 1)		// SDV1 or MMC?
		{
//			j1Put_String(j1PC, "SD1 ", 0);
			j1microSD_Ctrl->cardType	= `$INSTANCE_NAME`_CARD_TYPE_SD1;
			cmd							= CMD_SEND_OP_APP;
		}
		else
		{
//			j1Put_String(j1PC, "MMC ", 0);
			j1microSD_Ctrl->cardType	= `$INSTANCE_NAME`_CARD_TYPE_MMC;
			cmd							= CMD_SEND_OP;
		}
		
		// wait for end of initialization
//		j1Put_String(j1PC, "EOI",0);
		while (!TIMEOUT(tickStart, `$TICKS_INIT`) && SendCmd(j1microSD_Ctrl, cmd, 0)) {}
//		j1Put_String(j1PC, "! ", 0);
		
		if (TIMEOUT(tickStart, `$TICKS_INIT`)) j1microSD_Ctrl->cardType = 0;
		else
		{
			// set block length to 512
			if (SendCmd(j1microSD_Ctrl, CMD_BLOCKLEN, 512) != 0)
			{
				j1microSD_Ctrl->cardType = 0;
			}
		}
	}
	
	DeselectCard(j1microSD_Ctrl);
	
//	j1Put_String(j1PC, "tick=", 0);
//	j1Put_Hex16(j1PC, TICK_FUNC, j1Put_PSTSPC);

//	j1Put_String(j1PC, "cT=", 0);
//	j1Put_Hex8(j1PC, cardType, j1Put_PSTSPC);

	if (j1microSD_Ctrl->cardType)
	{
		FastSPI(j1microSD_Ctrl);
		return true;
	}
	
	return false;
}

//---------------------------------------------------------------------
//	SelectCard() - select the card
//---------------------------------------------------------------------

static bool SelectCard(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl)
{
	`$SS_INSTANCE`_Write(0);			// take it low
	WriteReadSPI(j1microSD_Ctrl,0xff);	// and send out dummy clocks
	
	if (WaitReady(j1microSD_Ctrl)) return true;
	
	DeselectCard(j1microSD_Ctrl);
	return false;
}

//---------------------------------------------------------------------
//	DeselectCard() - deselect the card
//---------------------------------------------------------------------

static void DeselectCard(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl)
{
	`$SS_INSTANCE`_Write(1);			// take it high
	WriteReadSPI(j1microSD_Ctrl, 0xff);	// and send out dummy clocks
}

//---------------------------------------------------------------------
//	WaitReady() - wait for card to come ready
//---------------------------------------------------------------------

static bool WaitReady(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl)
{
	uint8		readData;
	TICK_TYPE	tickStart;
	
	tickStart = TICK_FUNC;
	
	do
	{
		readData = WriteReadSPI(j1microSD_Ctrl, 0xff);
	}
	while (readData != 0xff && !TIMEOUT(tickStart, `$TICKS_INIT`));
	
//	j1Put_Hex8(j1PC, readData, j1Put_PSTSPC);
	
	if (TIMEOUT(tickStart, `$TICKS_INIT`)) return false;
	return true;
}

//---------------------------------------------------------------------
//	SendCmd() - send a command and get response
//---------------------------------------------------------------------

static uint8 SendCmd(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl, uint8 cmd, uint32 val)
{
	uint8	i, res;
	
	if (cmd & 0x80)
	{
		cmd &= 0x7f;
		res = SendCmd(j1microSD_Ctrl, CMD_APP, 0);
		if (res > 1) return res;
	}
	
	if (cmd != CMD_STOP)
	{
//		j1Put_String(j1PC, "Deselect() ", 0);
		DeselectCard(j1microSD_Ctrl);
//		j1Put_String(j1PC, "Select() ", 0);
		if (!SelectCard(j1microSD_Ctrl)) return 0xff;
//		j1Put_String(j1PC, "! ", 0);
	}
	
//	j1Put_String(j1PC, "cmd=", 0);
//	j1Put_Hex8(j1PC, cmd, j1Put_PSTSPC);
	
	// send command packet
	WriteReadSPI(j1microSD_Ctrl, 0x40 | cmd);		// start bit + command
	
	WriteReadSPI(j1microSD_Ctrl, val >> 24);		// val[31..24]
	WriteReadSPI(j1microSD_Ctrl, val >> 16);		// val[23..16]
	WriteReadSPI(j1microSD_Ctrl, val >>  8);		// val[15...8]
	WriteReadSPI(j1microSD_Ctrl, val      );		// val[7....0]
	
	i = 0x01;										// dummy CRC + stop
	if (cmd == CMD_GO_IDLE) i = 0x95;				// valid CRC + stop
	if (cmd == CMD_SEND_IF) i = 0x87;				// valid CRC + stop
	WriteReadSPI(j1microSD_Ctrl, i);				// send it
	
	// receive command response
	if (cmd == CMD_STOP) WriteReadSPI(j1microSD_Ctrl, 0xff);	// discard one byte for stop
	i = 10;											// max 10 bytes to get response
	do
	{
		res = WriteReadSPI(j1microSD_Ctrl, 0xff);
	}
	while ((res & 0x80) && --i);

//	if (cmd == CMD_READ_OCR)
//	{
//		j1Put_String(j1PC, "res=", 0);
//		j1Put_Hex8(j1PC, res, j1Put_PSTSPC);
//	}
	
	return res;							// return the response
}

//---------------------------------------------------------------------
//	WriteReadSPI() - write a byte to spi and get read
//---------------------------------------------------------------------

static uint8 WriteReadSPI(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl, uint8 writeVal)
{
	`$INSTANCE_NAME`_iSPIM_ClearTxBuffer();
	`$INSTANCE_NAME`_iSPIM_ClearRxBuffer();
	`$INSTANCE_NAME`_iSPIM_WriteTxData(writeVal);
	while (`$INSTANCE_NAME`_iSPIM_GetRxBufferSize() == 0) {}
	return `$INSTANCE_NAME`_iSPIM_ReadRxData();
	
//    *`$INSTANCE_NAME`_iSPIM_TXDATA_PTR = writeVal;
//	while ((`$INSTANCE_NAME`_iSPIM_TX_STATUS_REG & `$INSTANCE_NAME`_iSPIM_STS_SPI_DONE) == 0) {}
//	return *`$INSTANCE_NAME`_iSPIM_RXDATA_PTR;
}

//---------------------------------------------------------------------
//	ReadDataPacket() - read in a data packet after a read command
//---------------------------------------------------------------------

static bool ReadDataPacket(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl, uint8 *bfrPtr, uint16 len)
{
	uint8	token;
	TICK_TYPE	tickStart;
	
	tickStart = TICK_FUNC;
	
	// wait for DataStart toke or timeout
	do
	{
		token = WriteReadSPI(j1microSD_Ctrl, 0xff);
	}
	while (!TIMEOUT(tickStart, `$TICKS_READ`) && (token == 0xff));
	
	if (token != 0xfe) return false;	// invalid or timeout
	
	// read the data into buffer
	while (len--) *bfrPtr++ = WriteReadSPI(j1microSD_Ctrl, 0xff);
	
	// discard CRC
	WriteReadSPI(j1microSD_Ctrl, 0xff);
	WriteReadSPI(j1microSD_Ctrl, 0xff);
	
	return true;
}

//---------------------------------------------------------------------
//	SlowSPI() - set the clock to low speed for init
//---------------------------------------------------------------------

static void SlowSPI(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl)
{
}

//---------------------------------------------------------------------
//	FastSPI() - set the clock to high speed for operation
//---------------------------------------------------------------------

static void FastSPI(`$INSTANCE_NAME`_CTRL *j1microSD_Ctrl)
{
}
